#!/usr/bin/python

## clientDialog.py

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import gtk.glade
import gettext

from clusterpkg.cluster_functions     import OptionError
from clusterpkg.cluster_functions_gui import errorMessage

##
## I18N
##

class clientDialog:
    def __init__(self, glade_xml, client):
        self.glade_xml = glade_xml
        self.client = client

        self.dialog = glade_xml.get_widget('clientDialog')

        self.refresh = gtk.TRUE

    def on_cancelbutton_clicked(self, button):
        self.refresh = gtk.TRUE

    def on_okbutton_clicked(self, button):

        ### "get" the new values from the Gui widgets ###

        name = self.glade_xml.get_widget('client_name_entry').get_text()
        options = self.glade_xml.get_widget('client_options_entry').get_text()

        ### If user didn't update any fields, then just return.
        if name    == self.name and \
           options == self.options:
              self.dialog.response(gtk.RESPONSE_OK)
              return

        validate_options = { "name"    : name,
                             "options" : options }

        self.refresh = gtk.TRUE
        rc = gtk.RESPONSE_OK
        try:
            ### Invoke client(ClusterEntity) "check" function to validate
            ### option.
            self.client.validate(validate_options)
        except OptionError, e:
            if e.getSeverity() == 'FATAL':
                errorMessage(e.getMessage())
                self.refresh = gtk.FALSE # Don't refresh - leave bad values
                                         # on screen.
                rc = gtk.RESPONSE_REJECT

            elif e.getSeverity() == 'WARNING':
                msg = _("%s\n\nPress 'OK' to Continue, or  'Cancel' to Exit.") % e.getMessage()
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                   self.refresh = gtk.FALSE # Don't refresh - leave bad values
                                            # on screen.
                   rc = gtk.RESPONSE_REJECT # Need to differentiate this
                                            # 'Cancel' from the main Dialog
                                            # 'Cancel', so make this rc
                                            # gtk.RESPONSE_REJECT.

        if rc == gtk.RESPONSE_OK:
            ### Validation passed on all fields - now "set" each by calling
            ### the ClusterEntity "set".
            self.client.set("name", name)
            self.client.set("options", options)

            self.client.change_action_to_modify()

        self.dialog.response(rc)
        return

    def run(self, client, action):
        self.client = client
        self.action = action

        ### Now connect the buttons to the functions ###
        self.ok = self.glade_xml.get_widget('clientDialog_okbutton').connect("clicked", self.on_okbutton_clicked)
        self.cancel = self.glade_xml.get_widget('clientDialog_cancelbutton').connect("clicked", self.on_cancelbutton_clicked)

        ### Load screen widgets with data from XML DOM for selected client ###

        if self.refresh:
            self.name = client.get("name")
            self.glade_xml.get_widget('client_name_entry').set_text(self.name)

            self.options = client.get("options")
            self.glade_xml.get_widget('client_options_entry').set_text(self.options)

        rc = self.dialog.run()
        self.glade_xml.get_widget('clientDialog_okbutton').disconnect(self.ok)
        self.glade_xml.get_widget('clientDialog_cancelbutton').disconnect(self.cancel)
        return rc

    def hide(self):
        return self.dialog.hide()

class clientItem:
    def __init__(self, parent, client):
        self.parent    = parent
        self.glade_xml = parent.glade_xml
        self.treeview  = parent.treeview
        self.model     = parent.treeview.get_model()
        self.columns   = parent.columns
        self.client    = client

        self.dialog = clientDialog(self.glade_xml, client)

    def GUI_load_row_columns(self, client_item, iter):
        row_display_string = _("%s  <client>") % client_item.client.get("name")
        self.model.set(iter,
                       self.columns['COLUMN_KEY'], row_display_string,
                       self.columns['COLUMN_DATA'], client_item)

    def GUI_tree_load_client(self, iter):
        client_iter = self.model.append(iter)
        self.GUI_load_row_columns(self, client_iter)

    def get_type(self):
        return "client"

    def get_value_of_key_field(self):
        return self.get("name") # Client key field is "name"

    def get(self, option):
        return self.client.get(option)

    def __modify(self, iter, action="modify"):
        if action == "modify":
            client = self.client
            client_item = self
        else:
            ### Must be an "add" ###
            nfsexport_item = self.parent
            nfsexport_entity = nfsexport_item.nfsexport

            client = nfsexport_entity.add_client()
            client_item = clientItem(self.parent, client)

        while 1:
            rc = self.dialog.run(client, action)
            if rc == gtk.RESPONSE_OK:
                if action == "add":
                    ### model.append needs the "iter" of the *parent* of the
                    ### thing that is being appended.
                    #parent_iter = self.model.iter_parent(iter)
                    parent_iter = iter
                    iter = self.model.append(parent_iter)

                self.GUI_load_row_columns(client_item, iter)

            elif rc == gtk.RESPONSE_CANCEL:
                if action == "add":
                    nfsexport_entity.delete_client_by_id(client.get("id"))
            else:
                ### rc was gtk.RESPONSE_REJECT:
                ###   1. User got a FATAL error and clicked 'OK' (only option)
                ###      so display same dialog again,
                ###        OR
                ###   2. User got a WARNING and clicked 'Cancel' to go back
                ###      and re-enter.
                continue
            break

        self.dialog.hide()

    def add(self, iter):
        self.__modify(iter, "add")

    def properties(self, iter):
        self.__modify(iter)

    def delete(self):
        nfsexport_item = self.parent
        nfsexport_entity = nfsexport_item.nfsexport
        nfsexport_entity.delete_client_by_id(self.client.get("id"))
