#!/usr/bin/python
from cluster_functions import OptionError
from xml.dom import minidom
import copy ### for deepcopy
import getopt

TRUE = (1==1)
FALSE = not TRUE

class ClusterEntity:
   def __init__(self, my_info, node, parent=None, action="modify"):
      self.my_info = my_info
      self.node    = node
      self.parent  = parent

      self.whoami          = my_info['WHOAMI']
      self.myoptions       = my_info['OPTIONS']
      self.add_getopts     = my_info['ADD_GETOPTS']
      self.delete_getopts  = my_info['DELETE_GETOPTS']
      self.update_getopts  = my_info['UPDATE_GETOPTS']
      self.listall_getopts = my_info['LISTALL_GETOPTS']

      self.action = action # Default is "modify" - when parent does
                           # add_child, add_child will change this
                           # setting for the child to "add" after the
                           # entity is created.

      ### node will only be "None" if instantiating an object just
      ### to get at the objects MY_INFO - this is for convenience,
      ### and is probably only done in the abc_module.py's where the
      ### abc_module needs to get its children's MY_INFO.

      ### Only give attributes "default" values if there are *NO* attributes
      ### yet.  That is, give this node default values if this node is *new*.
      if node != None and node.attributes.length == 0:
         for option in self.myoptions.keys():
            if self.myoptions[option].has_key("default"):
               value = self.myoptions[option]["default"]
               self.set(option, value)
          
      #####################################################################
      # options is a dictionary with 2 levels:
      #   1st level: option name(like "loglevel")
      #   2nd level: a dictionary containing
      #      'datatype' with value either "str"(string) or "int"(integer)
      #      'order'    with a number representing the sequential order
      #                 that options should be printed out.
      #####################################################################

   def do_getopt(self, cmdline_args, myoptions, mygetopts):
      try:
         opts, rest_opts = getopt.getopt(cmdline_args, "", mygetopts)
      except getopt.GetoptError, e:
         raise OptionError("FATAL", e.msg, e.opt, None)

      if len(rest_opts) != 0:
         raise OptionError("FATAL",
                           _("Illegal extra options were submitted."),
                           None,
                           None)

      ### From attributes in myoptions, construct a dict with those same
      ### attributes, but with a "--" prepended, so that each one is a
      ### valid getopt argument.
      valid_getopt_options = []
      for option_name in myoptions.keys():
         dashes_attr = "--" + option_name
         valid_getopt_options.append(dashes_attr)
         
      options_submitted = {}
      for opt, val in opts:
         if opt in valid_getopt_options:
            opt_no_dashes = opt[2:]
            options_submitted[opt_no_dashes] = val

      ### Dictionary "options_submitted" now contains all the submitted options.
      return options_submitted

   ### setOptions only called by "modify"(bottom of this class), and
   ### "modify" is only called by redhat-config-cluster-cmd command line tool.
   def setOptions(self, submitted_options):
      ### 'validate' will raise an exception if errors found ###
      try:
         self.validate(submitted_options)
      except OptionError, e:
        if e.getSeverity() == "WARNING":
            pass
        else: 
            ### error must be FATAL ###
            raise

      ctr=0
      for option, val in submitted_options.items():
         ctr=ctr+1
         self.set(option, val)
      return ctr

   def genericOptionChecks(self, options):
      for option, value in options.items():
         if option in self.myoptions.keys():
            if self.myoptions[option]['datatype'] == "yes_no":
               if not value in ("yes", "no"):
                  raise OptionError("FATAL",
                                    _("Option \"%s\" value must be \"yes\" or \"no\"") % option,
                                    option,
                                    value)
            elif self.myoptions[option]['datatype'] == "int":
               try:
                  y=int(value) ### will cause exception if val is a string ###
               except:
                  raise OptionError("FATAL",
                                    _("Bad value \"%s\" for option \"%s\"") % (value, option),
                                    option,
                                    value)
         else:
            raise OptionError("FATAL", _("%s bad option") % option, option, value)

   def set(self, option, val):
      self.node.setAttribute(option, str(val)) # str - int to str
      self.action = "modify"

   def get(self, option):
      if option in self.myoptions.keys():
         value = self.node.getAttribute(option)
         if self.myoptions[option]['datatype'] == "int" and \
            value != "": # value == "" when getAttribute didn't find the option
               return int(value)
         else:
            return value
      else:
         raise OptionError("FATAL", _("%s bad option") % option, option, None)
      
   def out(self):
      ### if element has attributes, print those attributes out in order ###
      if len(self.myoptions) > 0:
         option_by_order = {}
         for option_name, option_dict in self.myoptions.items():
            ### If option is not in xml(dom), then continue to next option.
            ### "id"s should not be exposed to users, so don't print id's.
            if (not self.node.hasAttribute(option_name)) or option_name == "id":
               continue

            order = self.myoptions[option_name]['order']
            option_by_order[order] = option_name

         ordered_options = option_by_order.keys()
         ordered_options.sort()
         print "\n%s:" % self.whoami
         for seq in ordered_options:
            option_name = option_by_order[seq]
            option_value = self.node.getAttribute(option_name)
            if option_value == "":
                option_value = "<null>"
            print "  %s = %s" % (option_name, option_value)

   def add_options(self, args):
      if args == None:
         ### args will be 'None' coming from GUI add
         return 0
      submitted_options = self.do_getopt(args, self.myoptions, self.add_getopts)
      return self.setOptions(submitted_options)

   def modify_options(self, args):
      submitted_options=self.do_getopt(args, self.myoptions,self.update_getopts)
      if len(submitted_options) > 0:
         ### only call setOptions if there are options to set ###
         ct_options_changed = self.setOptions(submitted_options)
      else:
         ct_options_changed = 0
      return ct_options_changed

   def delete_option(self, option):
      self.node.removeAttribute(option)

   def change_action_to_modify(self):
      self.action = "modify"
