#!/usr/bin/python
from clusterpkg import cluster_functions
from clusterpkg.cluster_functions import OptionError
from xml.dom import minidom
from ClusterEntity import *
import getopt
import string

TRUE = (1==1)
FALSE = not TRUE

class ClusterList(ClusterEntity):
   def __init__(self, my_info, node, parent=None, action="modify"):
      ClusterEntity.__init__(self, my_info, node, parent, action)

   def get_list(self, child_info):
      nodeList = []
      elements = self.node.getElementsByTagName(child_info["WHOAMI"])
      for e in elements:
         nodeList.append(ClusterEntity(child_info, e, self.parent))
      return nodeList
      
   def get_child(self, child_info, search_options, child_being_validated=None):
      num_search_options = len(search_options)
      elements = self.node.getElementsByTagName(child_info["WHOAMI"])
      for e in elements:
         child_entity = ClusterEntity(child_info, e, self.parent)
         if child_being_validated != None and \
            child_being_validated.get("id") == child_entity.get("id"):
               ### child_being_validated was passed in - user wants to
               ### find a child whose options match all the search_options
               ### passed in, and where the child found is
               ### *NOT* child_being_validated.
               continue

         ct_options_found = 0
         for option, value in search_options.items():
            if child_entity.get(option) == value:
               ct_options_found = ct_options_found + 1

         if ct_options_found == num_search_options:
            return child_entity

      ### Match not found - raise error ###
      err_fld_list = []
      for option, value in search_options.items():
         err_fld_list.append("%s=%s" % (option, value))
      err_flds = string.join(err_fld_list, ", ")
      raise OptionError("FATAL",
                        _("%s with %s does not exist") % (string.capitalize(child_info["WHOAMI"]), err_flds),
                        "err_flds",
                        err_flds)

   def child_found(self, child_info, search_options, child_being_validated=None):
      try:
         self.get_child(child_info, search_options, child_being_validated)
      except OptionError, e:
         return FALSE

      return TRUE

   #def delete_child(self, child_info, args, val, key="name"):
   def delete_child(self, child_info, search_options, args):
      ### args will come in as "None" from Gui, but will be a dictionary
      ### if from cluconfig command line.
      if args != None:
         self.do_getopt(args, child_info["OPTIONS"], child_info["DELETE_GETOPTS"])
      child_entity = self.get_child(child_info, search_options)
      cluster_functions.remove_dom_child(self, child_entity.node)

   def delete_by_id(self, child_info, id):
      search_options = {"id" : id}
      child_entity = self.get_child(child_info, search_options)
      cluster_functions.remove_dom_child(self, child_entity.node)

   #########################################################################
   ### add: 1. calls do_getopt (process command line options)
   ###      2. calls add_child_entity to create a new child ClusterEntity
   ###      3. for that new child ClusterEntity, call setOptions
   ###           to set its options.
   #########################################################################
   def add_child(self, child_info):
      ####################################################
      ### 1. Build list of "id"s for existing children ###
      ####################################################
      self.ids=[]
      elements = self.node.getElementsByTagName(child_info["WHOAMI"])
      for e in elements:
         child_entity = ClusterEntity(child_info, e, parent=self)
         try:
            self.ids.append(child_entity.get("id"))
         except ValueError:
            pass
         except OptionError:
            pass

      ########################################################
      ### 2. Find the 1st open "id" slot for the new child ###
      ########################################################
      for x in range(cluster_functions.MAX_ID):
         if x not in self.ids:
            break

      if x == cluster_functions.MAX_ID:
         raise OptionError("FATAL",
                           _("ERROR: %ss add_child_entity - couldn't find slot for new id.") % child_info["WHOAMI"],
                           None,
                           None)

      ####################################
      ### 3. Create the DOM child node ###
      ####################################
      new_node = \
         cluster_functions.append_dom_child(self, child_info["WHOAMI"])

      ##########################################################
      ### 4. Create new ClusterEntity from that DOM child node,
      ###    and set the "id" to the open slot that was found.
      ##########################################################
      new_entity = ClusterEntity(child_info, new_node, self, "add")
      try:
         new_entity.set("id", x)
      except OptionError:
         pass

      return new_entity

