#!/usr/bin/python
from ClusterEntity import *
import socket
#from xml.dom import minidom

MINIMUM_TKO_COUNT = 3

TRUE = 1
FALSE = 0

### 9/8/2003 1. changed loglevel default from 4(Warning)
###               to 5(Notice), and
###          2. changed failover time
###               from 10 seconds (interval 500000, tko_count 20)
###               to   15 seconds (interval 750000, tko_count 20)
MY_INFO = {
   "WHOAMI"  : "clumembd",

   "OPTIONS" : {
      "loglevel" : { "datatype"  : "int",
                     "order"     : 0,
                     "default"   : 5 },

      "interval"  : { "datatype" : "int",
                      "order"    : 1,
                      "default"  : 750000 },

      "tko_count" : { "datatype" : "int",
                      "order"    : 2,
                      "default"  : 20 },

### commented out 3/20/2003 HM ###
#      "strict"    : { "datatype" : "yes_no",
#                      "order"    : 3,
#                      "default"  : "no" },
#
#      "adaptive"  : { "datatype" : "yes_no",
#                      "order"    : 4,
#                      "default"  : "yes" },

      "thread"    : { "datatype" : "yes_no",
                      "order"    : 5,
                      "default"  : "yes" },

      "broadcast" : { "datatype" : "yes_no",
                      "order"    : 6,
                      "default"  : "no" },

      "multicast" : { "datatype" : "yes_no",
                      "order"    : 7,
                      "default"  : "yes" },

      "multicast_ipaddress" : { "datatype" : "str",
                                "order"    : 8,
                                "default"  : "225.0.0.11" },
   },

   "ADD_GETOPTS"     : [],

   "UPDATE_GETOPTS"  : [ "clumembd",
                            "loglevel=",
                            "interval=",
                            "tko_count=",
                            #"strict=",
                            #"adaptive=",
                            "thread=",
                            "broadcast=",
                            "multicast=",
                            "multicast_ipaddress=" ],

   "DELETE_GETOPTS"  : [],
   "LISTALL_GETOPTS" : [],
}

def failover_seconds_greater_than_min_disk_ping_seconds(interval, tko_count, cluquorumd_pinginterval):
   if cluquorumd_pinginterval == "":
      ### This whole failover seconds > min disk ping seconds check
      ### isn't necessary if 'cluquorumd' pinginterval isn't specified.
      ### So just return success.
      rc = 0 # success
      error_msg = ""
      new_disk_ping_seconds = 0
      return (rc, error_msg, new_disk_ping_seconds)
   cluquorumd_pinginterval = int(cluquorumd_pinginterval)

   if interval != "":
      interval = int(interval)
   else:
      interval = int(self.get("interval"))
   if tko_count != "":
      tko_count = int(tko_count)
   else:
      tko_count = int(self.get("tko_count"))

   ### 1.0 - force 'float' context so that we don't loose
   ### fractional parts in multiplication.
   failover_seconds = (tko_count * interval * 1.0) / 1000000
   failover_seconds_rounded = round(failover_seconds)
   failover_seconds_rounded_int = int(failover_seconds_rounded)
   failover_seconds = failover_seconds_rounded_int

   minimum_disk_ping_seconds = cluquorumd_pinginterval * MINIMUM_TKO_COUNT

   rc = 0
   error_msg = ""
   new_disk_ping_seconds = 0
   if failover_seconds > minimum_disk_ping_seconds:
      ### edit passed ###
      return (rc, error_msg, new_disk_ping_seconds)

   ### edit failed ###
   rc = 1
   error_msg = _("Failover seconds [%d] must be greater than the 'cluquorumd' Ping Interval minimum (Ping Interval [%d] * minimum tko count [%d] = %d)") % (failover_seconds, cluquorumd_pinginterval, MINIMUM_TKO_COUNT, minimum_disk_ping_seconds)
   (quotient, remainder) = divmod(failover_seconds, MINIMUM_TKO_COUNT)
   if remainder == 0 and quotient > 1:
      quotient = quotient - 1
   new_disk_ping_seconds = quotient
   return (rc, error_msg, new_disk_ping_seconds)


class clumembd(ClusterEntity):
   def __init__(self, node):
      ClusterEntity.__init__(self, MY_INFO, node)


   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):
      #print "----> clumembd_module validate NOW!<-----"
      #print "      self.action = %s" % self.action
      #print "      options     = %s" % options

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      if options.has_key("loglevel") and options["loglevel"] != "":
         loglevel = options["loglevel"]
         error = FALSE
         try:
            int_loglevel = int(loglevel)
         except:
            error = TRUE
         if not error and \
            (int_loglevel < 0 or int_loglevel > 7):
               error = TRUE
         if error:
            raise OptionError("FATAL",
                              _("""Log Level must be a number from 0 to 7:
  0 - EMERG
  1 - ALERT
  2 - CRIT
  3 - ERR
  4 - WARNING (DEFAULT)
  5 - NOTICE
  6 - INFO
  7 - DEBUG"""),
                              "loglevel",
                              loglevel)

      multicast = ""
      if options.has_key("multicast") and options["multicast"] != "":
         multicast = options["multicast"]

      multicast_ipaddress = ""
      if options.has_key("multicast_ipaddress") and options["multicast_ipaddress"] != "":
         multicast_ipaddress = options["multicast_ipaddress"]
         try:
            num = socket.inet_aton(multicast_ipaddress)
         except socket.error, e:
            raise OptionError("FATAL",
                              _("Illegal Multicast IP Address"),
                              "multicast_ipaddress",
                              multicast_ipaddress)

      if multicast == "yes" and multicast_ipaddress == "":
            raise OptionError("FATAL",
                              _("When Multicast is selected, Multicast IP Address is required"),
                              "multicast_ipaddress",
                              multicast_ipaddress)

      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################
