#!/usr/bin/python
from ClusterEntity import *
from clusterpkg.cluster_functions import cluPing, countMembers, hostInDNS
import re

### 9/8/2003 1. changed loglevel default from 4(Warning)
###             to 5(Notice)
MY_INFO = {
   "WHOAMI"  : "cluquorumd",

   "OPTIONS" : {
      "loglevel"      : { "datatype" : "int",
                          "order"    : 0,
                          "default"  : 5 },
      "pinginterval"  : { "datatype" : "str",
                          "order"    : 1,
                          "default"  : "2" },
      "tiebreaker_ip" : { "datatype" : "str",
                          "order"    : 2 },
   },

   "ADD_GETOPTS"     : [],

   "UPDATE_GETOPTS"  : [ "cluquorumd",
                            "loglevel=",
                            "pinginterval=",
                            "tiebreaker_ip=" ],

   "DELETE_GETOPTS"  : [],
   "LISTALL_GETOPTS" : [],
}

class cluquorumd(ClusterEntity):
   def __init__(self, node):
      ClusterEntity.__init__(self, MY_INFO, node)

   def __ipInCluster(self, test_ip):
      ### First, if test_ip is a DNS name, convert it to dotted quad ###
      if re.search("^\s*[A-Za-z]+", test_ip):
         test_ip = hostInDNS(test_ip)
         if test_ip == None:
            return ""

      ### Second, run through all the members and see if the member
      ### name matches the test_ip.
      ownerDocument = self.node.ownerDocument
      members_node = ownerDocument.getElementsByTagName("members")[0]
      from clusterpkg.members_module import members
      members_entity = members(members_node)
      members_list = members_entity.get_list()
      for m in members_list:
         member_name = m.get("name")
         member_ip = member_name
         if re.search("^\s*[A-Za-z]+", member_ip):
            member_ip = hostInDNS(member_ip)
            if member_ip == None:
               return ""
         if member_ip == test_ip:
            return _("Tiebreaker IP Address already exists in cluster as member '%s'") % member_name

      ### Next, run through all the service ip addresses and see
      ### if any of those match the test_ip.
      services_node = ownerDocument.getElementsByTagName("services")[0]
      from clusterpkg.services_module import services
      services_entity = services(services_node)
      for s in services_entity.get_list():
         service_ipaddresses = s.get_service_ipaddresses()
         for service_ipaddress_entity in service_ipaddresses:
            service_ipaddress = service_ipaddress_entity.get("ipaddress")
            service_ipaddress_before = service_ipaddress
            if re.search("^\s*[A-Za-z]+", service_ipaddress):
               service_ipaddress = hostInDNS(service_ipaddress)
               if service_ipaddress == None:
                  return ""
            if service_ipaddress == test_ip:
               return _("Tiebreaker IP Address already exists in cluster as service ip address '%s'") % service_ipaddress_before

      ### Next, make sure test_ip is NOT localhost ###
      ### Split dotted quad on the period.         ###
      ip_tokens = test_ip.split('.')
      if ip_tokens[0] == "127":
         return _("Tiebreaker IP Address cannot start with '127' (localhost)")

      ### test_ip didn't match any member names nor any service ips ###
      return ""

   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):
      #print "----> cluquroumd validate NOW!<-----"
      #print "      self.action = %s" % self.action
      #print "      options     = %s" % options

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      two_node_cluster_and_no_tiebreaker_ip = FALSE

      if options.has_key("tiebreaker_ip") and options["tiebreaker_ip"] != "":
         tiebreaker_ip = options["tiebreaker_ip"]
         #print "cluquorumd tiebreaker_ip = [%s]" % tiebreaker_ip
         if not cluPing(tiebreaker_ip):
            raise OptionError("FATAL",
                              _("Tiebreaker IP Address not found on network"),
                              "tiebreaker_ip",
                              tiebreaker_ip)

         errmsg = self.__ipInCluster(tiebreaker_ip)
         if errmsg != "":
            raise OptionError("FATAL",
                              errmsg,
                              "tiebreaker_ip",
                              tiebreaker_ip)

      else:
         ### Tiebreaker IP Address *NOT* submitted ###
         count_members = countMembers(self.node)

         if count_members == 2:
            ### set warning flag, and raise WARNING below ###
            two_node_cluster_and_no_tiebreaker_ip = TRUE

         elif count_members == 4:
            raise OptionError("FATAL",
                              _("Tiebreaker IP Address is required for 4-node clusters"),
                              "tiebreaker_ip",
                              None)

      if options.has_key("loglevel") and options["loglevel"] != "":
         loglevel = options["loglevel"]
         error = FALSE
         try:
            int_loglevel = int(loglevel)
         except:
            error = TRUE
         if not error and \
            (int_loglevel < 0 or int_loglevel > 7):
               error = TRUE
         if error:
            raise OptionError("FATAL",
                              _("""Log Level must be a number from 0 to 7:
  0 - EMERG
  1 - ALERT
  2 - CRIT
  3 - ERR
  4 - WARNING (DEFAULT)
  5 - NOTICE
  6 - INFO
  7 - DEBUG"""),
                              "loglevel",
                              loglevel)

      if options.has_key("pinginterval") and options["pinginterval"] != "":
         pinginterval = options["pinginterval"]
         error = FALSE
         try:
            int_pinginterval = int(pinginterval)
         except:
            error = TRUE

         if not error and \
            (int_pinginterval < 1 or int_pinginterval > 10):
               error = TRUE

         if error:
            raise OptionError("FATAL",
                              _("Ping Interval must be a number from 1 to 10"),
                              "pinginterval",
                              pinginterval)

      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################
      if two_node_cluster_and_no_tiebreaker_ip:
         raise OptionError("WARNING",
                           _("Tiebreaker IP Address is recommended for 2-node clusters"),
                           "tiebreaker_ip",
                           None)
