## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gettext
import os
import re
import socket
import string
from xml.dom import minidom

TRUE = (1==1)
FALSE = not TRUE

CONFIG_FILE = "/etc/cluster.xml"

def countMembers(xml_node):
    ### Figure out how many members have been defined ###
    ownerDocument = xml_node.ownerDocument
    members_node = ownerDocument.getElementsByTagName("members")[0]
    from clusterpkg.members_module import members
    members_entity = members(members_node)
    members_list = members_entity.get_list()
    count_members = len(members_list)
    return count_members

def hostInDNS(ip_or_name):
    try:
        ip = socket.gethostbyname(ip_or_name)
    except socket.gaierror, e:
        #errno, errstr = e
        return None # host is *NOT* in DNS
    return ip # host *IS* in DNS

def getStatusOutput(cmd):
    """Return (status, output) of executing cmd in a shell."""
    import os
    text=""
    pipe = os.popen('{ ' + cmd + '; } 2>&1', 'r')
    try:
        text = pipe.read()
    except IOError,v:
        text = pipe.read()

    try:
      sts = pipe.close()
    except IOError:
      sts = None

    if sts is None: sts = 0
    if text[-1:] == '\n': text = text[:-1]
    return sts, text

def cluPing(ip):
    #cmd = 'ping -c 1 -w 1 %s 2>/dev/null' % ip
    #rc = os.system(cmd)
    #if rc == 0:
    #    return TRUE # ping successful
    #else:
    #    return FALSE
    ping_cmd = 'ping -c 1 -w 1 %s' % ip
    (status, output) = getStatusOutput(ping_cmd)
    if status == 0:
        return TRUE # ping successful
    else:
        return FALSE

class cluStat:
    def __init__(self):
        (status, XML) = getStatusOutput("/usr/sbin/clustat -x")
        dom = minidom.parseString(XML)
        self.__XML = XML
        self.__dom = dom

        cluquorumdElement=dom.getElementsByTagName("cluquorumd")[0]
        quorum_view_attribute = cluquorumdElement.getAttribute("quorum_view")
        quorum_view = int(quorum_view_attribute)
        if quorum_view == -1:
            self.__cluster_is_running = FALSE ### Cluster is *NOT* running ###
        else:
            self.__cluster_is_running = TRUE  ### Cluster *IS* running ###

        servicesElement=dom.getElementsByTagName("services")[0]
        service_elements = servicesElement.getElementsByTagName("service")
        self.service_state = {}
        for service_element in service_elements:
            service_id    = int(service_element.getAttribute("id"))
            service_state = service_element.getAttribute("state")
            self.service_state[service_id] = service_state

    def getXML(self):
        return self.__XML
    def getDOM(self):
        return self.__dom
    def clusterIsRunning(self):
        return self.__cluster_is_running
    def serviceIsRunning(self, id):
        #stateTable: 0=Uninitialized
        #            1=Running
        #            2=Stopped
        #            3=Disabled
        #            4=Pending
        #            5=Failed
        #            6=Unknown
        try:
            this_service_state = self.service_state[id]
        except KeyError, e:
            ### id doesn't exist yet - may have been added by gui and
            ### config not saved yet.
            return FALSE
        if this_service_state == "1":
            return TRUE
        return FALSE

class OptionError:
    def __init__(self, severity, message, option_name=None, option_value=None):
        self.__severity     = severity
        self.__msg          = message
        self.__option_name  = option_name
        self.__option_value = option_value
    def getSeverity(self):
        return self.__severity
    def getMessage(self):
        return self.__msg
    def getOptionName(self):
        return self.__option_name
    def getOptionValue(self):
        return self.__option_value

PROGNAME='cluster-config'
AUTHORIZATIONDIR='/etc/pam.d/'
MAX_ID=1000

gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
_ = gettext.gettext

def initialize_shared_state(clu):
    (status, output) = getStatusOutput("shutil -i")
    if status != 0:
        shared_state_entity = clu.getsharedstate()
        shared_state_type = shared_state_entity.get("type")
        if shared_state_type == 'raw' and string.find(output, 'initSharedFD'):
            msg = _("""Raw devices have NOT been created yet.

Follow the instructions in the manual
for 'Creating Raw Devices', and then
try again.""")
            raise OptionError('FATAL', msg, '', '')

        msg = _("Shared Storage could *NOT* be initialized!\n\n%s") % output
        raise OptionError('FATAL', msg, '', '')

    cmd = "shutil -s " + CONFIG_FILE
    (status, output) = getStatusOutput(cmd)
    if status != 0:
        msg = _("Error: Shared Storage could *NOT* be initialized!\n\n%s") % output
        raise OptionError('FATAL', msg, '', '')

def append_dom_child(self, element_name):
    spaces_over_to_tag = {
        "cluconfig"           : "",
        "sharedstate"         : "  ",
        "clumembd"            : "  ",
        "clupowerd"           : "  ",
        "cluquorumd"          : "  ",
        "clurmtabd"           : "  ",
        "clusvcmgrd"          : "  ",
        "clulockd"            : "  ",
        "cluster"             : "  ",

        "members"             : "  ",
        "member"              : "    ",
        "powercontroller"     : "      ",

        "services"            : "  ",
        "service"             : "    ",
        "device"              : "      ",
        "mount"               : "        ",
        "nfsexport"           : "        ",
        "client"              : "          ",

        "service_ipaddresses" : "      ",
        "service_ipaddress"   : "        ",

        "failoverdomains"     : "  ",
        "failoverdomain"      : "    ",
        "failoverdomainnode"  : "      ",
    }

    parent = {
        "sharedstate" : "cluconfig",
        "clumembd" : "cluconfig",
        "clupowerd" : "cluconfig",
        "cluquorumd" : "cluconfig",
        "clurmtabd" : "cluconfig",
        "clusvcmgrd" : "cluconfig",
        "clulockd" : "cluconfig",
        "cluster" : "cluconfig",
        "members" : "cluconfig",
        "services" : "cluconfig",
        "failoverdomains" : "cluconfig",

        "member" : "members",
        "powercontroller": "member",
        "service": "services",
        "device": "service",
        "service_ipaddresses": "service",
        "service_ipaddress": "service_ipaddresses",
        "mount": "device",
        "nfsexport": "device",
        "client": "nfsexport",
        "failoverdomain": "failoverdomains",
        "failoverdomainnode": "failoverdomain",
    }


    ##########################
    ### Append new element ###
    ##########################

    ### First, create Text nodes(for spacing) and Element before
    ### actually inserting them into the DOM.
    text_node_newline_before_new_element = \
        self.node.ownerDocument.createTextNode("\n")
    mystring = "self.node.ownerDocument.createTextNode(\"" + spaces_over_to_tag[element_name] + "\")"
    text_node_spaces_before_new_element = eval(mystring)
    mystring = "self.node.ownerDocument.createElement(\"" + element_name + "\")"
    temp_element = eval(mystring)
    text_node_newline_after_new_element = \
        self.node.ownerDocument.createTextNode("\n")
    parent_name = parent[element_name]
    mystring = "self.node.ownerDocument.createTextNode(\"" + spaces_over_to_tag[parent_name] + "\")"
    text_node_spaces_before_end_parent = eval(mystring)

    ### Here is the sequence for creating a new "channel"
    # 1. appendChild new channel
    # 2. if not first child:
    #        removeChild text spaces prev to new channel
    #    else:
    #        insertBefore - insert newline before new element just added
    # 3. insertBefore new channel - text spaces over to new channel
    # 4. appendChild text newline (after new channel element)
    # 5. appendChild text spaces over to end channels tag

    # When do appendChild, it appends the new child right before the
    # close tag of the parent who is appending the child, which is
    # right *after* the text node containing spaces from the beginning
    # of the line over to the parent close tag.  So, after appendChild
    # with the actual element we want, then removeChild the previousSibling
    # which is that text node containing spaces over to the parent close
    # tag.  *Then*, proceed to
    #
    #   a. insert a text node containing spaces over to the new element, then
    #   b. append a text node containing a newline to finish the new
    #        element line, then
    #   c. append a text node containing spaces over to the close tag
    #        of the parent
    new_element = self.node.appendChild(temp_element)
    text_node_prev = new_element.previousSibling
    if text_node_prev == None:
        ### The new element is the *FIRST* child ###
        self.node.insertBefore(text_node_newline_before_new_element, new_element)
    else:
        ### The new element is *NOT* the first child - remove the
        ### text node previous to the new element(the text node previous
        ### to the new element contains spaces from the beginning of the
        ### line over to the end tag of the new element's parent)
        self.node.removeChild(text_node_prev)

    self.node.insertBefore(text_node_spaces_before_new_element, new_element)
    self.node.appendChild(text_node_newline_after_new_element)
    self.node.appendChild(text_node_spaces_before_end_parent)

    return new_element

#####################################################################
# remove_dom_child: assumes that the child node is surrounded by
#                   a text node before(spaces), and a text node
#                   after(newline).  We want to remove the child
#                   node, and those surrounding text nodes.
#####################################################################
def remove_dom_child(parent, child_node_to_delete):
    #print "cluster_functions remove_dom_child TOP!"
    text_node_spaces_before = child_node_to_delete.previousSibling
    text_node_newline_after = child_node_to_delete.nextSibling

    parent.node.removeChild(text_node_spaces_before)
    parent.node.removeChild(child_node_to_delete)
    parent.node.removeChild(text_node_newline_after)
