#!/usr/bin/python
from xml.dom import minidom
from clusterpkg.cluster_functions import OptionError
from ClusterEntity import *
import os, stat

TRUE = (1==1)
FALSE = not TRUE

MY_INFO = {
   "WHOAMI"  : "mount",

   "OPTIONS" : {
      "id"            : { "datatype" : "int",
                          "order"    : 0 },
      "mountpoint"    : { "datatype" : "str",
                          "order"    : 1 },
      "fstype"        : { "datatype" : "str",
                          "order"    : 2 },
      "options"       : { "datatype" : "str",
                          "order"    : 3 },
      "forceunmount"  : { "datatype" : "yes_no",
                          "order"    : 4,
                          "default"  : "yes" },
   },

   "ADD_GETOPTS"     : [],

   "DELETE_GETOPTS"  : [],

   "LISTALL_GETOPTS" : [ "service=",
                            "device=",
                               "mount" ],

   "UPDATE_GETOPTS"  : [ "service=",
                            "device=",
                               "mount",
                                  "mountpoint=",
                                  "fstype=",
                                  "options=",
                                  "forceunmount=" ],
}

class mount(ClusterEntity):
   def __init__(self, node, parent, action="modify"):
      ClusterEntity.__init__(self, MY_INFO, node, parent, action)

   def __dup_found_in_cluster(self, option, value):
      this_device     = self.parent
      this_device_id  = this_device.get("id")

      this_service    = this_device.parent
      this_service_id = this_service.get("id")

      ownerDocument = self.node.ownerDocument
      services_node = ownerDocument.getElementsByTagName("services")[0]
      from clusterpkg.services_module import services
      services_entity = services(services_node)
      for service in services_entity.get_list():
         service_id = service.get("id")
         devices = service.get_devices()
         for device in devices:
            device_id = device.get("id")
            mount = device.get_mount()
            if mount.get(option) == value \
                  and \
              (device_id != this_device_id or \
               service_id != this_service_id):
                  return TRUE
      return FALSE

   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################

      ### Check for *required* fields on "add" ###
      ### Validate "name"
      if self.action == "add":
         if not options.has_key("mountpoint"):
            mountpoint = ""
         else:
            mountpoint = options["mountpoint"]
      else:
         ### This is a "modify".
         prev_mountpoint = self.get("mountpoint")
         if options.has_key("mountpoint"):
            mountpoint = options["mountpoint"]
         else:
            mountpoint = prev_mountpoint

      if mountpoint == "":
         raise OptionError("FATAL",
                           _("Device Mountpoint is required"),
                           "mountpoint",
                           None)

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      mountpoint_does_not_exist = FALSE
      if options.has_key("mountpoint") and options["mountpoint"] != "":
         mountpoint = options["mountpoint"]
         try:
            mode = os.stat(mountpoint)[stat.ST_MODE]
         except OSError, e:
            #msg = _("Mount Point does not exist")
            #raise OptionError("FATAL", msg, "mountpoint", mountpoint)
            mountpoint_does_not_exist = TRUE
      
         #if not stat.S_ISDIR(mode):
         #   msg = _("Mount Point must be a directory")
         #   raise OptionError("FATAL", msg, "mountpoint", mountpoint)

         if self.__dup_found_in_cluster("mountpoint", mountpoint):
            raise OptionError("FATAL",
                              _("Device Mountpoint already exists"),
                              "mountpoint",
                              mountpoint)

      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################
      if mountpoint_does_not_exist:
            raise OptionError("WARNING",
                              _("Device Mountpoint '%s' does not exist, but will be created when the service is started.") % mountpoint,
                              "mountpoint",
                              mountpoint)
