#!/usr/bin/python
from xml.dom import minidom
from clusterpkg.cluster_functions import OptionError, cluPing
from clusterpkg.failoverdomainnode_module import failoverdomainnode
import clusterpkg.failoverdomainnode_module
from ClusterList import *
import os
import stat
import string

FAILOVERDOMAIN_NODE_INFO = clusterpkg.failoverdomainnode_module.MY_INFO

TRUE = (1==1)
FALSE = not TRUE

MY_INFO = {
   "WHOAMI"  : "failoverdomain",

   "OPTIONS" : {
      "id"         : { "datatype" : "int",
                       "order"    : 0 },

      "name"       : { "datatype" : "str",
                       "order"    : 1 },

      "restricted" : { "datatype" : "yes_no",
                       "order"    : 2,
                       "default"  : "no" },

      "ordered"    : { "datatype" : "yes_no",
                       "order"    : 3,
                       "default"  : "no" },
   },

   "ADD_GETOPTS"     : [ "failoverdomain=",
                            "name=",
                            "restricted=",
                            "ordered=",
                         "add_failoverdomain" ],

   "DELETE_GETOPTS"  : [ "failoverdomain=",
                            "name=",
                         "del_failoverdomain" ],

   "LISTALL_GETOPTS" : [ "failoverdomains" ],

   "UPDATE_GETOPTS"  : [ "failoverdomain=",
                            "name=",
                            "restricted=",
                            "ordered=" ],
} ### end MY_INFO

class failoverdomain(ClusterList):
   def __init__(self, node, parent=None, action="modify"):
      ClusterList.__init__(self, MY_INFO, node, parent, action)

   def __dup_found(self, key, value):
      prev_value = self.get(key)

      if self.action == "add" or \
         (self.action == "modify" and value != prev_value):

          failoverdomains_entity = self.parent
          search_options = {key : value}
          failoverdomain_being_validated = self
          if failoverdomains_entity.failoverdomain_found(search_options, failoverdomain_being_validated):
              return TRUE
      return FALSE


   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################

      ### Check for *required* fields on "add" ###
      if self.action == "add":

         ### 1st, for any options that were not submitted, see if a
         ### "default" was defined above.  If there was, make the
         ### option value equal to the default.
         for option in MY_INFO["OPTIONS"]:
            if not options.has_key(option):
               ### This option was not submitted - assign the default if
               ### there is one.
               if MY_INFO["OPTIONS"][option].has_key("default"):
                  options[option] = MY_INFO["OPTIONS"][option]["default"]

         if not options.has_key("name") or options["name"] == "":
            raise OptionError("FATAL",
                              _("Domain Name is required"),
                              "name",
                              None)

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      ### Validate submitted options ###
      if options.has_key("name"):
         name = options["name"]
         ### 'name' is key - check for another failoverdomain with same name
         if self.__dup_found("name", name):
            raise OptionError("FATAL",
                              _("Domain Name already exists"),
                              "name",
                              name)

      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################
      # None

#############################################################################

   def get_failoverdomainnodes(self):
      nodeList = self.node.getElementsByTagName(FAILOVERDOMAIN_NODE_INFO["WHOAMI"])

      ### Sort the nodes by "id" to ensure that the failover domain
      ### nodes list that is returned is in the proper order.
      nodes = {}
      for n in nodeList:
         nodes[n.getAttribute("id")] = n
      sorted_nodes = nodes.items()
      sorted_nodes.sort()

      failoverdomainnode_entities = []
      for id, n in sorted_nodes:
         failoverdomainnode_entities.append(failoverdomainnode(n, self))
      return failoverdomainnode_entities

   def get_failoverdomainnode(self, name):
      search_options = {"name" : name}
      entity = ClusterList.get_child(self, FAILOVERDOMAIN_NODE_INFO, search_options)
      return failoverdomainnode(entity.node, self)

   def failoverdomainnode_found(self, search_options, failoverdomainnode_being_validated):
      return ClusterList.child_found(self, FAILOVERDOMAIN_NODE_INFO, search_options, failoverdomainnode_being_validated)

   def add_failoverdomainnode(self, cmdline_args=None):
      entity = ClusterList.add_child(self, FAILOVERDOMAIN_NODE_INFO)
      fod_node_entity = failoverdomainnode(entity.node, self, "add")
      fod_node_entity.add_options(cmdline_args)
      return fod_node_entity

   def delete_failoverdomainnode(self, name, args=None):
      ### args will come in as "None" from Gui, but will be a dictionary
      ### if from cluconfig command line.
      if args != None:
         self.do_getopt(args, FAILOVERDOMAIN_NODE_INFO["OPTIONS"], FAILOVERDOMAIN_NODE_INFO["DELETE_GETOPTS"])
      search_options = {"name": name}
      ClusterList.delete_child(self, FAILOVERDOMAIN_NODE_INFO, search_options, args)

   def delete_failoverdomainnode_by_id(self, id):
      ClusterList.delete_by_id(self, FAILOVERDOMAIN_NODE_INFO, id)

   def __change_failoverdomain_on_services(self, old_name, new_name):
      if old_name != new_name and \
         not old_name in ("", "None", None):
            ### First, get a list of services that refer to the
            ### *old* domain name.  To do that, we need to
            ###   1. get ownerDocument, and user that to
            ###   2. get services node, then
            ###   3. user services node to spin through all existing
            ###      services.
            ownerDocument = self.node.ownerDocument
            services_node = ownerDocument.getElementsByTagName("services")[0]
            from clusterpkg.services_module import services
            services_entity = services(services_node)
            for service in services_entity.get_list():
                if service.get("failoverdomain") == old_name:
                    service.set("failoverdomain", new_name)

   #######################################################
   # Override the ClusterEntity 'set' function so that
   # if a failover domain 'name' is being changed, we
   # can ensure that all services referring to the
   # "old" name get changed to refer to the "new" name.
   #######################################################
   def set(self, option, value):
      if option == "name":
         old_name = self.get("name")
         new_name = value
         if old_name in ("", "None", "none"):
            old_name = ""
         if new_name in ("", "None", "none"):
            new_name = ""
      ClusterEntity.set(self, option, value)
      if option == "name" and \
         new_name != old_name and \
         old_name != "":
            self.__change_failoverdomain_on_services(old_name, new_name)

   def out(self, print_domain_nodes_only=FALSE):
      if not print_domain_nodes_only:
         ClusterEntity.out(self)

      nodeList = self.get_failoverdomainnodes()
      nodes_dict = {}
      for n in nodeList:
         id = n.get("id")
         name = n.get("name")
         nodes_dict[id] = name

      sorted_nodes = nodes_dict.items()
      sorted_nodes.sort()
      print _("  failoverdomainnodes:")
      for id, node_name in sorted_nodes:
         print "    %s" % node_name
