#!/usr/bin/python
from xml.dom import minidom
from socket import gethostbyname
from clusterpkg.cluster_functions import OptionError, cluPing, hostInDNS
from clusterpkg.member_powercontroller_module import powercontroller
from clusterpkg.util_ipconvert_module         import ipConvert
import clusterpkg.member_powercontroller_module
from ClusterList import *
import os
import stat
import string

POWERCONTROLLER_INFO = clusterpkg.member_powercontroller_module.MY_INFO

TRUE = (1==1)
FALSE = not TRUE

MY_INFO = {
   "WHOAMI"  : "member",

   "OPTIONS" : {
      "id"       : { "datatype" : "int",
                     "order"    : 0 },

      "name"     : { "datatype" : "str",
                     "order"    : 1 },

      "watchdog" : { "datatype" : "yes_no",
                     "order"    : 2,
                     "default"  : "yes" },
   },

   "ADD_GETOPTS"     : [ "member=",
                            "name=",
                            "watchdog=",
                         "add_member" ],

   "DELETE_GETOPTS"  : [ "member=",
                            "name=",
                         "del_member" ],

   "LISTALL_GETOPTS" : [ "members" ],

   "UPDATE_GETOPTS"  : [ "member=",
                            "name=",
                            "watchdog=" ],
} ### end MY_INFO

class member(ClusterList):
   def __init__(self, node, parent=None, action="modify"):
      ClusterList.__init__(self, MY_INFO, node, parent, action)

      member_name = self.get("name")
      self.dns_ipaddress = hostInDNS(member_name)

   def __dup_dns_ip_found(self, dns_ipaddress):
      prev_dns_ipaddress = self.dns_ipaddress
      if self.action == "add" or \
         (self.action == "modify" and dns_ipaddress != prev_dns_ipaddress):
          members_entity = self.parent
          members_list = members_entity.get_list()
          for m in members_list:
             if m.dns_ipaddress == dns_ipaddress:
                return TRUE
      return FALSE

   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################

      ### Check for *required* fields on "add" ###
      if self.action == "add":

         ### 1st, for any options that were not submitted, see if a
         ### "default" was defined above.  If there was, make the
         ### option value equal to the default.
         for option in MY_INFO["OPTIONS"]:
            if not options.has_key(option):
               ### This option was not submitted - assign the default if
               ### there is one.
               if MY_INFO["OPTIONS"][option].has_key("default"):
                  options[option] = MY_INFO["OPTIONS"][option]["default"]

         if not options.has_key("name") or options["name"] == "":
            raise OptionError("FATAL",
                              _("Member Name is required"),
                              "name",
                              None)

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      ### Validate submitted options ###
      if options.has_key("name"):
         name = options["name"]
         dns_ipaddress = hostInDNS(name)
         if dns_ipaddress == None:
            raise OptionError("FATAL",
                              _("Member '%s' not found in DNS") % name,
                              "name",
                              name)

         ### 'name' is key - check for another member with same name
         if self.__dup_dns_ip_found(dns_ipaddress):
            raise OptionError("FATAL",
                              _("Member '%s' already exists in cluster") % name,
                              "name",
                              name)

      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################
      if options.has_key("name"):
         ipconvert = ipConvert()
         netmask = "None"
         if not ipconvert.ipIsInSameNetAsHost(dns_ipaddress, netmask):
            raise OptionError("WARNING",
                              _("Member is not on same network as this host"),
                              "ipaddress",
                              dns_ipaddress)

      ### END WARNINGS ###

   def get_powercontrollers(self):
      powercontroller_entities = []
      nodeList = self.node.getElementsByTagName("powercontroller")
      for n in nodeList:
         powercontroller_entities.append(powercontroller(n, self))
      return powercontroller_entities

   def get_serial_powercontroller(self, device):
      search_options = {"device" : device}
      entity = ClusterList.get_child(self, POWERCONTROLLER_INFO, search_options)
      return powercontroller(entity.node, self)

   def get_network_powercontroller(self, search_options):
      entity = ClusterList.get_child(self, POWERCONTROLLER_INFO, search_options)
      return powercontroller(entity.node, self)

   def add_powercontroller(self, cmdline_args=None):
      entity = ClusterList.add_child(self, POWERCONTROLLER_INFO)
      pc_entity = powercontroller(entity.node, self, "add")
      pc_entity.add_options(cmdline_args)
      return pc_entity

   def delete_powercontroller(self, key, args=None):
      ### args will come in as "None" from Gui, but will be a dictionary
      ### if from cluconfig command line.
      if args != None:
         self.do_getopt(args, POWERCONTROLLER_INFO["OPTIONS"], POWERCONTROLLER_INFO["DELETE_GETOPTS"])
      tokens = string.split(key, ",")
      ct_tokens = len(tokens)
      if ct_tokens == 1:
         ### Must be deleting a 'Serial' Powercontroller since that has only
         ### one key field(device).
         device = tokens[0]
         powercontroller_entity = self.get_serial_powercontroller(device)
      else:
         ### Must be deleting a 'Network' Powercontroller since that has
         ### two key fields (ipaddress,port).
         ipaddress = tokens[0]
         port      = tokens[1]
         search_options = { "ipaddress" : ipaddress,
                            "port"      : port }
         powercontroller_entity = self.get_network_powercontroller(search_options)
      cluster_functions.remove_dom_child(self, powercontroller_entity.node)

   def delete_powercontroller_by_id(self, id):
      ClusterList.delete_by_id(self, POWERCONTROLLER_INFO, id)

   def __change_member_on_failoverdomainnodes(self, old_name, new_name):
      if old_name != new_name and \
         not old_name in ("", "None", None):
            ### First, get a list of failoverdomainnodes that refer to the
            ### *old* member name.  To do that, we need to
            ###   1. get ownerDocument, and user that to
            ###   2. get failoverdomains node, then
            ###   3. get the list of failoverdomains
            ###   4. for each failoverdomain, scan through the list
            ###        of failoverdomainnodes - if a failoverdomainnode
            ###        refers to the "old" member "name", then change
            ###        that to refer to the new member name.
            ownerDocument = self.node.ownerDocument
            failoverdomains_node = ownerDocument.getElementsByTagName("failoverdomains")[0]
            from clusterpkg.failoverdomains_module import failoverdomains
            from clusterpkg.failoverdomain_module import failoverdomain
            failoverdomains_entity = failoverdomains(failoverdomains_node)
            for failoverdomain_entity in failoverdomains_entity.get_list():
                try:
                    failoverdomainnode_entity = failoverdomain_entity.get_failoverdomainnode(old_name)
                except:
                    continue # node(failoverdomainnode) match *NOT* found in
                             # this domain - continue to next domain

                ### To get here, match *WAS* found - change node's name from
                ### old_name to new_name.
                failoverdomainnode_entity.set("name", new_name)

   #######################################################
   # Override the ClusterEntity 'set' function so that
   # if a member 'name' is being changed, we can ensure
   # that all failover domain nodes referring to the
   # "old" member name get changed to refer to the
   # "new" member name.
   #######################################################
   def set(self, option, value):
      if option == "name":
         old_name = self.get("name")
         new_name = value
         if old_name in ("", "None", "none"):
            old_name = ""
         if new_name in ("", "None", "none"):
            new_name = ""
      ClusterEntity.set(self, option, value)
      if option == "name" and \
         new_name != old_name and \
         old_name != "":
            self.__change_member_on_failoverdomainnodes(old_name, new_name)

   def out(self):
      ClusterEntity.out(self)
      nodeList = self.node.getElementsByTagName("powercontroller")
      for n in nodeList:
         powercontroller(n, self).out()
