#!/usr/bin/python
from xml.dom import minidom
from clusterpkg.cluster_functions import OptionError, cluStat
from ClusterEntity import *

TRUE = (1==1)
FALSE = not TRUE

MY_INFO = {
   "WHOAMI"  : "client",

   "OPTIONS" : {
      "id"      : { "datatype" : "int",
                    "order"    : 0 },
      "name"    : { "datatype" : "str",
                    "order"    : 1,
                    "default"  : "*" },
      "options" : { "datatype" : "str",
                    "order"    : 2,
                    "default"  : "None" },
   },

   "ADD_GETOPTS"     : [ "service=",
                            "device=",
                               "nfsexport=",
                                  "add_client",
                                     "name=",
                                     "options=" ],

   "DELETE_GETOPTS"  : [ "service=",
                            "device=",
                               "nfsexport=",
                                  "client=",
                                  "del_client" ],

   "LISTALL_GETOPTS" : [ "service=",
                            "device=",
                               "nfsexport=",
                                  "clients" ],

   "UPDATE_GETOPTS"  : [ "service=",
                            "device=",
                               "nfsexport=",
                                  "client=",
                                     "name=",
                                     "options=" ],
} ### end self.MY_INFO

class client(ClusterEntity):
   def __init__(self, node, parent, action="modify"):

      ClusterEntity.__init__(self, MY_INFO, node, parent, action)

   def __dup_found(self, key, value):
      prev_value = self.get(key)
      if self.action == "add" or \
         (self.action == "modify" and value != prev_value):
          nfsexport_entity = self.parent
          search_options = {key : value}
          client_being_validated = self
          if nfsexport_entity.client_found(search_options, client_being_validated):
              return TRUE
      return FALSE

   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################

      ### Check for *required* fields on "add" ###
      ### Validate "name"
      if self.action == "add":
         if not options.has_key("name") or options["name"] == "":
            raise OptionError("FATAL",
                              _("NFS Export Client Name is required"),
                              "name",
                              None)
      else:
         ### Must be 'modify' ###
         nfsexport = self.parent
         device = nfsexport.parent
         service = device.parent
         service_id = service.get("id")
         clustat = cluStat()
         if clustat.serviceIsRunning(service_id):
            raise OptionError("FATAL",
                              _("Service must be disabled before updates can be made"),
                              "name",
                              service.get("name"))

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      ### Validate submitted options ###
      ### Validate "name"
      if options.has_key("name"):
         name = options["name"]
         if self.__dup_found("name", name):
            raise OptionError("FATAL",
                              _("NFS Export Client Name already exists"),
                              "name",
                              name)

      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################

      # No warnings
