#!/usr/bin/python
from xml.dom import minidom
from clusterpkg.cluster_functions import OptionError, cluStat
from clusterpkg.nfsexport_client_module import client
import clusterpkg.nfsexport_client_module
from ClusterList import *
import os, stat

CLIENT_INFO = clusterpkg.nfsexport_client_module.MY_INFO

TRUE = (1==1)
FALSE = not TRUE

MY_INFO = {
   "WHOAMI"  : "nfsexport",

   "OPTIONS" : {
      "id"   : { "datatype" : "int",
                 "order"    : 0 },
      "name" : { "datatype" : "str",
                 "order"    : 1 },
   },

   "ADD_GETOPTS"     : [ "service=",
                            "device=",
                               "add_nfsexport",
                                  "name=" ],

   "DELETE_GETOPTS"  : [ "service=",
                            "device=",
                               "nfsexport=",
                               "del_nfsexport" ],

   "LISTALL_GETOPTS" : [ "service=",
                            "device=",
                               "nfsexports" ],

   "UPDATE_GETOPTS"  : [ "service=",
                            "device=",
                               "nfsexport=",
                                  "name=" ],
}

class nfsexport(ClusterList):
   def __init__(self, node, parent, action="modify"):
      ClusterList.__init__(self, MY_INFO, node, parent, action)

   def __dup_found(self, key, value):
      prev_value = self.get(key)
      if self.action == "add" or \
         (self.action == "modify" and value != prev_value):
          device_entity = self.parent
          search_options = {key : value}
          nfsexport_being_validated = self
          if device_entity.nfsexport_found(search_options, nfsexport_being_validated):
              return TRUE
      return FALSE

   ### No need to validate that there are no dups for this nfsexport
   ### within the whole cluster (all services and devices), since
   ### the nfsexport directory must be underneath the device mountpoint,
   ### and since the device mountpoint must be unique within the whole
   ### cluster, then there is no way that there can be duplicate
   ### nfsexport directories in 2 different services or 2 different
   ### devices.  The only dup an nfsexport could have is within the
   ### same device, so keep the __dup_found check above.
   #def __dup_found_in_cluster(self, option, value):
   #   this_nfsexport_id = self.get("id")
   #   this_device_id    = self.parent.get("id")
   #   this_service_id   = self.parent.parent.get("id")
   #
   #   ownerDocument = self.node.ownerDocument
   #   services_node = ownerDocument.getElementsByTagName("services")[0]
   #   from clusterpkg.services_module import services
   #   services_entity = services(services_node)
   #   for service in services_entity.get_list():
   #      service_id = service.get("id")
   #      devices = service.get_devices()
   #      for device in devices:
   #         device_id = device.get("id")
   #         nfsexports = device.get_nfsexports()
   #         for nfsexport in nfsexports:
   #            nfsexport_id = nfsexport.get("id")
   #            if nfsexport.get(option) == value \
   #                  and \
   #              (nfsexport_id != this_nfsexport_id or \
   #               device_id    != this_device_id or \
   #               service_id   != this_service_id):
   #                  return TRUE
   #   return FALSE

   def export_dir_under_mountpoint(self, export_dir):
      device = self.parent
      mount = device.get_mount()
      mountpoint = mount.get("mountpoint")
      if mountpoint == "":
         return FALSE # mountpoint not defined - return error
      match_position = string.find(export_dir, mountpoint)
      if match_position == 0:
         return TRUE
      return FALSE

   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################

      ### Check for *required* fields on "add" ###
      ### Validate "name"
      if self.action == "add":
         if not options.has_key("name") or options["name"] == "":
            raise OptionError("FATAL",
                              _("NFS Export Directory Name is required"),
                              "name",
                              None)
      else:
         ### Must be 'modify' ###
         device = self.parent
         service = device.parent
         service_id = service.get("id")
         clustat = cluStat()
         if clustat.serviceIsRunning(service_id):
            raise OptionError("FATAL",
                              _("Service must be disabled before updates can be made"),
                              "name",
                              service.get("name"))

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      ### Validate submitted options ###
      ### Validate "name"
      if options.has_key("name"):
         name = options["name"]

         if name[0] != "/":
            raise OptionError("FATAL", _("NFS Export Directory Name must start with a slash '/'"), "name", name)

         path_exists = TRUE
         try:
            mode = os.stat(name)[stat.ST_MODE]
         except OSError, e:
            #msg = _("NFS Export Directory Name \"%s\" does not exist") % name
            #raise OptionError("FATAL", msg, "name", name)
            path_exists = FALSE
      
         if path_exists and not stat.S_ISDIR(mode):
            msg = _("NFS Export Directory Name must be a directory")
            raise OptionError("FATAL", msg, "name", name)
      
         if self.__dup_found("name", name):
            raise OptionError("FATAL", _("NFS Export Directory Name already exists in device"), "name", name)

         if not self.export_dir_under_mountpoint(name):
            raise OptionError("FATAL", _("NFS Export Directory Name must be underneath the device mount point"), "name", name)

      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################

      # No warnings

   def get_client(self, name):
      search_options = {"name": name}
      client_entity = ClusterList.get_child(self, CLIENT_INFO, search_options)
      return client(client_entity.node, self)

   def client_found(self, search_options, client_being_validated):
      ### "client_being_validated" is either the client being modified, or
      ### the new client just added.  We want to be sure its key info doesn't
      ### duplicate an existing client.
      return ClusterList.child_found(self, CLIENT_INFO, search_options, client_being_validated)

   def get_clients(self):
      client_entityList = []
      nodeList = self.node.getElementsByTagName("client")
      for n in nodeList:
         client_entityList.append(client(n, self))
      return client_entityList

   def add_client(self, cmdline_args=None):
      entity = ClusterList.add_child(self, CLIENT_INFO)
      client_entity = client(entity.node, self, "add")
      client_entity.add_options(cmdline_args)
      return client_entity

   def delete_client(self, name, cmdline_args=None):
      search_options = {"name": name}
      ClusterList.delete_child(self, CLIENT_INFO, search_options, cmdline_args)

   def delete_client_by_id(self, id):
      ClusterList.delete_by_id(self, CLIENT_INFO, id)

   def out(self):
      ClusterEntity.out(self)
      nodeList = self.node.getElementsByTagName("client")
      for n in nodeList:
         client(n, self).out()
