#!/usr/bin/python
from xml.dom import minidom
from clusterpkg.cluster_functions   import OptionError, cluStat
from clusterpkg.nfsexport_module    import nfsexport
import clusterpkg.nfsexport_module
from clusterpkg.device_mount_module import mount
from clusterpkg.util_smbconf_module import smbConf
from ClusterList import *
import os, stat
import string

NFSEXPORT_INFO = clusterpkg.nfsexport_module.MY_INFO

TRUE = (1==1)
FALSE = not TRUE

MY_INFO = {
   "WHOAMI"  : "device",

   "OPTIONS" : {
      "id"           : { "datatype" : "int",
                         "order"    : 0 },

      "name"         : { "datatype" : "str",
                         "order"    : 1 },

      "sharename"    : { "datatype" : "str",
                         "order"    : 2 },
   },

   "ADD_GETOPTS"     : [ "service=",
                            "device=",
                            "add_device",
                               "name=",
                               "sharename=" ],

   "DELETE_GETOPTS"  : [ "service=",
                            "device=",
                            "del_device" ],

   "LISTALL_GETOPTS" : [ "service=",
                            "devices" ],

   "UPDATE_GETOPTS"  : [ "service=",
                            "device=",
                               "name=",
                               "sharename=" ],
}

class device(ClusterList):
   def __init__(self, node, parent, action="modify"):
      ClusterList.__init__(self, MY_INFO, node, parent, action)
      sharename = node.getAttribute("sharename")
      self.smbconf = smbConf(self, node)

   #def __dup_found(self, key, value):
   #   prev_value = self.get(key)
   #   if self.action == "add" or \
   #      (self.action == "modify" and value != prev_value):
   #       service_entity = self.parent
   #       search_options = {key : value}
   #       device_being_validated = self
   #       if service_entity.device_found(search_options, device_being_validated):
   #           return TRUE
   #   return FALSE

   def __dup_found_in_cluster(self, option, value):
      this_service_id = self.parent.get("id")
      this_device_id  = self.get("id")

      ownerDocument = self.node.ownerDocument
      services_node = ownerDocument.getElementsByTagName("services")[0]
      from clusterpkg.services_module import services
      services_entity = services(services_node)
      for service in services_entity.get_list():
         service_id = service.get("id")
         devices = service.get_devices()
         for device in devices:
            if device.get(option) == value \
                  and \
              (device.get("id") != this_device_id or \
               service_id != this_service_id):
                  return TRUE
      return FALSE

   def __svc_already_has_sharename(self, sharename):
      this_device_id  = self.get("id")
      this_device_sharename = sharename
      if this_device_sharename in ("None", "none", ""):
         return FALSE

      service_entity = self.parent
      devices = service_entity.get_devices()
      for device in devices:
         sharename = device.get("sharename")
         if not sharename in ("None", "none", ""):
            if device.get("id") != this_device_id:
               return TRUE
      return FALSE

   def __no_service_ips(self):
      service_entity = self.parent
      service_ip_list = service_entity.get_service_ipaddresses()
      num_service_ips = len(service_ip_list)
      if num_service_ips == 0:
          return TRUE
      return FALSE

   def __no_mountpoint(self):
      try:
         mount_entity = self.get_mount()
      except:
         return TRUE
      mountpoint = mount_entity.get("mountpoint")
      if mountpoint == "":
          return TRUE
      return FALSE

   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):

      if options.has_key("mountpoint"):
         ### Save mountpoint and then remove it from 'options' since
         ### it really isn't a 'device' option.  But we need it to
         ### validate sharename.
         self.mountpoint = options["mountpoint"]
         del options["mountpoint"]
      else:
         self.mountpoint = ""

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################

      ### Check for *required* fields on "add" ###
      ### Validate "name"
      if self.action == "add":
         if not options.has_key("name"):
            name = ""
         else:
            name = options["name"]
      else:
         ### Must be 'modify' ###
         service = self.parent
         service_id = service.get("id")
         clustat = cluStat()
         if clustat.serviceIsRunning(service_id):
            raise OptionError("FATAL",
                              _("Service must be disabled before updates can be made"), 
                              "name",
                              service.get("name"))

         prev_name = self.get("name")
         if options.has_key("name"):
            name = options["name"]
         else:
            name = prev_name

      if name == "":
         raise OptionError("FATAL",
                           _("Device Name is required"),
                           "name",
                           None)

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      ### Validate submitted options ###
      ### Validate "name"
      if options.has_key("name"):
         name = options["name"]
         try:
            mode = os.stat(name)[stat.ST_MODE]
         except OSError, e:
            msg = _("Device Special File \"%s\" does not exist") % name
            raise OptionError("FATAL", msg, "name", name)
      
         if not stat.S_ISBLK(mode):
            msg = _("Device Special File must be a block special device file")
            raise OptionError("FATAL", msg, "name", name)
      
         if self.__dup_found_in_cluster("name", name):
            raise OptionError("FATAL",
                              _("Device Name already exists in cluster"),
                              "name",
                              name)

      ### Validate "sharename" ###
      if options.has_key("sharename") and options["sharename"] != "":
         sharename = options["sharename"]
         if self.__svc_already_has_sharename(sharename):
            raise OptionError("FATAL",
                              _("The Service already contains a device with a Samba Share Name specified - more than one Samba Share Name within a service is not allowed"),
                              "sharename",
                              sharename)

         if string.find(sharename, ' ') != -1:
            raise OptionError("FATAL",
                              _("Samba Share Name may not contain spaces"),
                              "sharename",
                              sharename)

         if self.__dup_found_in_cluster("sharename", sharename):
            raise OptionError("FATAL",
                              _("Samba Share Name already exists in cluster"),
                              "sharename",
                              sharename)

         if self.__no_service_ips():
            raise OptionError("FATAL",
                              _("A Service IP Address must be entered before specifying a Samba Share Name, since the samba share configuration file to be created requires it"),
                              "sharename",
                              sharename)

         ### Samba Share Name *has* been specified - verify that
         ### a 'mountpoint' was either specified, or exists in the
         ### device 'mount'(__no_mountpoint).
         if self.mountpoint == "":
            ### no mountpoint specified - check existing 'mount' tag ###
            if self.__no_mountpoint():
               raise OptionError("FATAL",
                                 _("A Device Mountpoint must be entered before specifying a Samba Share Name, since the samba share configuration file to be created requires it"),
                                 "sharename",
                                 sharename)

      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################

      # No warnings

   def get_nfsexport(self, name):
      search_options = {"name": name}
      nfsexport_entity = ClusterList.get_child(self, NFSEXPORT_INFO, search_options)
      return nfsexport(nfsexport_entity.node, self)

   def nfsexport_found(self, search_options, nfsexport_being_validated):
      ### "nfsexport_being_validated" is either the nfsexport being modified, or
      ### the new nfsexport just added.  We want to be sure his key info doesn't
      ### duplicate an existing nfsexport.
      return ClusterList.child_found(self, NFSEXPORT_INFO, search_options, nfsexport_being_validated)

   def get_nfsexports(self):
      nfsexport_entityList = []
      nodeList = self.node.getElementsByTagName("nfsexport")
      for n in nodeList:
         nfsexport_entityList.append(nfsexport(n, self))
      return nfsexport_entityList

   def add_nfsexport(self, cmdline_args=None):
      entity = ClusterList.add_child(self, NFSEXPORT_INFO)
      nfsexport_entity = nfsexport(entity.node, self, "add")
      nfsexport_entity.add_options(cmdline_args)
      return nfsexport_entity

   def delete_nfsexport(self, name, cmdline_args=None):
      search_options = {"name": name}
      ClusterList.delete_child(self, NFSEXPORT_INFO, search_options, cmdline_args)

   def delete_nfsexport_by_id(self, id):
      ClusterList.delete_by_id(self, NFSEXPORT_INFO, id)

   #####################################################################

   def get_mount(self):
      mount_node = self.node.getElementsByTagName("mount")[0]
      return mount(mount_node, self)

   #####################################################################

   def out(self):
      ClusterEntity.out(self)
      nodeList = self.node.getElementsByTagName("mount")
      for n in nodeList:
         mount(n, self).out()

      nodeList = self.node.getElementsByTagName("nfsexport")
      for n in nodeList:
         nfsexport(n, self).out()
