#!/usr/bin/python
from xml.dom import minidom
from clusterpkg.cluster_functions import OptionError, cluPing, hostInDNS, cluStat
from ClusterEntity import *
import os
import stat
import socket
from clusterpkg.util_ipconvert_module import ipConvert

TRUE = (1==1)
FALSE = not TRUE

MY_INFO = {
   "WHOAMI"  : "service_ipaddress",

   "OPTIONS" : {
      "id"        : { "datatype" : "int",
                      "order"    : 0 },
      "ipaddress" : { "datatype" : "str",
                      "order"    : 1 },
      "netmask"   : { "datatype" : "str",
                      "order"    : 2,
                      "default"  : "None" },
      "broadcast" : { "datatype" : "str",
                      "order"    : 3,
                      "default"  : "None" },
      "monitor_link" : { "datatype" : "int",
                      "order"    : 4,
                      "default"  : "0" },
   },

   "ADD_GETOPTS"     : [ "service=",
                            "add_service_ipaddress",
                               "ipaddress=",
                               "netmask=",
                               "broadcast=" ],

   "DELETE_GETOPTS"  : [ "service=",
                            "service_ipaddress=",
                            "del_service_ipaddress" ],

   "LISTALL_GETOPTS" : [ "service=",
                            "service_ipaddresses" ],

   "UPDATE_GETOPTS"  : [ "service=",
                            "service_ipaddress=",
                               "ipaddress=",
                               "netmask=",
                               "broadcast=" ],
}

class service_ipaddress(ClusterEntity):
   def __init__(self, node, parent, action="modify"):
      ClusterEntity.__init__(self, MY_INFO, node, parent, action)

   #def __dup_found(self, key, value):
   #   "This finds duplicate ip addresses *within the service*"
   #   prev_value = self.get(key)
   #   if self.action == "add" or \
   #      (self.action == "modify" and value != prev_value):
   #       service_ipaddresses_entity = self.parent
   #       search_options = {key : value}
   #       service_ipaddress_being_validated = self
   #       if service_ipaddresses_entity.service_ipaddress_found(search_options, service_ipaddress_being_validated):
   #           return TRUE
   #   return FALSE

   def __dup_found_in_cluster(self, option, value):
      ipaddresses_entity = self.parent
      service_entity = ipaddresses_entity.parent
      this_service_id = service_entity.get("id")
      this_service_ipaddress_id  = self.get("id")
                                                                                
      ownerDocument = self.node.ownerDocument
      services_node = ownerDocument.getElementsByTagName("services")[0]
      from clusterpkg.services_module import services
      services_entity = services(services_node)
      for service in services_entity.get_list():
         service_id = service.get("id")
         service_ipaddresses = service.get_service_ipaddresses()
         for service_ipaddress in service_ipaddresses:
            if service_ipaddress.get(option) == value \
                  and \
              (service_ipaddress.get("id") != this_service_ipaddress_id or \
               service_id != this_service_id):
                  return TRUE
      return FALSE

   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################

      ### Make sure service is stopped before allow user to add or update
      ### service ipaddress(es).
      service_ipaddresses_entity = self.parent
      service_entity = service_ipaddresses_entity.parent
      service_id = service_entity.get("id")
      clustat = cluStat()
      if clustat.serviceIsRunning(service_id):
         raise OptionError("FATAL",
                           _("Service must be disabled before you can add or update service ip addresses"),
                           "name",
                           service_entity.get("name"))

      ### Check for *required* fields on "add" ###
      ### Validate "ipaddress"
      if self.action == "add":
         if (not options.has_key("ipaddress")) or options["ipaddress"] == "":
            raise OptionError("FATAL",
                              _("IP Address is required"),
                              "ipaddress",
                              None)

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      ### Validate submitted options ###
      ### Validate "name"
      if options.has_key("ipaddress"):
         ipaddress = options["ipaddress"]
         #if self.__dup_found("ipaddress", ipaddress):
         if self.__dup_found_in_cluster("ipaddress", ipaddress):
            raise OptionError("FATAL",
                              _("IP Address already exists in cluster"),
                              "ipaddress",
                              ipaddress)

      if options.has_key("ipaddress"):
         ipaddress = options["ipaddress"]

         if hostInDNS(ipaddress) == None:
            raise OptionError("FATAL",
                              _("IP Address not found in DNS"),
                              "ipaddress",
                              ipaddress)

      netmask = ""
      if options.has_key("netmask") and \
         options["netmask"] != "None" and options["netmask"] != "":
         netmask = options["netmask"]
         try:
            socket.inet_aton(netmask)
         except socket.error, e:
            raise OptionError("FATAL",
                              _("Netmask is invalid"),
                              "netmask",
                              netmask)

      if options.has_key("broadcast") and \
         options["broadcast"] != "None" and options["broadcast"] != "":
         broadcast = options["broadcast"]
         try:
            socket.inet_aton(broadcast)
         except socket.error, e:
            raise OptionError("FATAL",
                              _("Broadcast is invalid"),
                              "broadcast",
                              broadcast)

      if options.has_key("ipaddress"):
         ipconvert = ipConvert()
         if not ipconvert.ipIsInSameNetAsHost(ipaddress, netmask):
            raise OptionError("FATAL",
                              _("IP Address not on same network as this host"),
                              "ipaddress",
                              ipaddress)

         if cluPing(ipaddress):
            raise OptionError("FATAL",
                              _("IP Address already in use on network"),
                              "ipaddress",
                              ipaddress)

      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################
      #
