#!/usr/bin/python
from xml.dom import minidom
from clusterpkg.cluster_functions import OptionError, countMembers, cluStat
from clusterpkg.service_device_module import device
import clusterpkg.service_device_module
from clusterpkg.service_ipaddresses_module import service_ipaddresses
import clusterpkg.service_ipaddress_module
from clusterpkg.service_ipaddress_module import service_ipaddress
from ClusterList import *
import os
import stat

DEVICE_INFO = clusterpkg.service_device_module.MY_INFO
NETWORK_INFO = clusterpkg.service_ipaddress_module.MY_INFO

TRUE = (1==1)
FALSE = not TRUE

MY_INFO = {
   "WHOAMI"  : "service",

   "OPTIONS" : {
      "id"                          : { "datatype" : "int",
                                        "order"    : 0 },
      "name"                        : { "datatype" : "str",
                                        "order"    : 1 },
      "checkinterval"               : { "datatype" : "int",
                                        "order"    : 2,
                                        "default"  : 0 },
      "failoverdomain"              : { "datatype" : "str",
                                        "order"    : 3,
                                        "default"  : "None" },
      "userscript"                  : { "datatype" : "str",
                                        "order"    : 4,
                                        "default"  : "None" },
      "maxrestarts"                 : { "datatype" : "int",
                                        "order"    : 5,
                                        "default"  : 0 },
      "maxfalsestarts"              : { "datatype" : "int",
                                        "order"    : 6,
                                        "default"  : 0 },
   },

   "ADD_GETOPTS"     : [ "service=",
                            "name=",
                            "checkinterval=",
                            "failoverdomain=",
                            "userscript=",
                            "maxrestarts=",
                            "maxfalsestarts=",
                         "add_service" ],

   "DELETE_GETOPTS"  : [ "service=",
                            "name=",
                         "del_service" ],

   "LISTALL_GETOPTS" : [ "services" ],

   "UPDATE_GETOPTS"  : [ "service=",
                            "name=",
                            "checkinterval=",
                            "failoverdomain=",
                            "userscript=",
                            "maxrestarts="
                            "maxfalsestarts=" ],
}

class service(ClusterList):
   def __init__(self, node, parent=None, action="modify"):
      ClusterList.__init__(self, MY_INFO, node, parent, action)

   def __dup_found(self, key, value):
      prev_value = self.get(key)
      if self.action == "add" or \
         (self.action == "modify" and value != prev_value):
          services_entity = self.parent
          search_options = {key : value}
          service_being_validated = self
          if services_entity.service_found(search_options, service_being_validated):
              return TRUE
      return FALSE

   ####################################################################
   # NOTE: In 'validate', WARNING errors must come *Last*, after all
   #       FATAL errors.  This is because the command line tool will
   #       override WARNING's in favor of doing the add or update,
   #       whereas the GUI will present the WARNING in a popup dialog.
   #       Since WARNINGs and ERROR's are 'raise'd, editing stops at
   #       that point.  So for the command line, we want to be sure
   #       that all FATAL checks have already been done, so that if a
   #       WARNING is raised, it can be safely overridden sinced we
   #       know that only WARNING checks remain.
   ####################################################################
   def validate(self, options):

      #######################################################################
      ########################   1. FATAL ERRORS   ##########################
      #######################################################################
      ### Check for *required* fields on "add" ###
      if self.action == "add":
         if not options.has_key("name") or options["name"] == "":
            raise OptionError("FATAL",
                              _("Service Name is required"),
                              "name",
                              None)
      else:
         ### Must be 'modify' ###
         service_id = self.get("id")
         clustat = cluStat()
         if clustat.serviceIsRunning(service_id):
            raise OptionError("FATAL",
                              _("Service must be disabled before updates can be made"),
                              "name",
                              self.get("name"))

      ### 'genericOptionChecks' will raise an error if either the data
      ### doesn't match the datatype(str or int), or the option is
      ### not in the list of valid options for this class.
      ClusterEntity.genericOptionChecks(self, options)

      ### Validate submitted options ###
      ### Validate "name"
      if options.has_key("name"):
         name = options["name"]
         ### 'name' is key - check for another service with same name
         if self.__dup_found("name", name):
            raise OptionError("FATAL",
                              _("Service Name already exists"),
                              "name",
                              name)


      if options.has_key("failoverdomain") and \
         (not options["failoverdomain"] in ("None", "none", "")) :
         ### Verify that Failover Domain Name specified for this service
         ### exists as a current Failover Domain name in the cluster.
         failoverdomain = options["failoverdomain"]
         ownerDocument = self.node.ownerDocument
         failoverdomains_node = ownerDocument.getElementsByTagName("failoverdomains")[0]
         from clusterpkg.failoverdomains_module import failoverdomains
         failoverdomains_entity = failoverdomains(failoverdomains_node)
         search_options = { "name" : failoverdomain }
         if not failoverdomains_entity.failoverdomain_found(search_options):
            raise OptionError("FATAL",
                              _("Failover Domain Name '%s' not found in cluster") % failoverdomain,
                              "failoverdomain",
                              failoverdomain)
     
      ### Validate "userscript"
      if options.has_key("userscript") and not options["userscript"] in ("None", ""):
         userscript = options["userscript"]
         try:
            mode = os.stat(userscript)[stat.ST_MODE]
         except OSError, e:
            msg = _("User Script does not exist")
            raise OptionError("FATAL", msg, "userscript", userscript)
         if not stat.S_ISREG(mode):
            msg = _("User Script must be a regular file")
            raise OptionError("FATAL", msg, "userscript", userscript)

         if not os.access(userscript, os.X_OK):
            raise OptionError("FATAL",
                              _("User Script is not executable"),
                              "userscript",
                              userscript)

      if countMembers(self.node) < 1:
            raise OptionError("FATAL",
                              _("Must define cluster members before a service can be added"),
                              "name",
                              None)


      #######################################################################
      ##########################   2. WARNINGS   ############################
      #######################################################################

      # No warnings

   def get_devices(self):
      device_entities = []
      nodeList = self.node.getElementsByTagName("device")
      for n in nodeList:
         device_entities.append(device(n, self))
      return device_entities

   def get_device(self, name):
      search_options = {"name": name}
      device_entity = ClusterList.get_child(self, DEVICE_INFO, search_options)
      return device(device_entity.node, self)

   #def device_found(self, device_being_validated, val, key="name"):
   def device_found(self, search_options, device_being_validated):
      ### "device_being_validated" is either the device being modified, or
      ### the new device just added.  We want to be sure his key info doesn't
      ### duplicate an existing device.
      return ClusterList.child_found(self, DEVICE_INFO, search_options, device_being_validated)

   def add_device(self, cmdline_args=None):
      ### 1st, create "device" ###
      entity = ClusterList.add_child(self, DEVICE_INFO)
      device_entity = device(entity.node, self, "add")
      device_entity.add_options(cmdline_args)

      ### Next, create "mount"(only 1 per device) DOM node ###
      cluster_functions.append_dom_child(device_entity, "mount")
      
      return device_entity

   def delete_device(self, name, cmdline_args=None):
      search_options = {"name": name}
      ClusterList.delete_child(self, DEVICE_INFO, search_options, cmdline_args)

   def delete_device_by_id(self, id):
      ClusterList.delete_by_id(self, DEVICE_INFO, id)

   # # # # # # # # # # # # # # # # # # # # # # # # # # # 

   def get_service_ipaddresses(self):
      try:
         service_ipaddresses_node = self.node.getElementsByTagName("service_ipaddresses")[0]
      except IndexError, e:
         return [] # return an empty list
      nodeList = service_ipaddresses(service_ipaddresses_node, self).get_list()
      return nodeList

   def get_service_ipaddress(self, ipaddress):
      service_ipaddresses_node = self.node.getElementsByTagName("service_ipaddresses")[0]
      service_ipaddresses_entity = service_ipaddresses(service_ipaddresses_node, self)
      service_ipaddress_entity = service_ipaddresses_entity.get_service_ipaddress(ipaddress)
      return service_ipaddress_entity

   def add_service_ipaddress(self, cmdline_args=None):
      ### if 'service_ipaddresses' tag doesn't yet exist, create it ###
      try:
         service_ipaddresses_node = self.node.getElementsByTagName("service_ipaddresses")[0]
      except IndexError, e:
         ### Adding first service, and 'service_ipaddresses' tag doesn't
         ### exist yet, so create it now.
         service_entity = self
         service_ipaddresses_node = cluster_functions.append_dom_child(service_entity, "service_ipaddresses")

      service_ipaddresses_entity = service_ipaddresses(service_ipaddresses_node, self, "add")
      return service_ipaddresses_entity.add_service_ipaddress(cmdline_args)

   def delete_service_ipaddress(self, ipaddress, cmdline_args=None):
      service_ipaddresses_node = self.node.getElementsByTagName("service_ipaddresses")[0]
      service_ipaddresses_entity = service_ipaddresses(service_ipaddresses_node, self)
      service_ipaddresses_entity.delete_service_ipaddress(ipaddress, cmdline_args)

   def delete_service_ipaddress_by_id(self, id):
      service_ipaddresses_node = self.node.getElementsByTagName("service_ipaddresses")[0]
      service_ipaddresses_entity = service_ipaddresses(service_ipaddresses_node, self)
      ClusterList.delete_by_id(service_ipaddresses_entity, NETWORK_INFO, id)

   # # # # # # # # # # # # # # # # # # # # # # # # # # # 

   def out(self):
      ClusterEntity.out(self)

      ### Should only be 1 "service_ipaddresses" node for a service, but this
      ### handles the case when there is *no* "service_ipaddresses" node
      ### for this svc.
      nodeList = self.node.getElementsByTagName("service_ipaddresses")
      for n in nodeList:
         service_ipaddresses(n, self).out()

      nodeList = self.node.getElementsByTagName("device")
      for n in nodeList:
         device(n, self).out()
