#!/usr/bin/python

## daemonPropDialog.py

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

############################################################################
# NOTES:
# ======
# 05/20/2003 HM - Commented out all code referring to 'clupowerd', since
#                 clupowerd has been removed from the daemonPropDialog
#                 in glade.
# 03/20/2003 HM - Commented out all code referring to Stability ('strict'
#                 and 'adaptive', since we've decided to turn both on
#                 by default and not expose them to users.
############################################################################

import gtk
import gtk.glade
import gettext
import string

from clusterpkg.cluster_functions_gui import errorMessage, warningMessage
from clusterpkg.cluster_functions import OptionError, cluStat, countMembers
from clusterpkg.clumembd_module import failover_seconds_greater_than_min_disk_ping_seconds, MINIMUM_TKO_COUNT

TRUE = (1==1)
FALSE = not TRUE

MULTICAST_IP_DEFAULT = "225.0.0.11"
PINGINTERVAL_DEFAULT = "2"
LOGLEVEL_DEFAULT = "4" # WARNING
LOGLEVELS = [ "EMERG - system is unusable",
              "ALERT - action must be taken immediately",
              "CRIT - critical conditions",
              "ERR - error conditions",
              "WARNING - warning conditions",
              "NOTICE - normal but significant condition",
              "INFO - informational",
              "DEBUG - debug-level messages" ]

##
## I18N
##

### FAILOVER_SECONDS key is the failover seconds
###                  value is "tko_count|interval" combination that equals
###                     the failover seconds(when multiplied together.  The
###                     interval is in microseconds.
FAILOVER_SECONDS = { 30 : "30|1000000",
                     25 : "25|1000000",
                     20 : "20|1000000",
                     15 : "20|750000",
                     10 : "20|500000",
                     5  : "15|330000" }

### STABILITY key is the stability scale units(1-3)
###           value is the 'strict' and 'adaptive' fields, and whether
###           or not(0 or 1) they are on.
STABILITY = {
    1 : {
           "label_text" : " No Strict Voting\n No Adaptive Heartbeat",
           "options"    : { "strict"   : "no",
                            "adaptive" : "no" },
        },

    2 : {
           "label_text" : "Yes Strict Voting\n No Adaptive Heartbeat",
           "options"    : { "strict"   : "yes",
                            "adaptive" : "no" },
        },

    3 : {
           "label_text" : "Yes Strict Voting\nYes Adaptive Heartbeat",
           "options"    : { "strict"   : "yes",
                            "adaptive" : "yes" },
        },
}

def getFailoverSecondsAndUnits(tko_count, interval):
    value = "%s|%s" % (tko_count, interval)
    for seconds, tko_count_and_interval_string in FAILOVER_SECONDS.items():
       if tko_count_and_interval_string == value:
          units = seconds / 5
          return (seconds, units)
    ### If tko_count and interval combo don't exist in FAILOVER_SECONDS,
    ### then default to 1st(slowest) FAILOVER_SECONDS element.
    keys = FAILOVER_SECONDS.keys()
    keys.sort()
    seconds = keys[-1]
    units = seconds / 5
    return (seconds, units)

def getTkoCountAndInterval(failover_scale_units):
    failover_seconds = failover_scale_units * 5
    tko_count_and_interval_string = FAILOVER_SECONDS[failover_seconds]
    (tko_count, interval) = string.split(tko_count_and_interval_string, "|")
    return (tko_count, interval)

def getStabilityLabelAndUnits(strict, adaptive):
    for unit, unit_dict in STABILITY.items():
       if unit_dict["options"]["strict"]   == strict and \
          unit_dict["options"]["adaptive"] == adaptive:
             return (unit_dict["label_text"], unit)
    ### If strict and adaptive combo doesn't exist in STABILITY,
    ### then default to highest STABILITY element.
    keys = STABILITY.keys()
    keys.sort()
    highest_stability_unit = keys[-1]
    highest_label_text = STABILITY[highest_stability_unit]["label_text"]
    return (highest_label_text, highest_stability_unit)

def getStabilityStrictAndAdaptive(stability_scale_units):
    strict   = STABILITY[stability_scale_units]["options"]["strict"]
    adaptive = STABILITY[stability_scale_units]["options"]["adaptive"]
    return (strict, adaptive)

class daemonPropDialog(gtk.Dialog):
    def __init__(self, glade_xml, clu):
        gtk.Dialog.__init__(self, "")
        self.glade_xml = glade_xml
        self.clu = clu
        self.cluster    = clu.getcluster() # get general cluster attributes
        self.clumembd   = clu.getclumembd()
        #self.clupowerd  = clu.getclupowerd()
        self.cluquorumd = clu.getcluquorumd()
        self.clurmtabd  = clu.getclurmtabd()
        self.clusvcmgrd = clu.getclusvcmgrd()
        self.clulockd   = clu.getclulockd()

        self.dialog = glade_xml.get_widget('daemonPropertiesDialog')

        self.refresh = TRUE

        self.broadcast_radiobutton_widget = self.glade_xml.get_widget('clumembd_broadcast_radiobutton')
        self.multicast_radiobutton_widget = self.glade_xml.get_widget('clumembd_multicast_radiobutton')
        self.multicast_ipaddress_widget = self.glade_xml.get_widget('clumembd_multicast_ipaddress_entry')

        self.pinginterval_radiobutton_widget = self.glade_xml.get_widget('cluquorumd_pinginterval_radiobutton')
        self.pinginterval_entry_widget       = self.glade_xml.get_widget('cluquorumd_pinginterval_entry')

        self.tiebreakerip_radiobutton_widget = self.glade_xml.get_widget('cluquorumd_tiebreakerip_radiobutton')
        self.tiebreakerip_entry_widget       = self.glade_xml.get_widget('cluquorumd_tiebreakerip_entry')

    def on_speed_value_changed(self, scale):
	val = int(scale.get_value()) * 5
	self.seconds_label.set_text(_("%2d second failover") % val)

    def on_stability_value_changed(self, scale):
	scale_units = int(scale.get_value())
        label_text = "%s" % STABILITY[scale_units]["label_text"]
	self.stability_label.set_text(label_text)

    def on_cancelbutton_clicked(self, button):
        self.refresh = TRUE

    def on_okbutton_clicked(self, button):
        ### Copy the new values from the Gui widgets into the DOM xml ###
        clumembd_loglevel = self.glade_xml.get_widget('clumembd_loglevel_optionmenu').get_history()

        failover_speed_scale_units = int(self.glade_xml.get_widget('clumembd_speed_hscale').get_value())
        (clumembd_tko_count, clumembd_interval) = getTkoCountAndInterval(failover_speed_scale_units)

        #stability_scale_units = int(self.glade_xml.get_widget('clumembd_stability_hscale').get_value())
        #(clumembd_strict, clumembd_adaptive) = getStabilityStrictAndAdaptive(stability_scale_units)

        clumembd_broadcast = self.broadcast_radiobutton_widget.get_active()
        if clumembd_broadcast == TRUE:
            clumembd_broadcast = "yes"
        else:
            clumembd_broadcast = "no"

        clumembd_multicast = self.multicast_radiobutton_widget.get_active()
        if clumembd_multicast == TRUE:
            clumembd_multicast = "yes"
        else:
            clumembd_multicast = "no"

        if clumembd_multicast == "no":
            clumembd_multicast_ipaddress = ""
        else:
            clumembd_multicast_ipaddress = self.multicast_ipaddress_widget.get_text()

        ##########################################
        #clupowerd_loglevel = self.glade_xml.get_widget('clupowerd_loglevel_optionmenu').get_history()
        ##########################################
        cluquorumd_loglevel = self.glade_xml.get_widget('cluquorumd_loglevel_optionmenu').get_history()
        cluquorumd_pinginterval = self.glade_xml.get_widget('cluquorumd_pinginterval_entry').get_text()
        cluquorumd_tiebreakerip = self.glade_xml.get_widget('cluquorumd_tiebreakerip_entry').get_text()

        cluquorumd_tiebreakerip_selected = self.tiebreakerip_radiobutton_widget.get_active()
        if cluquorumd_tiebreakerip_selected == TRUE:
            cluquorumd_tiebreakerip = string.strip(cluquorumd_tiebreakerip)
            if cluquorumd_tiebreakerip == "":
                msg = _("Network Tiebreaker was selected but no IP Address was entered.\n\nPlease enter an IP Address for Network Tiebreaker.")
                errorMessage(msg, self.dialog)
                self.refresh = FALSE # Don't refresh - leave bad values
                                     # on screen.
                rc = gtk.RESPONSE_REJECT
                self.dialog.response(rc)
                return

        cluquorumd_pinginterval_selected = self.pinginterval_radiobutton_widget.get_active()
        if cluquorumd_pinginterval_selected == TRUE:
            cluquorumd_pinginterval = string.strip(cluquorumd_pinginterval)
            if cluquorumd_pinginterval == "" and countMembers(self.clumembd.node) == 2:
                msg = _("Disk Tiebreaker was selected but no Disk Ping Interval was entered.\n\nPlease enter a Disk Ping Interval (in seconds) for Disk Tiebreaker.")
                errorMessage(msg, self.dialog)
                self.refresh = FALSE # Don't refresh - leave bad values
                                     # on screen.
                rc = gtk.RESPONSE_REJECT
                self.dialog.response(rc)
                return

        ##########################################
        clurmtabd_loglevel = self.glade_xml.get_widget('clurmtabd_loglevel_optionmenu').get_history()

        clurmtabd_pollinterval = self.glade_xml.get_widget('clurmtabd_pollinterval_spinbutton').get_value_as_int()
        ##########################################
        clusvcmgrd_loglevel = self.glade_xml.get_widget('clusvcmgrd_loglevel_optionmenu').get_history()
        ##########################################
        clulockd_loglevel = self.glade_xml.get_widget('clulockd_loglevel_optionmenu').get_history()

        clumembd_options = {
            "loglevel"   : clumembd_loglevel,
            "tko_count"  : clumembd_tko_count,
            "interval"   : clumembd_interval,
            #"strict"     : clumembd_strict,
            #"adaptive"   : clumembd_adaptive,
            "broadcast"  : clumembd_broadcast,
            "multicast"  : clumembd_multicast,
            "multicast_ipaddress" : clumembd_multicast_ipaddress
        }

        #clupowerd_options = { "loglevel"  : clupowerd_loglevel }

        cluquorumd_options = { "loglevel"     : cluquorumd_loglevel,
                               "pinginterval" : cluquorumd_pinginterval,
                               "tiebreaker_ip": cluquorumd_tiebreakerip }

        clurmtabd_options = { "loglevel"     : clurmtabd_loglevel,
                              "pollinterval" : clurmtabd_pollinterval }

        clusvcmgrd_options = { "loglevel"  : clusvcmgrd_loglevel }

        clulockd_options = { "loglevel"  : clulockd_loglevel }

        self.refresh = TRUE
        rc = gtk.RESPONSE_OK
        try:
            ### Invoke service 'validate' to validate all options
            self.clumembd.validate(clumembd_options)

            if countMembers(self.clumembd.node) == 2 and \
               cluquorumd_pinginterval != "":
                (rc,
                 error_msg,
                 new_disk_pinginterval) = \
                    failover_seconds_greater_than_min_disk_ping_seconds(
                        clumembd_interval,
                        clumembd_tko_count,
                        cluquorumd_pinginterval)
    
                if rc != 0:
                    real_msg = _("Ping Interval * %d must be less than Failover time.  Click 'OK' to change 'cluquorumd' Ping Interval to %d") % (MINIMUM_TKO_COUNT, new_disk_pinginterval)
                    msg = _("%s\n\nPress 'OK' to Continue, or  'Cancel' to Exit.") % real_msg
                    rc = warningMessage(msg, self.dialog)
                    if rc != gtk.RESPONSE_OK:
                       self.refresh = FALSE # Don't refresh - leave bad values
                                            # on screen.
                       rc = gtk.RESPONSE_REJECT
                       self.dialog.response(rc)
                       return
    
                    cluquorumd_options["pinginterval"] = new_disk_pinginterval
                    cluquorumd_pinginterval = new_disk_pinginterval

            #self.clupowerd.validate(clupowerd_options)
            self.cluquorumd.validate(cluquorumd_options)
            self.clurmtabd.validate(clurmtabd_options)
            self.clusvcmgrd.validate(clusvcmgrd_options)
            self.clulockd.validate(clulockd_options)
        except OptionError, e:
            if e.getSeverity() == 'FATAL':
                errorMessage(e.getMessage(), self.dialog)
                self.refresh = FALSE # Don't refresh - leave bad values
                                     # on screen.
                rc = gtk.RESPONSE_REJECT

            elif e.getSeverity() == 'WARNING':
                msg = _("%s\n\nPress 'OK' to Continue, or  'Cancel' to Exit.") % e.getMessage()
                rc = warningMessage(msg, self.dialog)
                if rc != gtk.RESPONSE_OK:
                   self.refresh = FALSE # Don't refresh - leave bad values
                                        # on screen.
                   rc = gtk.RESPONSE_REJECT

        if rc == gtk.RESPONSE_OK:
            ### Validation passed on all fields - now "set" each by calling
            ### the ClusterEntity "set".
            self.clumembd.set("loglevel", clumembd_loglevel)
            self.clumembd.set("tko_count", clumembd_tko_count)
            self.clumembd.set("interval", clumembd_interval)
            #self.clumembd.set("strict", clumembd_strict)
            #self.clumembd.set("adaptive", clumembd_adaptive)
            self.clumembd.set("broadcast", clumembd_broadcast)
            self.clumembd.set("multicast", clumembd_multicast)
            self.clumembd.set("multicast_ipaddress", clumembd_multicast_ipaddress)
    
            #self.clupowerd.set("loglevel", clupowerd_loglevel)
    
            self.cluquorumd.set("loglevel", cluquorumd_loglevel)
            self.cluquorumd.set("pinginterval", cluquorumd_pinginterval)
            self.cluquorumd.set("tiebreaker_ip", cluquorumd_tiebreakerip)
    
            self.clurmtabd.set("loglevel", clurmtabd_loglevel)
            self.clurmtabd.set("pollinterval", clurmtabd_pollinterval)
    
            self.clusvcmgrd.set("loglevel", clusvcmgrd_loglevel)

            self.clulockd.set("loglevel", clulockd_loglevel)

        self.dialog.response(rc)
        return

    def on_broadcast_or_multicast_radiobutton_clicked(self, button):
        if self.multicast_radiobutton_widget.get_active():
            ### clumembd 'multicast' *IS* checked
            self.multicast_ipaddress_widget.set_sensitive(TRUE)
            multicast_ipaddress = self.multicast_ipaddress_widget.get_text()
            if multicast_ipaddress == "":
                self.multicast_ipaddress_widget.set_text(MULTICAST_IP_DEFAULT)
        else:
            ### clumembd 'multicast' is *NOT* checked
            self.multicast_ipaddress_widget.set_sensitive(FALSE)

    def on_pinginterval_or_tiebreakerip_radiobutton_clicked(self, button):
        if self.pinginterval_radiobutton_widget.get_active():
            ### Ping Interval is selected ###
            self.pinginterval_entry_widget.set_sensitive(TRUE)
            self.tiebreakerip_entry_widget.set_sensitive(FALSE)
            self.pinginterval_entry_widget.set_text(PINGINTERVAL_DEFAULT)
            self.tiebreakerip_entry_widget.set_text("")
        else:
            ### Tiebreaker IP is selected ###
            self.tiebreakerip_entry_widget.set_sensitive(TRUE)
            self.pinginterval_entry_widget.set_sensitive(FALSE)
            self.pinginterval_entry_widget.set_text("")

    def build_menu(self):
        ### Create gtk Menu of loglevels that will be used to create
        ### loglevel optionmenu's for each of the daemons.
        menu = gtk.Menu()
        for LOGLEVEL in LOGLEVELS:
            mitem = gtk.MenuItem(LOGLEVEL)
            mitem.show()
            menu.append(mitem)
        return menu

    def build_optionmenu(self, daemon_entity):
        loglevel = daemon_entity.get("loglevel")
        if loglevel == "":
            loglevel = LOGLEVEL_DEFAULT
        loglevel = int(loglevel)

        daemon_name = daemon_entity.whoami
        optionmenu_widgetname = daemon_name + "_loglevel_optionmenu"
        loglevelOptionMenu = self.glade_xml.get_widget(optionmenu_widgetname)
        loglevelOptionMenu.set_menu(self.build_menu())
        loglevelOptionMenu.set_history(loglevel)
        return loglevelOptionMenu

    def run(self):
        ### Now connect the buttons to the functions ###
        self.ok     = self.glade_xml.get_widget('okbutton').connect("clicked", self.on_okbutton_clicked)
        self.cancel = self.glade_xml.get_widget('cancelbutton').connect("clicked", self.on_cancelbutton_clicked)
        broadcast_radiobutton = self.broadcast_radiobutton_widget.connect("clicked", self.on_broadcast_or_multicast_radiobutton_clicked)
        multicast_radiobutton = self.multicast_radiobutton_widget.connect("clicked", self.on_broadcast_or_multicast_radiobutton_clicked)
        pinginterval_radiobutton = self.pinginterval_radiobutton_widget.connect("clicked", self.on_pinginterval_or_tiebreakerip_radiobutton_clicked)
        tiebreakerip_radiobutton = self.tiebreakerip_radiobutton_widget.connect("clicked", self.on_pinginterval_or_tiebreakerip_radiobutton_clicked)

        if self.refresh:
            ### clumembd ###
            ### Load 'clumembd' Speed fields to screen ###
            tko_count = self.clumembd.get("tko_count")
            interval  = self.clumembd.get("interval")
            (failover_seconds, failover_scale_units) = \
                getFailoverSecondsAndUnits(tko_count, interval)
            speed_scale_widget = self.glade_xml.get_widget('clumembd_speed_hscale')
            speed_scale_widget.connect("value_changed", self.on_speed_value_changed)
            cannot_modify_seconds_message_widget = self.glade_xml.get_widget('clumembd_cannot_modify_seconds_label')
            clu_stat = cluStat()
            if clu_stat.clusterIsRunning():
                speed_scale_widget.set_sensitive(FALSE)
                cannot_modify_seconds_message_widget.set_text("Cluster is running - cannot modify failover speed")
            else:
                speed_scale_widget.set_sensitive(TRUE)
                cannot_modify_seconds_message_widget.set_text("")
                
            self.seconds_label = self.glade_xml.get_widget('clumembd_seconds_label')
            self.seconds_label.set_text("%s second failover" % failover_seconds)
            speed_scale_widget.set_value(failover_scale_units)
    
            ### Load 'clumembd' Stability fields to screen ###
            #strict   = self.clumembd.get("strict")
            #adaptive = self.clumembd.get("adaptive")
            #(label_text, stability_scale_units) = getStabilityLabelAndUnits(strict, adaptive)
            #stability_scale_widget = self.glade_xml.get_widget('clumembd_stability_hscale')
    	#stability_scale_widget.connect("value_changed", self.on_stability_value_changed)
            #self.stability_label = self.glade_xml.get_widget('clumembd_stability_text_label')
            #self.stability_label.set_text(label_text)
            #stability_scale_widget.set_value(stability_scale_units)
    
            ### Load 'clumembd' remaining fields to screen ###
            self.build_optionmenu(self.clumembd)
            broadcast = self.clumembd.get("broadcast")
            value = TRUE # default is TRUE (or yes)
            if broadcast == "no":
                value = FALSE
            self.glade_xml.get_widget('clumembd_broadcast_radiobutton').set_active(value)
    
            multicast = self.clumembd.get("multicast")
            value = TRUE # default is TRUE (or yes)
            if multicast == "no":
                value = FALSE
            self.glade_xml.get_widget('clumembd_multicast_radiobutton').set_active(value)
            multicast_ipaddress = self.clumembd.get("multicast_ipaddress")
            self.glade_xml.get_widget('clumembd_multicast_ipaddress_entry').set_text(multicast_ipaddress)
    
            ### Load 'clupowerd' options to screen ###
            #self.build_optionmenu(self.clupowerd)

            ### Load 'cluquorumd' options to screen ###
            self.build_optionmenu(self.cluquorumd)

            ### Figure out how many members have been defined ###
            #ownerDocument = self.cluquorumd.node.ownerDocument
            #members_node = ownerDocument.getElementsByTagName("members")[0]
            #from clusterpkg.members_module import members
            #members_entity = members(members_node)
            #members_list = members_entity.get_list()
            #count_members = len(members_list)
            count_members = countMembers(self.cluquorumd.node)

            tiebreaker_ip = self.cluquorumd.get("tiebreaker_ip")
            ping_interval = self.cluquorumd.get("pinginterval")
            if count_members == 2:
                msg = _("Cluster has 2 members - Network Tiebreaker IP Address is recommended")
                self.tiebreakerip_radiobutton_widget.set_sensitive(TRUE)
                self.pinginterval_radiobutton_widget.set_sensitive(TRUE)

                if ping_interval != "":
                    self.tiebreakerip_radiobutton_widget.set_active(FALSE)
                    self.tiebreakerip_entry_widget.set_sensitive(FALSE)
                    self.tiebreakerip_entry_widget.set_text("")

                    self.pinginterval_radiobutton_widget.set_active(TRUE)
                    self.pinginterval_entry_widget.set_sensitive(TRUE)
                    self.pinginterval_entry_widget.set_text(ping_interval)
                else:
                    self.tiebreakerip_radiobutton_widget.set_active(TRUE)
                    self.tiebreakerip_entry_widget.set_sensitive(TRUE)
                    self.tiebreakerip_entry_widget.set_text(tiebreaker_ip)

                    self.pinginterval_radiobutton_widget.set_active(FALSE)
                    self.pinginterval_entry_widget.set_sensitive(FALSE)
                    self.pinginterval_entry_widget.set_text("")

            elif count_members == 4:
                msg = _("Cluster has 4 members - Network Tiebreaker IP Address is required")
                self.pinginterval_radiobutton_widget.set_sensitive(FALSE)
                self.pinginterval_radiobutton_widget.set_active(FALSE)
                self.pinginterval_entry_widget.set_text("")
                self.pinginterval_entry_widget.set_sensitive(FALSE)

                self.tiebreakerip_radiobutton_widget.set_sensitive(TRUE)
                self.tiebreakerip_entry_widget.set_sensitive(TRUE)
                self.tiebreakerip_radiobutton_widget.set_active(TRUE)
                self.tiebreakerip_entry_widget.set_text(tiebreaker_ip)

            else:
                msg = _("Cluster has %s members:\nDisk Tiebreaker and Network Tiebreaker will not be used.") % count_members
                self.pinginterval_radiobutton_widget.set_sensitive(FALSE)
                self.pinginterval_radiobutton_widget.set_active(FALSE)
                self.pinginterval_entry_widget.set_sensitive(FALSE)
                self.pinginterval_entry_widget.set_text("")

                self.tiebreakerip_radiobutton_widget.set_sensitive(FALSE)
                self.tiebreakerip_radiobutton_widget.set_active(FALSE)
                self.tiebreakerip_entry_widget.set_sensitive(FALSE)
                self.tiebreakerip_entry_widget.set_text("")

            self.glade_xml.get_widget('tiebreaker_text').set_text(msg)

            ### Load 'clurmtabd' options to screen ###
            self.build_optionmenu(self.clurmtabd)
            pollinterval = self.clurmtabd.get("pollinterval")
            self.glade_xml.get_widget('clurmtabd_pollinterval_spinbutton').set_value(pollinterval)
    
            ### Load 'clusvcmgrd' options to screen ###
            self.build_optionmenu(self.clusvcmgrd)

            ### Load 'clulockd' options to screen ###
            self.build_optionmenu(self.clulockd)

        rc = self.dialog.run()
        self.glade_xml.get_widget('okbutton').disconnect(self.ok)
        self.glade_xml.get_widget('cancelbutton').disconnect(self.cancel)
        self.broadcast_radiobutton_widget.disconnect(broadcast_radiobutton)
        self.multicast_radiobutton_widget.disconnect(multicast_radiobutton)
        self.pinginterval_radiobutton_widget.disconnect(pinginterval_radiobutton)
        self.tiebreakerip_radiobutton_widget.disconnect(tiebreakerip_radiobutton)
        return rc

    def hide(self):
        return self.dialog.hide()
