import sys

import gtk
import gtk.glade
import signal
import os
import gettext
import gobject

import memberDialog

##
## I18N
## 

_=gettext.gettext

COLUMN_KEY = 0
COLUMN_DATA = 1

class membersTreeview:
    def __init__(self, glade_xml, members_entity):
        self.members_entity = members_entity
        self.glade_xml = glade_xml
        self.tree_view = self.glade_xml.get_widget ('membersTreeview')
        model = gtk.TreeStore(gobject.TYPE_STRING,
                              gobject.TYPE_PYOBJECT)
        self.tree_view.set_model(model)

        column = gtk.TreeViewColumn(_("Name"),
                                    gtk.CellRendererText(),
                                    text=COLUMN_KEY)
        self.tree_view.append_column(column)

        self.tree_view.get_selection().connect("changed",
                                            self.on_tree_view_selection_changed)
        self.tree_view.connect("row_activated",
                               self.properties, None)

        self.columns = { "COLUMN_KEY": COLUMN_KEY,
                         "COLUMN_DATA" : COLUMN_DATA }

        self.tree_view.get_selection().select_path ((0,))
        self.dialog = memberDialog.memberDialog(self.glade_xml)

        for m in members_entity.get_list():
            member_item = memberDialog.memberItem(self, m)
            member_item.GUI_tree_load_member()

    def on_tree_view_selection_changed(self, selection):
        result = selection.get_selected ()
        if result == None:
            return 
        (model, iter) = result
        if iter == None:
            return
        item = model.get_value (iter, COLUMN_DATA)
        if item.get_type() == "powercontroller":
            self.glade_xml.get_widget('add_child_button').set_sensitive(gtk.FALSE)
            self.glade_xml.get_widget('add_child_menu').set_sensitive(gtk.FALSE)
        else:
            self.glade_xml.get_widget('add_child_button').set_sensitive(gtk.TRUE)
            self.glade_xml.get_widget('add_child_menu').set_sensitive(gtk.TRUE)

    def add(self):
        iter = None
        result = self.tree_view.get_selection().get_selected ()
        if result != None:
            (model, iter) = result
        if iter != None:
            item = model.get_value(iter, COLUMN_DATA)
        else:
            ### iter == None, which means at the top level, adding a 'member'
            item = memberDialog.memberItem(self, None)

        newitem = item.add(iter)
        if newitem != None:
             model.set(iter,
                       COLUMN_KEY, newitem.get("name"),
                       COLUMN_DATA, newitem)

    def add_child(self):
        iter=None
        result = self.tree_view.get_selection().get_selected ()
        if result != None:
            (model, iter) = result
        if iter != None:
            item = model.get_value(iter, COLUMN_DATA)
        else:
            item = memberDialog.memberItem(self, None)

        newitem = item.add_child(iter)
        if newitem != None:
             model.set(iter,
                       COLUMN_KEY, newitem.get("name"),
                       COLUMN_DATA, newitem)

    def delete(self):
        result = self.tree_view.get_selection().get_selected ()
        if result == None:
            return 
        (model, iter) = result
        if iter == None:
            return
        item = model.get_value(iter, COLUMN_DATA)
        message=_("Are you sure you want to remove %s '%s'?") % (item.get_type()
, item.get_value_of_key_field())
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO,
                                gtk.BUTTONS_YES_NO,
                                message)
        dlg.set_position(gtk.WIN_POS_MOUSE)
        dlg.show_all()
        rc = dlg.run()
        dlg.destroy()
        if rc==gtk.RESPONSE_YES:
            ### remove "item" from DOM XML ###
            delete_succeeded = item.delete()
            if delete_succeeded:
                ### remove "iter" from Gui ###
                model.remove(iter)

    def properties(self, *args):
        result = self.tree_view.get_selection().get_selected ()
        if result == None:
            return 
        (model, iter) = result
        if iter == None:
            return
        item = model.get_value (iter, COLUMN_DATA)
        item.properties(iter)
