#!/usr/bin/python

## nfsBackend.py - Contains the backend nfs code needed for redhat-config-filesharing
## Copyright (C) 2002 Red Hat, Inc.
## Copyright (C) 2002 Brent Fox <bfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import string
import os
import gtk
import nfsData
from sharedfilesystem import *

def isRunning():
    rc=getstatusoutput("/sbin/service nfs status")
    if rc[1].find('stopped')!=-1:
        return FALSE
    else:
        return TRUE

def start():
    os.system('/sbin/service nfs start < /dev/null >/dev/null')

def stop():
    os.system('/sbin/service nfs stop < /dev/null >/dev/null')

def reload():
    os.system('/sbin/service nfs reload < /dev/null 2> /dev/null')

class nfsBackend(sharedFileSystem):

    def __init__(self,exportFile='/etc/exports'):
        self.entries = []
        self.exportFile=exportFile;
        self.parseFile()
        self.dirty=FALSE

    def isDirty(self):
        return self.dirty

    def stop(self):
        stop()

    def start(self):
        start()

    def isRunning(self):
        return isRunning()

    def reload(self):
        reload()
        
    def getEntries(self):
        return self.entries

    def getEntry(self,dir):
        for key in self.entries:
            if key.getDirectory() == nfsData.stripDir(dir):
                return key
        return None

    def setEntries(self,entry):
        ctr=0
        for key in self.entries:
            if key.getDirectory() == entry.getDirectory():
                self.entries[ctr]=entry
                return
            ctr=ctr+1
        self.entries.append(entry)

    def delete(self,dir):
        ctr=0
        for key in self.entries:
            if key.getDirectory() == nfsData.stripDir(dir):
                del self.entries[ctr]
                self.dirty=TRUE
                return
            ctr=ctr+1

    def out(self):
        for i in self.entries:
            print i.get_line()
            
    def save(self):
        tmpFile=self.exportFile+'.tmp';
        fd = open(tmpFile, 'w')

        for i in self.entries:
            fd.write(i.get_line()+"\n")

        fd.close()
        os.rename(tmpFile, self.__follow_link(self.exportFile))
        self.dirty=FALSE
    def __follow_link(self, path):
      try:
         target = os.readlink(path) # raises exception if not link
         target = os.path.join(os.path.dirname(path), target)
         path = target
      except:
         # path is not a link
         return path
      return self.__follow_link(path)
   
    def isExported(self,dir):
        for key in self.entries:
            if key.getDirectory() == nfsData.stripDir(dir):
                return NFS
        return FALSE

    def parseFile(self):
        fd = open(self.exportFile, 'r')

        line = fd.readline()
        while line:
            nfsDataObject = nfsData.nfsData(line.rstrip())
            self.entries.append(nfsDataObject)
            line = fd.readline()

        fd.close()
