#!/usr/bin/python

import gtk
import gobject
import string
import os
import stat
from clusterpkg.cluster_functions     import cluStat, cluPing, hostInDNS
from clusterpkg.cluster_functions_gui import errorMessage, warningMessage
from clusterpkg.util_df_module        import cluDF
from clusterpkg.util_ipconvert_module import ipConvert

TRUE = 1
FALSE = 0

### Defaults for screen fields ###
CLIENT_NAME_DEFAULT    = "*"
CLIENT_OPTIONS_DEFAULT = "None"

### Existing Service columns
COLUMN_NAME = 0
COLUMN_DATA = 1

### Existing Device columns
COL_DEVICE_NAME = 0
COL_DEVICE_MOUNTPOINT = 1
COL_DEVICE_DATA = 2

def dirUnderMountpoint(directory, mountpoint):
    match_position = string.find(directory, mountpoint)
    if match_position != 0:
        return FALSE # Directory is *NOT* under mountpoint
    return TRUE

class nfsExportDruid:
    def __init__(self, glade_xml, clu):
        self.glade_xml = glade_xml
        self.clu = clu
        self.services_entity = self.clu.getservices()
        self.dlg = self.glade_xml.get_widget("addNFSDruidDialog")
        self.druid = self.glade_xml.get_widget("addNFSDruid")
        self.druid.connect("cancel",self.cancel)
        self.page1 = self.glade_xml.get_widget("nfsdruidpagestart")

        #############################################
        ### Page 1 fields - Export Directory Name ###
        #############################################
        self.dirNameEntry = self.glade_xml.get_widget("nfsDruidExportDirEntry")
        self.clientNameEntry = self.glade_xml.get_widget("nfsDruidExportClientEntry")
        self.clientOptionsEntry = self.glade_xml.get_widget("nfsDruidExportClientOptionsEntry")
        self.glade_xml.get_widget("nfsDirNamePage").connect("next", self.on_dirNamePage_next)
        #print "dir(nfsDirNamePage) = %s" % dir(self.glade_xml.get_widget("nfsDirNamePage"))

        ######################################
        ### Page 2 fields - Select Service ###
        ######################################
        self.existSvcRadiobutton = self.glade_xml.get_widget("nfsExistSvcRadiobutton")
        self.newSvcRadiobutton = self.glade_xml.get_widget("nfsNewSvcRadiobutton")
        self.newSvcNameEntry = self.glade_xml.get_widget("nfsNewSvcNameEntry")
        self.newSvcIPAddressEntry = self.glade_xml.get_widget("nfsNewSvcIPAddressEntry")
        self.glade_xml.get_widget("nfsServicePage").connect("next", self.on_ServicePage_next)

        self.existSvcRadiobutton.set_active(TRUE)
        self.newSvcRadiobutton.set_active(FALSE)

        self.existSvcRadiobutton.connect("clicked", self.on_svcRadiobuttonChanged)
        self.newSvcRadiobutton.connect("clicked", self.on_svcRadiobuttonChanged)

        self.svcTreeview = self.glade_xml.get_widget ('nfsExistSvcTreeview')
        ### Just a simple list - no children, so use 'ListStore' (not TreeStore)
        self.svcModel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
        self.svcTreeview.set_model(self.svcModel)
        column = gtk.TreeViewColumn(_("Name"),
                                    gtk.CellRendererText(),
                                    text=COLUMN_NAME)
        self.svcTreeview.append_column(column)


        ##############################
        ### Page 3 fields - Device ###
        ##############################
        self.deviceServiceLabel = self.glade_xml.get_widget("nfsDeviceServiceLabel")
        self.existDeviceRadiobutton = self.glade_xml.get_widget("nfsExistDeviceRadiobutton")
        self.newDeviceRadiobutton = self.glade_xml.get_widget("nfsNewDeviceRadiobutton")
        self.newDeviceNameEntry = self.glade_xml.get_widget("nfsNewDeviceSpecialFileEntry")
        self.newDeviceMountpointEntry = self.glade_xml.get_widget("nfsNewDeviceMountpointEntry")

        self.existDeviceRadiobutton.connect("clicked", self.on_deviceRadiobuttonChanged)
        self.newDeviceRadiobutton.connect("clicked", self.on_deviceRadiobuttonChanged)

        self.deviceTreeview = self.glade_xml.get_widget ('nfsExistDeviceTreeview')
        ### Just a simple list - no children, so use 'ListStore' (not TreeStore)
        self.deviceModel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
        self.deviceTreeview.set_model(self.deviceModel)
        column = gtk.TreeViewColumn(_("Name"),
                                    gtk.CellRendererText(),
                                    text=COL_DEVICE_NAME)
        self.deviceTreeview.append_column(column)
        column = gtk.TreeViewColumn(_("Mountpoint"),
                                    gtk.CellRendererText(),
                                    text=COL_DEVICE_MOUNTPOINT)
        self.deviceTreeview.append_column(column)


        self.glade_xml.get_widget("nfsDevicePage").connect("next", self.on_devicePage_next)

        ##########################
        ### Last Page - Finish ###
        ##########################
        self.glade_xml.get_widget("addNFSDruidFinish").connect("finish",self.on_finish)

    #############################################################
    ### NOTE: if radiobutton is "connect"ed to a function, then
    ###       invoking the radiobutton's 'set_active' will fire
    ###       the function.
    #############################################################
    def on_svcRadiobuttonChanged(self, button):
        #print 'on_svcRadiobuttonChanged TOP'
        if self.existSvcRadiobutton.get_active():
            ### 'Existing Service' was selected, so
            ###   * turn on 'Existing'
            ###   * turn off 'New'
            self.svcTreeview.set_sensitive(TRUE)
            self.newSvcNameEntry.set_sensitive(FALSE)
            self.newSvcIPAddressEntry.set_sensitive(FALSE)
        else:
            ### 'New Service' was selected, so
            ###   * turn on 'New'
            ###   * turn off 'Existing'
            self.newSvcNameEntry.set_sensitive(TRUE)
            self.newSvcIPAddressEntry.set_sensitive(TRUE)
            self.svcTreeview.set_sensitive(FALSE)
            self.newSvcNameEntry.grab_focus()
            if self.count_services == 0:
                self.existSvcRadiobutton.set_sensitive(FALSE)

    def deviceRadiobuttonChanged(self, new_or_existing):
        #print "deviceRadiobuttonChanged TOP now!"
        if new_or_existing == "existing":
            #print "  device existing"
            ### 'Existing Device' was selected, so
            ###   * turn on 'Existing'
            ###   * turn off 'New'
            self.deviceTreeview.set_sensitive(TRUE)
            self.newDeviceNameEntry.set_sensitive(FALSE)
            self.newDeviceMountpointEntry.set_sensitive(FALSE)
        else:
            #print "  device new"
            ### 'New Device' was selected, so
            ###   * turn on 'New'
            ###   * turn off 'Existing'
            self.newDeviceNameEntry.set_sensitive(TRUE)
            self.newDeviceMountpointEntry.set_sensitive(TRUE)
            self.deviceTreeview.set_sensitive(FALSE)

    def on_deviceRadiobuttonChanged(self, *args):
        #print "on_deviceRadiobuttonChanged TOP now!"
        if self.existDeviceRadiobutton.get_active():
            self.deviceRadiobuttonChanged(new_or_existing="existing")
        else:
            self.deviceRadiobuttonChanged(new_or_existing="new")

    def cancel(self,button):
        #print "nfsExportDruid 'cancel' now!"
        self.dlg.response(gtk.RESPONSE_CANCEL)
                                                                                
    def run(self):
        #print "nfsExportDruid 'run' now!"

        self.svcModel.clear()
        services_list = self.services_entity.get_list()
        self.count_services = 0
        for s in services_list:
            service_ipaddresses = s.get_service_ipaddresses()
            if len(service_ipaddresses) == 0:
                continue # don't load service if it doesn't have an ip
            self.count_services += 1
            service_name = s.get("name")
            iter = self.svcModel.append()
            self.svcModel.set(iter,
                              COLUMN_NAME, service_name,
                              COLUMN_DATA, s)

        ### initialize screen fields ###
        self.dirNameEntry.set_text("")
        self.clientNameEntry.set_text(CLIENT_NAME_DEFAULT)
        self.clientOptionsEntry.set_text(CLIENT_OPTIONS_DEFAULT)
        self.newSvcNameEntry.set_text("")
        self.newSvcIPAddressEntry.set_text("")
        self.newDeviceNameEntry.set_text("")
        self.newDeviceMountpointEntry.set_text("")

        self.first_time_service = TRUE
        self.first_time_device = TRUE

        self.dirNameEntry.grab_focus()
        self.druid.set_page(self.page1)
        self.druid.show_all()
        rc=self.dlg.run()
        self.dlg.hide()
        return rc

    def on_dirNamePage_next(self, *args):
        dirname = self.dirNameEntry.get_text()
        clientname = self.clientNameEntry.get_text()
        clientoptions = self.clientOptionsEntry.get_text()
        if dirname == "":
            errorMessage(_("You must enter a directory name"))
            return TRUE
        if dirname[0] != "/":
            errorMessage(_("Directory Name must start with a slash"))
            return TRUE
        #if not os.path.isdir(dirname):
        #    errorMessage(_("Name specified is not a directory"))
        #    return TRUE

        if clientname == "":
            clientname = CLIENT_NAME_DEFAULT
        if clientoptions == "":
            clientoptions = CLIENT_OPTIONS_DEFAULT

        self.dirname = dirname
        self.clientname = clientname
        self.clientoptions = clientoptions

        ### Verify that directory name isn't a dup of existing ###
        dup_found = FALSE
        for service in self.services_entity.get_list():
            service_id = service.get("id")
            devices = service.get_devices()
            for device in devices:
                device_id = device.get("id")
                nfsexports = device.get_nfsexports()
                for nfsexport in nfsexports:
                    if nfsexport.get("name") == self.dirname:
                        dup_found = TRUE
                        break
                if dup_found:
                    break
            if dup_found:
                break
        if dup_found:
            errorMessage(_("NFS Export directory already exists in cluster"))
            return TRUE

        ##################################################################
        ### NFS Export Directory Page done - now prep for Service Page ###
        ##################################################################
        if self.first_time_service:
            if self.count_services > 0:
                self.existSvcRadiobutton.set_active(TRUE)
            else:
                self.newSvcRadiobutton.set_active(TRUE)
            self.on_svcRadiobuttonChanged(None)

        if self.newSvcRadiobutton.get_active():
            self.newSvcNameEntry.grab_focus()

        self.first_time_service = FALSE

    def __validate_ipaddress(self, ip_to_validate):
        dup_found = FALSE
        for service in self.services_entity.get_list():
            ipaddress_entities = service.get_service_ipaddresses()
            for ipaddress_entity in ipaddress_entities:
                if ipaddress_entity.get("ipaddress") == ip_to_validate:
                    dup_found = TRUE
                    break
            if dup_found:
                break
        if dup_found:
            errorMessage(_("Service IP Address already exists in cluster"))
            return FALSE

        ### This whole section copied from service_ipaddress_module.py ###
        if hostInDNS(ip_to_validate) == None:
            errorMessage(_("Service IP Address not found in DNS"))
            return FALSE

        netmask = ""
        ipconvert = ipConvert()
        if not ipconvert.ipIsInSameNetAsHost(ip_to_validate, netmask):
            errorMessage(_("Service IP Address not on same network as this host"))
            return FALSE

        if cluPing(ip_to_validate):
            errorMessage(_("Service IP Address already in use on network"))
            return FALSE

        return TRUE # Passed validation

    def on_ServicePage_next(self, *args):
        #print "on_ServicePage_next TOP now!"
        ct_existing_devices_loaded = 0
        if self.existSvcRadiobutton.get_active():
            ### 'Existing Service' selected ###
            self.existing_service = TRUE
            self.new_service = FALSE

            selection = self.svcTreeview.get_selection()
            (model, iter) = selection.get_selected()
            if iter == None:
                errorMessage(_("When 'Existing Service' is selected, you must select one of the existing services"))
                return TRUE
            self.service_name = model.get_value(iter, COLUMN_NAME)
            self.service      = model.get_value(iter, COLUMN_DATA)

            service_id = self.service.get("id")
            clustat = cluStat()
            if clustat.serviceIsRunning(service_id):
                errorMessage(_("Service must be disabled before updates can be made"))
                return TRUE

            ### Set Device Model before display Device page ###
            self.deviceModel.clear()
            service = self.service
            devices_list = service.get_devices()
            for d in devices_list:
                device_name = d.get("name")
                mount_entity = d.get_mount()
                device_mountpoint = mount_entity.get("mountpoint")
                if not dirUnderMountpoint(self.dirname, device_mountpoint):
                    continue

                ### Export directory *IS* under the device mountpoint, so
                ### load this device into 'Existing Device' model
                ct_existing_devices_loaded += 1
                iter = self.deviceModel.append()
                self.deviceModel.set(iter,
                                     COL_DEVICE_NAME, device_name,
                                     COL_DEVICE_MOUNTPOINT, device_mountpoint,
                                     COL_DEVICE_DATA, d)
        else:
            ### 'New Service' selected ###
            self.existing_service = FALSE
            self.new_service = TRUE

            self.service_name = self.newSvcNameEntry.get_text()
            self.service_ipaddress = self.newSvcIPAddressEntry.get_text()
            if self.service_name == "":
                errorMessage(_("When 'New Service' is selected, you must enter a name for the new service"))
                return TRUE
            if self.service_ipaddress == "":
                errorMessage(_("When 'New Service' is selected, you must enter an ip address for the new service"))
                return TRUE

            ### Verify that the new service name does not already exist ###
            search_options = {"name" : self.service_name}
            if self.services_entity.service_found(search_options, service_being_validated=None):
                errorMessage(_("New Service name already exists"))
                return TRUE

            ### Validate IP Address ###
            passed_validation = self.__validate_ipaddress(self.service_ipaddress)
            if not passed_validation:
                return TRUE # return to druid

        ####################################################
        ### Service Page done - now prep for Device Page ###
        ####################################################
        #if self.first_time_device:
        #    pass

        if self.existing_service and ct_existing_devices_loaded > 0:
            ### Enable choice of existing 'device' from that
            ### existing service
            self.existDeviceRadiobutton.set_sensitive(TRUE)
            if self.first_time_device:
                self.existDeviceRadiobutton.set_active(TRUE)
                self.existDeviceRadiobutton.grab_focus()
            self.deviceRadiobuttonChanged(new_or_existing="existing")
        else:
            ### Disable existing device - only enable new device ###
            self.existDeviceRadiobutton.set_sensitive(FALSE)
            if self.first_time_device:
                self.newDeviceRadiobutton.set_active(TRUE)
            self.newDeviceNameEntry.grab_focus()
            self.deviceRadiobuttonChanged(new_or_existing="new")
    
        if self.existing_service:
            label = _("Existing Service '%s' was selected") % self.service_name
            self.deviceServiceLabel.set_text(label)
        else:
            label = _("New Service '%s' was specified") % self.service_name
            self.deviceServiceLabel.set_text(label)

        self.first_time_device = FALSE

    def on_devicePage_next(self, *args):
        self.first_time_device = FALSE
        if self.existDeviceRadiobutton.get_active():
            self.existing_device = TRUE
            self.new_device = FALSE

            selection = self.deviceTreeview.get_selection()
            (model, iter) = selection.get_selected()
            if iter == None:
                errorMessage(_("When 'Existing Device' is selected, you must select one of the existing devices"))
                return TRUE
            self.device_name       = model.get_value(iter, COL_DEVICE_NAME)
            self.device_mountpoint = model.get_value(iter, COL_DEVICE_MOUNTPOINT)
            self.device            = model.get_value(iter, COL_DEVICE_DATA)

            ### Verify that the export directory is *underneath* the mountpoint
            ###
            ### This check shouldn't even be necessary for *existing*
            ### devices, since an existing device presented in the
            ### selection list should only be presented in that list
            ### if the export directory *is* under the device mountpoint.
            if not dirUnderMountpoint(self.dirname, self.device_mountpoint):
                errorMessage(_("Export Directory must be underneath device mountpoint"))
                return TRUE
        else:
            ### 'New Device' selected ###
            self.existing_device = FALSE
            self.new_device = TRUE

            self.device_name       = self.newDeviceNameEntry.get_text()
            self.device_mountpoint = self.newDeviceMountpointEntry.get_text()

            if self.device_name == "":
                errorMessage(_("Special File name is required for a new device"))
                return TRUE
            try:
                mode = os.stat(self.device_name)[stat.ST_MODE]
            except OSError, e:
                errorMessage(_("Device Special File \"%s\" does not exist") % self.device_name)
                return TRUE
            if not stat.S_ISBLK(mode):
                errorMessage(_("Device Special File must be a block special device file"))
                return TRUE

            if self.device_mountpoint == "":
                errorMessage(_("Device Mountpoint is required for a new device"))
                return TRUE

            ### Verify that the export directory is *underneath* the mountpoint
            if not dirUnderMountpoint(self.dirname, self.device_mountpoint):
                errorMessage(_("Export Directory must be underneath device mountpoint"))
                return TRUE

            try:
                mode = os.stat(self.device_mountpoint)[stat.ST_MODE]
            except OSError, e:
                #errorMessage(_("Mount Point does not exist"))
                #return TRUE
                msg = _("Device Mountpoint '%s' does not exist, but will be created when the service is started.") % self.device_mountpoint
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                   return TRUE

            #if not stat.S_ISDIR(mode):
            #    errorMessage(_("Mount Point must be a directory"))
            #    return TRUE

            ### Verify that neither
            ###   * the new device name, nor
            ###   * the new device mountpoint
            ### already exists in the cluster.
            for service in self.services_entity.get_list():
               devices = service.get_devices()
               for device in devices:
                  if device.get("name") == self.device_name:
                      errorMessage(_("Device Special File already exists in cluster"))
                      return TRUE
                  mount = device.get_mount()
                  mountpoint = mount.get("mountpoint")
                  if mountpoint == self.device_mountpoint:
                      errorMessage(_("Device mountpoint already exists in cluster"))
                      return TRUE

    def on_finish(self, *args):
        #print "\nFinish now"
        #print "----------"
        #print "  Dir name     = [%s]" % self.dirname
        #print "  Svc name     = [%s]" % self.service_name
        #print "  Svc existing = [%s]" % self.existing_service
        #print "  Svc new      = [%s]" % self.new_service
        #print
        #print "  Dev name     = [%s]" % self.device_name
        #print "  Dev mount    = [%s]" % self.device_mountpoint
        if self.new_service:
            service = self.services_entity.add_service()
            service.set("name", self.service_name)
            service.change_action_to_modify() # for "validate"
            self.service = service

            service_ipaddress_entity = service.add_service_ipaddress()
            service_ipaddress_entity.set("ipaddress", self.service_ipaddress)

        if self.new_device:
            device = self.service.add_device()
            device.set("name", self.device_name)
            self.device = device

            mount = device.get_mount()
            mount.set("mountpoint", self.device_mountpoint)

        ### do 'df [nfsexport dir]' to get device ###
        #dir_df = cluDF(self.dirname)
        #filesystem = dir_df.get_filesystem()
        #mountpoint = dir_df.get_mountpoint()
        #print "\ndf filesystem=[%s]" % filesystem
        #print "\ndf mountpoint=[%s]" % mountpoint

        ### Now add the NFS Export Directory and default client ###
        nfsexport = self.device.add_nfsexport()
        nfsexport.set("name", self.dirname)

        client = nfsexport.add_client() # with defaults only
        client.set("name", self.clientname)
        client.set("options", self.clientoptions)

        self.dlg.response(gtk.RESPONSE_OK)
