#!/usr/bin/python

## powercontrollerDialog.py

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

##############################################################
# NOTE: 'port' is a valid option for 'Serial' and 'Network'
#       powercontrollers.
##############################################################
import gtk
import gtk.glade
import gettext
import os
import rpm

from clusterpkg.cluster_functions import OptionError
from clusterpkg.cluster_functions_gui import errorMessage, warningMessage

import clusterpkg.member_powercontroller_module
from clusterpkg.members_module import members

POWERCONTROLLER_INFO = clusterpkg.member_powercontroller_module.MY_INFO
VALID_SERIAL_TYPES = clusterpkg.member_powercontroller_module.VALID_SERIAL_TYPES
VALID_SERIAL_PORTS = clusterpkg.member_powercontroller_module.VALID_SERIAL_PORTS
VALID_NETWORK_TYPES = clusterpkg.member_powercontroller_module.VALID_NETWORK_TYPES

TRUE = (1==1)
FALSE = not TRUE

##
## I18N
##

class powercontrollerDialog:
    def __init__(self, glade_xml, powercontroller):
        self.glade_xml = glade_xml
        self.powercontroller = powercontroller

        self.dialog = glade_xml.get_widget('powercontrollerDialog')

        self.refresh = TRUE

        self.firsttime = TRUE

        serial_type_combo_widget = self.glade_xml.get_widget('pc_serial_type_combo')
        serial_type_combo_widget.set_popdown_strings(VALID_SERIAL_TYPES)

        serial_port_combo_widget = self.glade_xml.get_widget('pc_serial_port_combo')
        serial_port_combo_widget.set_popdown_strings(VALID_SERIAL_PORTS)

        network_type_combo_widget = self.glade_xml.get_widget('pc_network_type_combo')
        network_type_combo_widget.set_popdown_strings(VALID_NETWORK_TYPES)

        gulm_selected = self.glade_xml.get_widget('pc_gulm_radiobutton')
        if(not(self.check_for_gfs())):
          gulm_selected.hide()

    def check_for_gfs(self):
      ts = rpm.TransactionSet()
      mi = ts.dbMatch('name', 'GFS')
      if mi.count() > 0:
        return TRUE
      else: 
        return FALSE

    def on_cancelbutton_clicked(self, button):
        self.refresh = TRUE

    def on_okbutton_clicked(self, button):

        ### "get" the new values from the Gui widgets ###
        self.__serial_selected  = self.glade_xml.get_widget('pc_serial_radiobutton').get_active()
        self.__network_selected = self.glade_xml.get_widget('pc_network_radiobutton').get_active()
        self.__gulm_selected = self.glade_xml.get_widget('pc_gulm_radiobutton').get_active()

        serial_type = self.glade_xml.get_widget('pc_serial_type_combo-entry').get_text()
        serial_device = self.glade_xml.get_widget('pc_serial_device_entry').get_text()
        serial_port = self.glade_xml.get_widget('pc_serial_port_combo-entry').get_text()

        network_type = self.glade_xml.get_widget('pc_network_type_combo-entry').get_text()
        network_ipaddress = self.glade_xml.get_widget('pc_network_ipaddress_entry').get_text()
        network_port = self.glade_xml.get_widget('pc_network_port_entry').get_text()
        network_user = self.glade_xml.get_widget('pc_network_user_entry').get_text()
        network_password = self.glade_xml.get_widget('pc_network_password_entry').get_text()

        ### Now "set" the XML DOM option values.
        self.refresh = TRUE

        if(self.__serial_selected):
            type = serial_type
            network_ipaddress = ""     
            network_port = ""     
            network_user = ""     
            network_password = ""     
            port = serial_port
        elif(self.__network_selected):
            type = network_type
            serial_device = ""
            port = network_port

        else:
            type = "gulm-bridge"
            port = None 

        options = { "type"      : type,
                    "device"    : serial_device,
                    "ipaddress" : network_ipaddress,
                    "port"      : port,
                    "user"      : network_user,
                    "password"  : network_password }

        rc = gtk.RESPONSE_OK
        try:
            ### Invoke powercontroller 'validate' to validate all options
            self.powercontroller.validate(options)
        except OptionError, e:
            if e.getSeverity() == 'FATAL':
                errorMessage(e.getMessage())
                self.refresh = FALSE # Don't refresh - leave bad values
                                     # on screen.
                rc = gtk.RESPONSE_REJECT

            elif e.getSeverity() == 'WARNING':
                msg = _("%s\n\nPress 'OK' to Continue, or 'Cancel' to Exit.") % e.getMessage()
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                   self.refresh = FALSE     # Don't refresh - leave bad values
                                            # on screen.
                   rc = gtk.RESPONSE_REJECT # Need to differentiate this
                                            # 'Cancel' from the main Dialog
                                            # 'Cancel', so make this rc
                                            # gtk.RESPONSE_REJECT.

        if rc == gtk.RESPONSE_OK:
            ### Validation passed - now 'set' options ###
            if(self.__serial_selected):
                self.powercontroller.set("type", serial_type)
                self.powercontroller.set("device", serial_device)
                self.powercontroller.set("port", serial_port)
                ### If this was previously a 'Network' powercontroller, then
                ### remove the old network options.
                if self.powercontroller.get("ipaddress"):
                    self.powercontroller.delete_option("ipaddress")
                if self.powercontroller.get("user"):
                    self.powercontroller.delete_option("user")
                if self.powercontroller.get("password"):
                    self.powercontroller.delete_option("password")
            elif(self.__network_selected):
                ### Must be network powercontroller ###
                self.powercontroller.set("type", network_type)
                self.powercontroller.set("ipaddress", network_ipaddress)
                self.powercontroller.set("port", network_port)
                self.powercontroller.set("user", network_user)
                self.powercontroller.set("password", network_password)
                ### If this was previously a 'Serial' powercontroller, then
                ### remove the old serial options.
                if self.powercontroller.get("device"):
                    self.powercontroller.delete_option("device")
            else: #type gulm
                self.powercontroller.set("type", "gulm-bridge")
                if self.powercontroller.get("device"):
                    self.powercontroller.delete_option("device")
                if self.powercontroller.get("ipaddress"):
                    self.powercontroller.delete_option("ipaddress")
                if self.powercontroller.get("user"):
                    self.powercontroller.delete_option("user")
                if self.powercontroller.get("password"):
                    self.powercontroller.delete_option("password")
 
            self.powercontroller.change_action_to_modify()

        self.dialog.response(rc)
        return

    def get_all_members(self, powercontroller):
        ownerDocument = powercontroller.node.ownerDocument
        members_node = ownerDocument.getElementsByTagName("members")[0]
        members_entity = members(members_node)
        return members_entity.get_list()

    #####################################################################
    # get_other_member_name:
    #    Given a powercontroller, that is a child of a member, find
    #    the name of the *other* member.  This can only be done for
    #    serial powercontrollers that are valid only for 2-node
    #    clusters.
    #####################################################################
    def get_other_member_name(self, powercontroller):
        members_list = self.get_all_members(powercontroller)
        parent_member_name = powercontroller.parent.get("name")
        other_member_name = ""
        for m in members_list:
            member_name = m.get("name")
            if member_name != parent_member_name:
                other_member_name = member_name
                break
        return other_member_name

    def set_serial_or_network_radiobutton_sensitive(self, serial_selected, network_selected, gulm_selected):
        if(serial_selected):
            ### 'Serial' selected - 1. unprotect 'Serial' fields ###
            self.glade_xml.get_widget('pc_serial_type_combo-entry').set_sensitive(TRUE)
            self.glade_xml.get_widget('pc_serial_device_entry').set_sensitive(TRUE)
            self.glade_xml.get_widget('pc_serial_port_combo-entry').set_sensitive(TRUE)
            # set Serial Device *Owner* to name of *other* member -
            # *NOT* the name of the member that this powercontroller
            # is a child of.
            other_member_name = self.get_other_member_name(self.powercontroller)
            self.glade_xml.get_widget('pc_serial_owner_entry').set_text(other_member_name)

            ### 'Serial' selected - 2. protect 'Network' fields ###
            self.glade_xml.get_widget('pc_network_type_combo-entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_network_ipaddress_entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_network_port_entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_network_user_entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_network_password_entry').set_sensitive(FALSE)
        elif(network_selected):
            ### 'Network' selected - 1. unprotect 'Network' fields ###
            self.glade_xml.get_widget('pc_network_type_combo-entry').set_sensitive(TRUE)
            self.glade_xml.get_widget('pc_network_ipaddress_entry').set_sensitive(TRUE)
            self.glade_xml.get_widget('pc_network_port_entry').set_sensitive(TRUE)
            self.glade_xml.get_widget('pc_network_user_entry').set_sensitive(TRUE)
            self.glade_xml.get_widget('pc_network_password_entry').set_sensitive(TRUE)

            ### 'Network' selected - 2. protect 'Serial' fields ###
            self.glade_xml.get_widget('pc_serial_type_combo-entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_serial_device_entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_serial_port_combo-entry').set_sensitive(FALSE)

        else:  #gulm selected
            #Protect all other fields
            self.glade_xml.get_widget('pc_serial_type_combo-entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_network_type_combo-entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_network_ipaddress_entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_network_port_entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_network_user_entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_network_password_entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_serial_device_entry').set_sensitive(FALSE)
            self.glade_xml.get_widget('pc_serial_port_combo-entry').set_sensitive(FALSE)
             

    def on_serial_or_network_radiobutton_clicked(self, button):
        if self.firsttime:
            ### When loading screen for the 1st time, we invoke radio button
            ### set_active to set one of the radio buttons to "on", which fires
            ### this function.  When that happens(firsttime) just return.
            self.firsttime = FALSE
            return

        self.__serial_selected  = self.glade_xml.get_widget('pc_serial_radiobutton').get_active()
        self.__network_selected = self.glade_xml.get_widget('pc_network_radiobutton').get_active()
        self.__gulm_selected = self.glade_xml.get_widget('pc_gulm_radiobutton').get_active()


        self.set_serial_or_network_radiobutton_sensitive(self.__serial_selected, self.__network_selected,self.__gulm_selected)

    def run(self, powercontroller, action):
        self.powercontroller = powercontroller
        self.action          = action

        ### Now connect the buttons to the functions ###
        self.ok = self.glade_xml.get_widget('powercontrollerDialog_okbutton').connect("clicked", self.on_okbutton_clicked)
        self.cancel = self.glade_xml.get_widget('powercontrollerDialog_cancelbutton').connect("clicked", self.on_cancelbutton_clicked)

        self.serial_radiobutton = self.glade_xml.get_widget('pc_serial_radiobutton').connect("clicked", self.on_serial_or_network_radiobutton_clicked)
        self.network_radiobutton = self.glade_xml.get_widget('pc_network_radiobutton').connect("clicked", self.on_serial_or_network_radiobutton_clicked)
        self.gulm_radiobutton = self.glade_xml.get_widget('pc_gulm_radiobutton').connect("clicked", self.on_serial_or_network_radiobutton_clicked)

        ### Load screen widgets with data from XML DOM for selected
        ### powercontroller.
        if self.refresh: 
            members_list = self.get_all_members(powercontroller)
            count_members = len(members_list)
            if action == "add":
                if count_members <= 2:
                    ### On "add", if 2 members in cluster, default to 'Serial'
                    serial_selected = TRUE
                    network_selected = FALSE
                    gulm_selected = FALSE
                    self.set_serial_or_network_radiobutton_sensitive(serial_selected, network_selected, gulm_selected)

                    self.glade_xml.get_widget('pc_serial_radiobutton').set_active(TRUE)
                    self.glade_xml.get_widget('pc_network_radiobutton').set_active(FALSE)
                    self.glade_xml.get_widget('pc_gulm_radiobutton').set_active(FALSE)

                    self.glade_xml.get_widget('pc_serial_type_combo-entry').set_text("")

                    self.glade_xml.get_widget('pc_serial_device_entry').set_text("")

                    self.glade_xml.get_widget('pc_serial_port_combo-entry').set_text("")

                    # set Serial Device *Owner* to name of *other* member -
                    # *NOT* the name of the member that this powercontroller
                    # is a child of.
                    other_member_name = self.get_other_member_name(powercontroller)
                    self.glade_xml.get_widget('pc_serial_owner_entry').set_text(other_member_name)

                    self.glade_xml.get_widget('pc_network_type_combo-entry').set_text("")
                    self.glade_xml.get_widget('pc_network_ipaddress_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_port_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_user_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_password_entry').set_text("")
                else:
                    ### 'add', but count_members >2, so default to 'Network'
                    serial_selected = FALSE
                    network_selected = TRUE
                    gulm_selected = FALSE 
                    self.set_serial_or_network_radiobutton_sensitive(serial_selected, network_selected, gulm_selected)

                    self.glade_xml.get_widget('pc_serial_radiobutton').set_sensitive(FALSE)
                    self.glade_xml.get_widget('pc_serial_radiobutton').set_active(FALSE)
                    self.glade_xml.get_widget('pc_gulm_radiobutton').set_sensitive(TRUE)
                    self.glade_xml.get_widget('pc_gulm_radiobutton').set_active(TRUE)
                    self.glade_xml.get_widget('pc_network_radiobutton').set_active(TRUE)

                    self.glade_xml.get_widget('pc_serial_type_combo-entry').set_text("")
                    self.glade_xml.get_widget('pc_serial_device_entry').set_text("")
                    self.glade_xml.get_widget('pc_serial_port_combo-entry').set_text("")
                    self.glade_xml.get_widget('pc_serial_owner_entry').set_text("")

                    self.glade_xml.get_widget('pc_network_type_combo-entry').set_text("")
                    self.glade_xml.get_widget('pc_network_ipaddress_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_port_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_user_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_password_entry').set_text("")

            else:
                ########################
                ### Must be "modify" ###
                ########################
                serial_selected = TRUE
                network_selected = FALSE
                gulm_selected = FALSE
                self.set_serial_or_network_radiobutton_sensitive(serial_selected, network_selected,gulm_selected)
                self.type = powercontroller.get("type")
                if self.type in VALID_SERIAL_TYPES:
                    # set Serial radiobutton
                    serial_selected = TRUE
                    network_selected = FALSE
                    gulm_selected = FALSE
                    self.set_serial_or_network_radiobutton_sensitive(serial_selected, network_selected,gulm_selected)
                    self.glade_xml.get_widget('pc_serial_radiobutton').set_active(TRUE)
                    self.glade_xml.get_widget('pc_network_radiobutton').set_active(FALSE)

                    # set Serial Type combo box
                    self.glade_xml.get_widget('pc_serial_type_combo-entry').set_text(self.type)

                    # set Serial Device
                    self.device = powercontroller.get("device")
                    self.glade_xml.get_widget('pc_serial_device_entry').set_text(self.device)

                    # set Serial Port combo box
                    self.port = powercontroller.get("port")
                    self.glade_xml.get_widget('pc_serial_port_combo-entry').set_text(self.port)

                    # set Serial Device *Owner* to name of *other* member -
                    # *NOT* the name of the member that this powercontroller
                    # is a child of.
                    other_member_name = self.get_other_member_name(powercontroller)
                    self.glade_xml.get_widget('pc_serial_owner_entry').set_text(other_member_name)

                    # set Network options to null
                    self.glade_xml.get_widget('pc_network_type_combo-entry').set_text("")
                    self.glade_xml.get_widget('pc_network_ipaddress_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_port_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_user_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_password_entry').set_text("")

                elif self.type in VALID_NETWORK_TYPES:
                    # User must have chosen "Network" radiobutton
                    serial_selected = FALSE
                    network_selected = TRUE
                    gulm_selected = FALSE
                    self.set_serial_or_network_radiobutton_sensitive(serial_selected, network_selected,gulm_selected)

                    self.glade_xml.get_widget('pc_serial_radiobutton').set_active(FALSE)
                    self.glade_xml.get_widget('pc_network_radiobutton').set_active(TRUE)

                    # set Network Type combo box
                    self.glade_xml.get_widget('pc_network_type_combo-entry').set_text(self.type)

                    self.ipaddress = powercontroller.get("ipaddress")
                    self.glade_xml.get_widget('pc_network_ipaddress_entry').set_text(self.ipaddress)
                    self.port = powercontroller.get("port")
                    self.glade_xml.get_widget('pc_network_port_entry').set_text(self.port)
                    self.user = powercontroller.get("user")
                    self.glade_xml.get_widget('pc_network_user_entry').set_text(self.user)
                    self.password = powercontroller.get("password")
                    self.glade_xml.get_widget('pc_network_password_entry').set_text(self.password)

                    # set Serial options to null
                    self.glade_xml.get_widget('pc_serial_type_combo-entry').set_text("")
                    self.glade_xml.get_widget('pc_serial_device_entry').set_text("")
                    self.glade_xml.get_widget('pc_serial_port_combo-entry').set_text("")
                    self.glade_xml.get_widget('pc_serial_owner_entry').set_text("")

                else: #type is gulm stonith driver 
                    #set serial and network options to null
                    serial_selected = FALSE
                    network_selected = FALSE
                    gulm_selected = TRUE
                    self.set_serial_or_network_radiobutton_sensitive(serial_selected, network_selected,gulm_selected)
                    # set Serial options to null
                    self.glade_xml.get_widget('pc_serial_type_combo-entry').set_text("")
                    self.glade_xml.get_widget('pc_serial_device_entry').set_text("")
                    self.glade_xml.get_widget('pc_serial_port_combo-entry').set_text("")
                    self.glade_xml.get_widget('pc_serial_owner_entry').set_text("")
                    # set Network options to null
                    self.glade_xml.get_widget('pc_network_type_combo-entry').set_text("")
                    self.glade_xml.get_widget('pc_network_ipaddress_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_port_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_user_entry').set_text("")
                    self.glade_xml.get_widget('pc_network_password_entry').set_text("")

        rc = self.dialog.run()
        self.glade_xml.get_widget('powercontrollerDialog_okbutton').disconnect(self.ok)
        self.glade_xml.get_widget('powercontrollerDialog_cancelbutton').disconnect(self.cancel)
        self.glade_xml.get_widget('pc_serial_radiobutton').disconnect(self.serial_radiobutton)
        self.glade_xml.get_widget('pc_network_radiobutton').disconnect(self.network_radiobutton)
        return rc

    def hide(self):
        return self.dialog.hide()

class powercontrollerItem:
    def __init__(self, parent, powercontroller):
        ### parent is a 'memberItem'
        self.parent    = parent
        self.glade_xml = parent.glade_xml
        self.tree_view = parent.tree_view
        self.model     = parent.tree_view.get_model()
        self.columns   = parent.columns
        self.powercontroller = powercontroller

        self.dialog = powercontrollerDialog(self.glade_xml, powercontroller)

    def GUI_load_row_columns(self, powercontroller_item, iter):
        pc_item = powercontroller_item.powercontroller
        type = pc_item.get("type")
        if type in VALID_SERIAL_TYPES:
            device = pc_item.get("device")
            row_display_string = _("%s: device=%s <serial powercontroller>") % (type, device)
        elif type in VALID_NETWORK_TYPES:
            ipaddress = pc_item.get("ipaddress")
            port      = pc_item.get("port")
            user      = pc_item.get("user")
            password  = pc_item.get("password")
            row_display_string = _("%s: ip=%s, port=%s <network powercontroller>") % (type, ipaddress, port)
        elif(type == "gulm-bridge"):
            row_display_string = _("%s: <GULM STONITH Bridge>") % (type)
        else:
            row_display_string = "None"

        self.model.set(iter,
                       self.columns['COLUMN_KEY'], row_display_string,
                       self.columns['COLUMN_DATA'], powercontroller_item)

    def GUI_tree_load_powercontroller(self, iter):
        new_iter = self.model.append(iter)
        powercontroller_item = self
        self.GUI_load_row_columns(powercontroller_item, new_iter)

    def get_type(self):
        return "powercontroller"

    def get_value_of_key_field(self):
        type = self.powercontroller.get("type")
        value_of_key_field = ""
        if type in VALID_SERIAL_TYPES:
            device = self.powercontroller.get("device")
            value_of_key_field = type + " with device " + device
        elif type in VALID_NETWORK_TYPES:
            ipaddress = self.powercontroller.get("ipaddress")
            port      = self.powercontroller.get("port")
            user      = self.powercontroller.get("user")
            value_of_key_field = "%s with ip=%s, port=%s" % (type, ipaddress, port)
        elif type == "gulm-bridge":
          value_of_key_field = type

        return value_of_key_field

    def get(self, option):
        return self.powercontroller.get(option)

    def __modify(self, iter, action="modify"):
        if action == "modify":
            powercontroller = self.powercontroller
            powercontroller_item = self
        else:
            ### Must be an "add" ###
            member_item = self.parent
            member_entity = member_item.member

            powercontroller = member_entity.add_powercontroller()
            powercontroller_item = powercontrollerItem(self.parent, powercontroller)

        while 1:
            rc = self.dialog.run(powercontroller, action)
            if rc == gtk.RESPONSE_OK:
                if action == "add":
                    ### model.append needs the "iter" of the *parent* of the
                    ### thing that is being appended.
                    parent_iter = self.model.iter_parent(iter)
                    iter = self.model.append(parent_iter)

                self.GUI_load_row_columns(powercontroller_item, iter)

            elif rc == gtk.RESPONSE_CANCEL:
                if action == "add":
                    member_entity.delete_powercontroller_by_id(powercontroller.get("id"))
            else:
                ### rc was gtk.RESPONSE_REJECT:
                ###   1. User got a FATAL error and clicked 'OK' (only option)
                ###      so display same dialog again,
                ###        OR
                ###   2. User got a WARNING and clicked 'Cancel' to go back
                ###      and re-enter.
                continue
            break

        self.dialog.hide()

    def add(self, iter):
        self.__modify(iter, "add")

    def properties(self, iter):
        self.__modify(iter)

    def delete(self):
        #member_item = self.parent
        #member_entity = member_item.member
        #member_entity.delete_powercontroller_by_id(self.powercontroller.get("id"))
        member_item = self.parent
        member_entity = member_item.member
        try:
            member_entity.delete_powercontroller_by_id(self.powercontroller.get("id"))
        except OptionError, e:
            errorMessage(e.getMessage())
            return FALSE # delete failed
        return TRUE      # delete succeeded
