#!/usr/bin/python

import sys
my_cmdline_argv = sys.argv # Do this 1st, to prevent --name from being stripped
                           # by 'import gtk' in cluster_functions.
import getopt, os, socket
import string
sys.path.append('/usr/share/redhat-config-cluster/configure')
from clusterpkg.cluconfig_module    import cluconfig
from clusterpkg import cluster_functions
from clusterpkg.cluster_functions   import OptionError, \
                                           hostInDNS, \
                                           initialize_shared_state, \
                                           getStatusOutput
from clusterpkg.util_lock_module    import lockFileException
from clusterpkg.util_smbconf_module import getAbsShareFilename

TRUE = (1==1)
FALSE = not TRUE

### Global variables ###
try:
    ### Parse the cluster.xml into DOM nodes, and lock the cluster.xml
    clu = cluconfig(lock=TRUE)
except lockFileException, e:
    ### Another process already has /etc/cluster.xml locked
    print e.getMessage()
    sys.exit(1)

### Check Shared State ###
(status, output) = getStatusOutput("shutil -p /cluster/header")
if status == 0:
    ### Shared Storage was previously initialized and is responding
    ### fine.
    pass
else:
    ### Shared Storage is NOT responding - maybe it hasn't been
    ### initialized yet.
    #if clu.created_config:
    try:
        initialize_shared_state(clu)
    except OptionError, e:
        print e.getMessage()
        if clu.created_config:
            ### Since shared storage initialization failed, remove default
            ### config file that was created.
            try:
                os.remove(clu.file)
            except OSError, e:
                pass
            sys.exit(1)

    ### Shared Storage was initialized successfully ###

    msg = ""
    if clu.created_config:
        print _("\n*** Configuration file %s was created.") % clu.file

    print _("""\n*** Shared Storage was initialized successfully.

====================================================""")
    
clu_saved = cluconfig(lock=FALSE)

cmdline_args = []
args_dict = {}

def modify_cluster(): ### attributes for the cluster as a whole
    if args_dict.has_key("config_viewnumber"):
        print _("Error: config_viewnumber cannot be changed")
        sys.exit(1)
    cluster = clu.getcluster()
    try:
        ct_options_updated = cluster.modify_options(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)
    if ct_options_updated == 0:
        cluster.out()

def modify_sharedstate(): ### attributes for the cluster's Shared State
    #print "redhat-config-cluster-cmd modify_sharedstate TOP!"
    from clusterpkg.sharedstate_module import DRIVERS
    sharedstate = clu.getsharedstate()
    old_type = sharedstate.get("type")
    try:
        ct_options_updated = sharedstate.modify_options(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)
    if ct_options_updated == 0:
        sharedstate.out()
    else:
        ### See if "type" changed, since each type has different options.
        ### If type changed, then delete options of other type(s).
        if args_dict.has_key("type"):
            new_type = args_dict["type"]
            if old_type != new_type:
                if new_type == "raw":
                    if not args_dict.has_key("driver"):
                        sharedstate.set("driver", DRIVERS["raw"])
                    ### New type is 'raw' - delete existing 'sfs' options ###
                    if sharedstate.get("sfspath"):
                        sharedstate.delete_option("sfspath")
                else:
                    if not args_dict.has_key("driver"):
                        sharedstate.set("driver", DRIVERS["sfs"])
                    ## New type is 'sfs' - delete existing 'raw' options ###
                    if sharedstate.get("rawprimary"):
                        sharedstate.delete_option("rawprimary")
                    if sharedstate.get("rawshadow"):
                        sharedstate.delete_option("rawshadow")

def modify_clumembd():
    clumembd = clu.getclumembd()
    if args_dict.has_key("multicast_ipaddress") and \
       args_dict['multicast_ipaddress'] != "":
        if not args_dict.has_key("multicast"):
            cmdline_args.append("--multicast=yes")
    if args_dict.has_key("multicast") and args_dict["multicast"] == "no":
        if not args_dict.has_key("multicast_ipaddress"):
            cmdline_args.append("--multicast_ipaddress=")
    try:
        ct_options_updated = clumembd.modify_options(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)
    if ct_options_updated == 0:
        clumembd.out()

#def modify_clupowerd():
#    clupowerd = clu.getclupowerd()
#    try:
#        ct_options_updated = clupowerd.modify_options(cmdline_args)
#    except OptionError, e:
#        print _("Error:"), e.getMessage()
#        sys.exit(2)
#    if ct_options_updated == 0:
#        clupowerd.out()

def modify_cluquorumd():
    cluquorumd = clu.getcluquorumd()
    try:
        ct_options_updated = cluquorumd.modify_options(cmdline_args)
    except OptionError, e:
        if e.getSeverity() == "WARNING":
            pass
        else:
            print _("Error:"), e.getMessage()
            sys.exit(2)

    if ct_options_updated == 0:
        cluquorumd.out()
        
def modify_clurmtabd():
    clurmtabd = clu.getclurmtabd()
    try:
        ct_options_updated = clurmtabd.modify_options(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)
    if ct_options_updated == 0:
        clurmtabd.out()

def modify_clusvcmgrd():
    clusvcmgrd = clu.getclusvcmgrd()
    try:
        ct_options_updated = clusvcmgrd.modify_options(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)
    if ct_options_updated == 0:
        clusvcmgrd.out()

def modify_clulockd():
    clulockd = clu.getclulockd()
    try:
        ct_options_updated = clulockd.modify_options(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)
    if ct_options_updated == 0:
        clulockd.out()

############################################################################

def add_member():
    #print "redhat-config-cluster-cmd add_member() NOW"
    members = clu.getmembers()
    try:
        #print "redhat-config-cluster-cmd add_member() NOW"
        members.add_member(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)

def del_member():
    members = clu.getmembers()
    if not args_dict.has_key("member"):
        print _("Error: Option \"%s\" requires \"%s\"") % ("--del_member", "--member")
        sys.exit(2)

    try:
        members.delete_member(args_dict['member'], cmdline_args)
    except OptionError, e:
        error_msg = e.getMessage()
        if string.find(error_msg, 'does not exist') != -1:
           print _("Error: Member '%s' does not exist in cluster") % args_dict["member"]
        else:
           print _("Error:"), e.getMessage()
        sys.exit(2)

def list_members():
    clu.getmembers().out()

def modify_member():
    #print "redhat-config-cluster-cmd modify_member TOP"
    member_name = args_dict['member']
    members_entity = clu.getmembers()
    try:
        member = members_entity.get_member(member_name)
    except OptionError, e:
        #print _("Error:"), e.getMessage()
        print _("Error: Member '%s' does not exist in cluster") % member_name
        sys.exit(2)

    ###########################################################
    ### Check for Main level children - if child exists, call
    ### the appropriate function to handle that child
    ###########################################################
    if args_dict.has_key("add_powercontroller"):
        add_powercontroller(member)
        return

    if args_dict.has_key("del_powercontroller"):
        del_powercontroller(member)
        return

    if args_dict.has_key("powercontrollers"):
        list_powercontrollers(member)
        return

    if args_dict.has_key("powercontroller"):
        modify_powercontroller(member)
        return

    # # # # # # # # # # # # # # # # # # # # # # # # # # # #

    try:
        ct_options_updated = member.modify_options(cmdline_args)
    except OptionError, e:
        #print "Error: Option \"--%s\" is not valid with \"--member\"." % e.getOptionName()
        print _("Error:"), e.getMessage()
        sys.exit(2)

    if ct_options_updated == 0:
        member.out()

    #############################
    ### end def modify_member ###
    #############################

##########################################################################

def add_powercontroller(member):
    try:
        member.add_powercontroller(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)

def del_powercontroller(member):
    if not args_dict.has_key("powercontroller"):
        print _("Error: Option \"%s\" requires \"%s\"") % ("--del_powercontroller", "--powercontroller")
        sys.exit(2)

    try:
        member.delete_powercontroller(args_dict['powercontroller'], cmdline_args)
    except OptionError, e:
        #print _("Error:"), e.getMessage()
        print _("Error: Powercontroller '%s' does not exist in Member '%s'") % (args_dict["powercontroller"], member.get("name"))
        sys.exit(2)

def list_powercontrollers(member):
    pc_entityList = member.get_powercontrollers()
    for p in pc_entityList:
        p.out()

def modify_powercontroller(member):
    pc_key = args_dict['powercontroller']
    tokens = string.split(pc_key, ',')
    ct_tokens = len(tokens)
    if ct_tokens == 1:
        ### test to see if valid ip address ###
        is_network = FALSE
        if pc_key == "255.255.255.255":
           is_network = TRUE
        else:
           try:
              socket.inet_aton(pc_key)
              is_network = TRUE
           except socket.error, e:
              ### Valid dns name will fall in here, so see if that's
              ### what it is.
              ip = hostInDNS(pc_key)
              if ip != None:
                 is_network = TRUE
              else:
                 is_network = FALSE

        if is_network:
           ### User passed in a 'powercontroller=' with a value containing
           ### a valid ip address, but no port.
           print _("Error: --powercontroller value is missing a port number - value must be in format '--powercontroller=ip,port'")
           sys.exit(2)

        ### Must be a serial device ###
        #print "redhat-config-cluster-cmd modify_powercontroller: serial - device"
        general_type = "serial"
        device = tokens[0]
    elif ct_tokens == 2:
        ### Must be a network powercontroller ip address, port ###
        #print "redhat-config-cluster-cmd modify_powercontroller: network - ipaddress,port"
        general_type = "network"
        ipaddress = tokens[0]
        port = tokens[1]

    try:
        if general_type == "serial":
            powercontroller = member.get_serial_powercontroller(device)
        else:
            search_options = { "ipaddress" : ipaddress,
                               "port"      : port }
            powercontroller = member.get_network_powercontroller(search_options)
    except OptionError, e:
        #print _("Error: Powercontroller \"%s\" does not exist in cluster") % e.getOptionValue()
        print _("Error:"), e.getMessage()
        sys.exit(2)

    try:
        ct_options_updated = powercontroller.modify_options(cmdline_args)
    except OptionError, e:
        if e.getSeverity() == "WARNING":
            pass
        else:
            print _("Error:"), e.getMessage()
            sys.exit(2)

    if args_dict.has_key("type"):
        new_type = args_dict['type']
        new_general_type = powercontroller.get_general_type(new_type)
    else:
        new_general_type = general_type

    #print "redhat-config-cluster-cmd 4444444444444444444444"
    #print "  general_type =", general_type
    #print "  new_general_type =", new_general_type
    if ct_options_updated == 0:
        powercontroller.out()
    else:
        ### If user is changing 'type' from Serial to Network, or
        ### Network to Serial, then remove the old fields.
        if new_general_type != general_type:
            if general_type == "network":
                ### If this was previously a 'Network' powercontroller, then
                ### remove the network options.
                if powercontroller.get("ipaddress"):
                    powercontroller.delete_option("ipaddress")
                if powercontroller.get("port"):
                    powercontroller.delete_option("port")
            else:
                ### If this was previously a 'Serial' powercontroller, then
                ### remove the serial options.
                if powercontroller.get("device"):
                    powercontroller.delete_option("device")

##########################################################################

def add_failoverdomain():
    #print "redhat-config-cluster-cmd add_failoverdomain() NOW"
    failoverdomains = clu.getfailoverdomains()
    try:
        #print "redhat-config-cluster-cmd add_failoverdomain() NOW"
        failoverdomains.add_failoverdomain(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)

def del_failoverdomain():
    failoverdomains = clu.getfailoverdomains()
    if not args_dict.has_key("failoverdomain"):
        print _("Error: Option \"%s\" requires \"%s\"") % ("--del_failoverdomain", "--failoverdomain")
        sys.exit(2)

    try:
        failoverdomains.delete_failoverdomain(args_dict['failoverdomain'], cmdline_args)
    except OptionError, e:
        error_msg = e.getMessage()
        if string.find(error_msg, 'does not exist') != -1:
           print _("Error: Failover Domain '%s' does not exist in cluster") % args_dict["failoverdomain"]
        else:
           print _("Error:"), e.getMessage()
        sys.exit(2)

def list_failoverdomains():
    clu.getfailoverdomains().out()

def modify_failoverdomain():
    failoverdomain_name = args_dict['failoverdomain']
    failoverdomains_entity = clu.getfailoverdomains()
    try:
        failoverdomain = failoverdomains_entity.get_failoverdomain(failoverdomain_name)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)

    ###########################################################
    ### Check for Main level children - if child exists, call
    ### the appropriate function to handle that child
    ###########################################################
    if args_dict.has_key("add_failoverdomainnode"):
        add_failoverdomainnode(failoverdomain)
        return

    if args_dict.has_key("del_failoverdomainnode"):
        del_failoverdomainnode(failoverdomain)
        return

    if args_dict.has_key("failoverdomainnodes"):
        list_failoverdomainnodes(failoverdomain)
        return

    if args_dict.has_key("failoverdomainnode"):
        modify_failoverdomainnode(failoverdomain)
        return

    # # # # # # # # # # # # # # # # # # # # # # # # # # # #

    try:
        ct_options_updated = failoverdomain.modify_options(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)

    if ct_options_updated == 0:
        failoverdomain.out()

    #####################################
    ### end def modify_failoverdomain ###
    #####################################

##########################################################################

def reorder_failoverdomainnodes(failoverdomain):
    ### Since we are
    ###   1. Deleting existing nodes for this failover domain, and *then*
    ###   2. Adding all existing nodes and new node in the right place
    ###
    ### all id's will be assigned starting at 0 and incrementing by 1.

    nodes = {}
    for node in failoverdomain.get_failoverdomainnodes():
        nodes[node.get("id")] = node.get("name")
        failoverdomain.delete_failoverdomainnode_by_id(node.get("id"))

    ### sort existing node id's ###
    sorted_nodes = nodes.items()
    sorted_nodes.sort()

    ### renumber id's starting at 0 and incrementing by 1, since
    ### coming in they might not start at 0, and there might be
    ### gaps in the id's. Insert reordered node in correct spot.
    for id, node_name in sorted_nodes:
        new_node = failoverdomain.add_failoverdomainnode()
        new_node.set("name", node_name)

def add_failoverdomainnode(failoverdomain):
    try:
        failoverdomain.add_failoverdomainnode(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)

def del_failoverdomainnode(failoverdomain):
    if not args_dict.has_key("failoverdomainnode"):
        print _("Error: Option \"%s\" requires \"%s\"") % ("--del_failoverdomainnode", "--failoverdomainnode")
        sys.exit(2)

    try:
        failoverdomain.delete_failoverdomainnode(args_dict['failoverdomainnode'], cmdline_args)
    except OptionError, e:
        #print _("Error:"), e.getMessage()
        print _("Error: Failover Domain Node '%s' does not exist in Failover Domain '%s'") % (args_dict["failoverdomainnode"], failoverdomain.get("name"))
        sys.exit(2)
    reorder_failoverdomainnodes(failoverdomain)

def list_failoverdomainnodes(failoverdomain):
    failoverdomain.out(print_domain_nodes_only=TRUE)

###########################################################################
# reorder_failoverdomainnode:
# ===========================
#   Want to be certain that new failoverdomainnodes are added *last*,
#   since order *IS* meaningful here.  Problem is that there may be
#   gaps in the "id" sequence since nodes may have been previously
#   deleted.
#
#   Since we are
#     * First deleting existing nodes for this failover domain, and *then*
#     * Second, adding all existing nodes and new node in the right place
#
#   all id's will be assigned by ClusterList.py starting at 0 and
#   incrementing by 1.
###########################################################################
def reorder_failoverdomainnode(failoverdomain, failoverdomainnode, order):
    ### Since we are
    ###   1. Deleting existing nodes for this failover domain, and *then*
    ###   2. Adding all existing nodes and new node in the right place
    ###
    ### all id's will be assigned starting at 0 and incrementing by 1.

    reorder_node_name = failoverdomainnode.get("name")
    reorder_node_order = int(order)

    nodes = {}
    for node in failoverdomain.get_failoverdomainnodes():
        nodes[node.get("id")] = node.get("name")
        failoverdomain.delete_failoverdomainnode_by_id(node.get("id"))

    ### sort existing node id's ###
    sorted_nodes = nodes.items()
    sorted_nodes.sort()

    ### renumber id's starting at 0 and incrementing by 1, since
    ### coming in they might not start at 0, and there might be
    ### gaps in the id's. Insert reordered node in correct spot.
    new_nodes = []
    ct = 0
    reorder_node_inserted = FALSE
    for id, node_name in sorted_nodes:
        if node_name == reorder_node_name:
            continue

        ct = ct + 1 ### increment count for *existing* node being added
        if ct >= reorder_node_order:
            if not reorder_node_inserted:
                reorder_node_inserted = TRUE
                new_nodes.append(reorder_node_name)
                ct = ct + 1 ### increment for *reordered* node being added
        new_nodes.append(node_name)
    if not reorder_node_inserted:
        new_nodes.append(reorder_node_name)
    #print "new_nodes =", new_nodes

    for node_name in new_nodes:
        new_node = failoverdomain.add_failoverdomainnode()
        new_node.set("name", node_name)

def modify_failoverdomainnode(failoverdomain):
    #print "redhat-config-cluster-cmd modify_failoverdomainnode TOP!"
    node_name = args_dict['failoverdomainnode']
    try:
        failoverdomainnode = failoverdomain.get_failoverdomainnode(node_name)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)

    #print "redhat-config-cluster-cmd modify_failoverdomainnode 1111"
    reorder = FALSE
    order = "0"
    if args_dict.has_key("order"):
        reorder = TRUE
        order = args_dict["order"]
        ### Remove(del) the "order" option, since that is NOT an actual
        ### failoverdomainnode option - it only exists to tell us
        ### what new position the user wants a node to take.
        search_for_arg = "--order=%s" % order
        ct = 0
        for arg in cmdline_args:
            if arg == search_for_arg:
                del cmdline_args[ct]
            ct = ct + 1
            
    try:
        ct_options_updated = failoverdomainnode.modify_options(cmdline_args)
    except OptionError, e:
        if e.getSeverity() == "WARNING":
            pass
        else:
            print _("Error:"), e.getMessage()
            sys.exit(2)

    if reorder:
        reorder_failoverdomainnode(failoverdomain, failoverdomainnode, order)
        ct_options_updated += 1
        
    if ct_options_updated == 0:
        failoverdomainnode.out()

##########################################################################

def add_service():
    services = clu.getservices()
    try:
        services.add_service(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)

def del_service():
    services = clu.getservices()
    if not args_dict.has_key("service"):
        print _("Error: Option \"%s\" requires \"%s\"") % ("--del_service", "--service")
        sys.exit(2)

    try:
        services.delete_service(args_dict['service'], cmdline_args)
    except OptionError, e:
        #print _("Error:"), e.getMessage()
        print _("Error: Service '%s' does not exist in cluster") % args_dict["service"]
        sys.exit(2)

def list_services():
    clu.getservices().out()

def modify_service():
    service_name = args_dict['service']
    try:
        service = clu.getservices().get_service(service_name)
    except OptionError, e:
        print _("Error: Service '%s' does not exist in cluster") % service_name
        sys.exit(2)

    ###########################################################
    ### Check for Main level children - if child exists, call
    ### the appropriate function to handle that child
    ###########################################################
    if args_dict.has_key("add_device"):
        add_device(service)
        return

    if args_dict.has_key("del_device"):
        del_device(service)
        return

    if args_dict.has_key("devices"):
        list_devices(service)
        return

    if args_dict.has_key("device"):
        modify_device(service)
        return

    # # # # # # # # # # # # # # # # # # # # # # # # # # # #

    if args_dict.has_key("add_service_ipaddress"):
        add_service_ipaddress(service)
        return

    if args_dict.has_key("del_service_ipaddress"):
        del_service_ipaddress(service)
        return

    if args_dict.has_key("service_ipaddresses"):
        list_service_ipaddresses(service)
        return

    if args_dict.has_key("service_ipaddress"):
        modify_service_ipaddress(service)
        return

    # # # # # # # # # # # # # # # # # # # # # # # # # # # #

    try:
        ct_options_updated = service.modify_options(cmdline_args)
    except OptionError, e:
        ### This should catch options that are legal for some part
        ### of the cluster, but that are *illegal* for "--service"
        print _("Error:"), e.getMessage()
        sys.exit(2)

    if ct_options_updated == 0:
        service.out()

    ###############################
    ### end def modify_service ###
    ###############################

##########################################################################

def add_device(service):
    try:
        device = service.add_device(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)

    if args_dict.has_key("sharename"):
        sharename = args_dict["sharename"]
        if sharename in ("", "None", "none"):
            sharename = ""
        if sharename != "":
            (sharename, ipaddresses, mountpoint, action) = device.smbconf.writeSambaShareFile()

def del_device(service):
    if not args_dict.has_key("device"):
        print _("Error: Option \"%s\" requires \"%s\"") % ("--del_device", "--device")
        sys.exit(2)

    try:
        service.delete_device(args_dict['device'], cmdline_args)
    except OptionError, e:
        #print _("Error:"), e.getMessage()
        print _("Error: Device '%s' does not exist in Service '%s'") % (args_dict["device"], service.get("name"))
        sys.exit(2)

def list_devices(service):
    device_entityList = service.get_devices()
    for d in device_entityList:
        d.out()

def modify_device(service):
    device_name = args_dict['device']
    try:
        device = service.get_device(device_name)
    except OptionError, e:
        print _("Error: Device \"%s\" does not exist for Service \"%s\"") \
              % (device_name, service.get("name"))
        sys.exit(2)

    ###########################################################
    ### Check for Main level children - if child exists, call
    ### the appropriate function to handle that child
    ###
    ### Children of "device":
    ###   * mount
    ###   * nfsexports
    ###########################################################
    ### "mount" gets added when "device" is added
    if args_dict.has_key("mount"):
        modify_mount(device)
        return

    if args_dict.has_key("add_nfsexport"):
        add_nfsexport(device)
        return

    if args_dict.has_key("del_nfsexport"):
        del_nfsexport(device)
        return

    if args_dict.has_key("nfsexports"):
        list_nfsexports(device)
        return

    if args_dict.has_key("nfsexport"):
        modify_nfsexport(device)
        return

    #####################################################

    sharename_before = device.get("sharename")

    try:
        ct_options_updated = device.modify_options(cmdline_args)
    except OptionError, e:
        ### This should catch options that are legal for some part
        ### of the cluster, but that are *illegal* for "--device"
        print _("Error:"), e.getMessage()
        sys.exit(2)

    if ct_options_updated == 0:
        device.out()
        return

    ### See if sharename changed ###
    if args_dict.has_key("sharename"):
        sharename = args_dict["sharename"]
        if sharename in ("", "None"):
            sharename = ""
        if sharename_before in ("", "None"):
            sharename_before = ""
        if sharename != sharename_before:
            if sharename == "":
                ### 8/12/2003 - HM comment out os.remove since we previously
                ### decided that when removing the device sharename, we do
                ### *NOT* want to actually remove the
                ### /etc/samba/smb.conf.[sharename] file.
                ###
                ### delete share ###
                #print "delete share"
                #try:
                #    os.remove(getAbsShareFilename(sharename_before))
                #except OSError, e:
                #    pass
                pass
            elif sharename_before == "":
                ### add share ###
                #print "add share"
                (sharename, ipaddresses, mountpoint, action) = device.smbconf.writeSambaShareFile()
                #abs_share_filename = getAbsShareFilename(sharename)
                #print "add smb sharename  =[%s]" % sharename
                #print "add smb ipaddresses=[%s]" % ipaddresses
                #print "add smb mountpoint =[%s]" % mountpoint
            else:
                ### change sharename: remove old one, then add new one ###
                #print "change sharename"
                try:
                    os.remove(getAbsShareFilename(sharename_before))
                except OSError, e:
                    pass
                (sharename, ipaddresses, mountpoint, action) = device.smbconf.writeSambaShareFile()
                #abs_share_filename = getAbsShareFilename(sharename)
                #print "chg smb sharename  =[%s]" % sharename
                #print "chg smb ipaddresses=[%s]" % ipaddresses
                #print "chg smb mountpoint =[%s]" % mountpoint

    ##############################
    ### end def modify_device ###
    ##############################

##########################################################################

def modify_mount(device):
    mount = device.get_mount()
    try:
        ct_options_updated = mount.modify_options(cmdline_args)
    except OptionError, e:
        ### This should catch options that are legal for some part
        ### of the cluster, but that are *illegal* for "--mount"
        print _("Error:"), e.getMessage()
        sys.exit(2)

    if ct_options_updated == 0:
        mount.out()

##########################################################################

def add_nfsexport(device):
    try:
        device.add_nfsexport(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)

def del_nfsexport(device):
    if not args_dict.has_key("nfsexport"):
        print _("Error: Option \"%s\" requires \"%s\"") % ("--del_nfsexport", "--nfsexport")
        sys.exit(2)

    try:
        device.delete_nfsexport(args_dict['nfsexport'], cmdline_args)
    except OptionError, e:
        #print _("Error:"), e.getMessage()
        print _("Error: NFS Export '%s' does not exist in Device '%s'") % (args_dict["nfsexport"], device.get("name"))
        sys.exit(2)

def list_nfsexports(device):
    nfsexport_entityList = device.get_nfsexports()
    for n in nfsexport_entityList:
        n.out()

def modify_nfsexport(device):
    nfsexport_name = args_dict['nfsexport']
    try:
        nfsexport = device.get_nfsexport(nfsexport_name)
    except OptionError, e:
        print _("Error: NFS Export \"%s\" does not exist for Device \"%s\"") \
              % (nfsexport_name, device.get("name"))
        sys.exit(2)

    ###########################################################
    ### Check for Main level children - if child exists, call
    ### the appropriate function to handle that child
    ###########################################################
    if args_dict.has_key("add_client"):
        add_client(nfsexport)
        return

    if args_dict.has_key("del_client"):
        del_client(nfsexport)
        return

    if args_dict.has_key("clients"):
        list_clients(nfsexport)
        return
    
    if args_dict.has_key("client"):
        modify_client(nfsexport)
        return

    ##########################################################

    try:
        ct_options_updated = nfsexport.modify_options(cmdline_args)
    except OptionError, e:
        ### This should catch options that are legal for some part
        ### of the cluster, but that are *illegal* for "--nfsexport"
        #print "Error: Option \"--%s\" is not valid with \"--nfsexport\"." \
        #      % e.opt
        print _("Error:"), e.getMessage()
        sys.exit(2)

    if ct_options_updated == 0:
        nfsexport.out()

    #################################
    ### end def modify_nfsexport ###
    #################################

##########################################################################

def add_client(nfsexport):
    try:
        nfsexport.add_client(cmdline_args)
    except OptionError, e:
        #print _("Error: --add_client requires --%s.") % e.getOptionName()
        print _("Error:"), e.getMessage()
        sys.exit(2)

def del_client(nfsexport):
    if not args_dict.has_key("client"):
        print _("Error: Option \"%s\" requires \"%s\"") % ("--del_client", "--client")
        sys.exit(2)

    try:
        nfsexport.delete_client(args_dict['client'], cmdline_args)
    except OptionError, e:
        #print _("Error:"), e.getMessage()
        print _("Error: NFS Export Client '%s' does not exist for NFS Export Directory '%s'") % (args_dict["client"], nfsexport.get("name"))
        sys.exit(2)

def list_clients(nfsexport):
    client_entityList = nfsexport.get_clients()
    for c in client_entityList:
        c.out()

def modify_client(nfsexport):
    client_name = args_dict['client']
    try:
        client = nfsexport.get_client(client_name)
    except OptionError, e:
        print _("Error: Client \"%s\" does not exist for NFS Export \"%s\"") \
              % (client_name, nfsexport.get("name"))
        sys.exit(2)

    #####################################################

    try:
        ct_options_updated = client.modify_options(cmdline_args)
    except OptionError, e:
        ### This should catch options that are legal for some part
        ### of the cluster, but that are *illegal* for "--client"
        #print "Error: Option \"--%s\" is not valid with \"--client\"." \
        #      % e.opt
        print _("Error:"), e.getMessage()
        sys.exit(2)

    if ct_options_updated == 0:
        client.out()

    #############################
    ### end def modify_client ###
    #############################

##########################################################################

def add_service_ipaddress(service):
    try:
        service.add_service_ipaddress(cmdline_args)
    except OptionError, e:
        print _("Error:"), e.getMessage()
        sys.exit(2)

def del_service_ipaddress(service):
    if not args_dict.has_key("service_ipaddress"):
        print _("Error: Option \"%s\" requires \"%s\"") % ("--del_service_ipaddress", "--service_ipaddress")
        sys.exit(2)

    try:
        service.delete_service_ipaddress(args_dict['service_ipaddress'], cmdline_args)
    except OptionError, e:
        #print _("Error:"), e.getMessage()
        print _("Error: IP Address '%s' does not exist in service '%s'") % (args_dict["service_ipaddress"], service.get("name"))
        sys.exit(2)

def list_service_ipaddresses(service):
    service_ipaddress_entityList = service.get_service_ipaddresses()
    for n in service_ipaddress_entityList:
        n.out()

def modify_service_ipaddress(service):
    service_ipaddress_ipaddress = args_dict['service_ipaddress']
    try:
        service_ipaddress = service.get_service_ipaddress(service_ipaddress_ipaddress)
    except OptionError, e:
        print _("Error: IP Address \"%s\" does not exist for Service \"%s\"") \
              % (service_ipaddress_ipaddress, service.get("name"))
        sys.exit(2)

    try:
        ct_options_updated = service_ipaddress.modify_options(cmdline_args)
    except OptionError, e:
        ### This should catch options that are legal for some part
        ### of the cluster, but that are *illegal* for "--service_ipaddress"
        #print "Error: Option \"--%s\" is not valid with \"--service_ipaddress\"." \
        #      % e.opt
        print _("Error:"), e.getMessage()
        sys.exit(2)

    if ct_options_updated == 0:
        service_ipaddress.out()

    ########################################
    ### end def modify_service_ipaddress ###
    ########################################

def usage():
    print """
Usage: redhat-config-cluster-cmd [h|help]

       -- unfinished --
"""

############################################################################
###############################  Main  #####################################
############################################################################

def main():
    if os.getuid() != 0:
        print _("Please restart %s with root permissions!") % (sys.argv[0])
        sys.exit(10)

    global cmdline_args
    cmdline_args = my_cmdline_argv[1:]

    try:
        opts, rest_opts = getopt.getopt(cmdline_args, "h",
        [
            "help",

            "cluster",
            "clumembd",
            #"clupowerd",
            "cluquorumd",
            "clurmtabd",
            "clusvcmgrd",
            "clulockd",
            "sharedstate",

            "members",
            "member=",
               "powercontrollers",
               "powercontroller=",

            "failoverdomains",
            "failoverdomain=",
               "failoverdomainnodes",
               "failoverdomainnode=",

            "services",
            "service=",
               "devices",
               "device=",
                  "mount",
                     "nfsexports",
                     "nfsexport=",
                        "clients",
                        "client=",
               "service_ipaddresses",
               "service_ipaddress=",

            "add_member",
            "add_powercontroller",

            "add_failoverdomain",
            "add_failoverdomainnode",

            "add_service",
            "add_device",
            "add_mount",
            "add_nfsexport",
            "add_client",
            "add_service_ipaddress",

            "del_member",
            "del_powercontroller",

            "del_failoverdomain",
            "del_failoverdomainnode",

            "del_service",
            "del_device",
            "del_nfsexport",
            "del_client",
            "del_service_ipaddress",

            ### parameter for each/all elements ###
            "loglevel=",
            "pinginterval=",
            "tiebreaker_ip=",
            "pollinterval=",
            "name=",
            "config_viewnumber=",
            "type=",
            "watchdog=",
            "quorumpartitionprimary=",
            "quorumpartitionshadow=",
            "ipaddr=",
            "port=",
            "checkinterval=",
            "mountpoint=",
            "fstype=",
            "options=",
            "forceunmount=",
            "userscript=",
            "sharename=",
            "ipaddress=",
            "netmask=",
            "broadcast=",
            "interval=",
            #"strict=",
            #"adaptive=",
            "tko_count=",
            "thread=",
            "multicast=",
            "multicast_ipaddress=",
            "restricted=",
            "ordered=",
            "order=",
            "driver=",
            "rawprimary=",
            "rawshadow=",
            "sfspath=",
        ])

    except getopt.GetoptError, exception:
        #print "Error: Main getopt error."
        #print "  exception       = %s" % exception
        #print "  exception.args  = ", exception.args
        #print "  exception.msg   = %s" % exception.msg
        #print "  type(exception) = %s" % type(exception)
        #print "  dir(exception)  = %s" % dir(exception)
        #print "------------------------------"
        print "Error: %s" % exception.msg
        sys.exit(2)

    if len(rest_opts) != 0:
        print "\nError: Illegal arguments (" + string.join(rest_opts, ', ') + ") were not parsed"
        sys.exit(2)

    #print "after getopts: opts      = %s" % opts
    #print "after getopts: rest_opts = %s" % rest_opts

    ct = 0
    for opt, val in opts:
        #print "opt=<%s>, val=<%s>" % (opt, val)
        if opt == "-h":
            opt_name = "h"
        else:
            ### must be a long option ###
            opt_name = opt[2:]
        args_dict[opt_name] = val

    #for opt in args_dict.keys():
    #    print "Main - args_dict[%s] = %s" % (opt, args_dict[opt])

    if args_dict.has_key("h") or args_dict.has_key("help"):
        usage()
        sys.exit(0)
    elif args_dict.has_key("clumembd")  : modify_clumembd()
    #elif args_dict.has_key("clupowerd") : modify_clupowerd()
    elif args_dict.has_key("cluquorumd"): modify_cluquorumd()
    elif args_dict.has_key("clurmtabd") : modify_clurmtabd()
    elif args_dict.has_key("clusvcmgrd"): modify_clusvcmgrd()
    elif args_dict.has_key("clulockd")  : modify_clulockd()
    elif args_dict.has_key("cluster")   : modify_cluster()
    elif args_dict.has_key("sharedstate") : modify_sharedstate()

    elif args_dict.has_key("add_member"): add_member()
    elif args_dict.has_key("del_member"): del_member()
    elif args_dict.has_key("members")   : list_members()
    elif args_dict.has_key("member")    : modify_member()

    elif args_dict.has_key("add_service"): add_service()
    elif args_dict.has_key("del_service"): del_service()
    elif args_dict.has_key("services")   : list_services()
    elif args_dict.has_key("service")    : modify_service()

    elif args_dict.has_key("add_failoverdomain"): add_failoverdomain()
    elif args_dict.has_key("del_failoverdomain"): del_failoverdomain()
    elif args_dict.has_key("failoverdomains")   : list_failoverdomains()
    elif args_dict.has_key("failoverdomain")    : modify_failoverdomain()

    else:
        #print "Error: *NO* Main options chosen - calling Usage now."
        usage()
        sys.exit(1)

    if clu != clu_saved:
        #print "redhat-config-cluster-cmd: Save XML to file Now!"
        clu.saveXMLToFile()

    sys.exit(0)


if __name__ == "__main__":
    main()
