#!/usr/bin/python

## serviceDialog.py

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import gtk.glade
import gettext

from clusterpkg.cluster_functions import OptionError
from clusterpkg.cluster_functions_gui import errorMessage

import deviceOrServiceIPAddressDialog
import deviceDialog
import serviceIPAddressDialog

### Need "failoverdomains" because in order to populate the "Failover Domain"
### combobox, we need to get the failover domains.
from sys import path
from clusterpkg.failoverdomains_module import failoverdomains

TRUE = (1==1)
FALSE = not TRUE

##
## I18N
##

class serviceDialog: #(gtk.Dialog):
    def __init__(self, glade_xml):
        self.glade_xml = glade_xml

        self.dialog = glade_xml.get_widget('serviceDialog')

        self.refresh = TRUE

    def on_cancelbutton_clicked(self, button):
        self.refresh = TRUE

    def on_okbutton_clicked(self, button):

        ### "get" the new values from the Gui widgets ###

        name           = self.glade_xml.get_widget('service_name_entry').get_text()
        failoverdomain = self.glade_xml.get_widget('service_failoverdomain_combo-entry').get_text()

        checkinterval  = self.glade_xml.get_widget('service_checkinterval_spinbutton').get_value_as_int()
        userscript     = self.glade_xml.get_widget('service_userscript_entry').get_text()
        maxrestarts     = self.glade_xml.get_widget('max_restarts_entry').get_text()
        maxfalsestarts     = self.glade_xml.get_widget('max_false_starts_entry').get_text()

        ### If user didn't update any fields, then just return.
        if name           == self.name           and \
           failoverdomain == self.failoverdomain and \
           checkinterval  == self.checkinterval  and \
           userscript     == self.userscript     and \
           maxrestarts    == self.maxrestarts    and \
           maxfalsestarts == self.maxfalsestarts:
              self.dialog.response(gtk.RESPONSE_OK)
              return

        options = { "name"          : name,
                    "failoverdomain" : failoverdomain,
                    "checkinterval"  : checkinterval,
                    "userscript"     : userscript,
                    "maxrestarts"      : maxrestarts,
                    "maxfalsestarts" : maxfalsestarts }

        self.refresh = TRUE
        rc = gtk.RESPONSE_OK
        try:
            ### Invoke service 'validate' to validate all options
            self.service.validate(options)
        except OptionError, e:
            if e.getSeverity() == 'FATAL':
                errorMessage(e.getMessage())
                self.refresh = FALSE # Don't refresh - leave bad values
                                     # on screen.
                rc = gtk.RESPONSE_REJECT

            elif e.getSeverity() == 'WARNING':
                msg = _("%s\n\nPress 'OK' to Continue, or  'Cancel' to Exit.") % e.getMessage()
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                    self.refresh = FALSE     # Don't refresh - leave bad values
                                             # on screen.
                    rc = gtk.RESPONSE_REJECT # Need to differentiate this
                                             # 'Cancel' from the main Dialog
                                             # 'Cancel', so make this rc
                                             # gtk.RESPONSE_REJECT.

        if rc == gtk.RESPONSE_OK:
            ### Validation passed on all fields - now "set" each by calling
            ### the ClusterEntity "set".
            self.service.set("name", name)
            self.service.set("failoverdomain", failoverdomain)
            self.service.set("checkinterval", checkinterval)
            self.service.set("userscript", userscript)
            self.service.set("maxrestarts", maxrestarts)
            self.service.set("maxfalsestarts", maxfalsestarts)

            self.service.change_action_to_modify() # for "validate"

        self.dialog.response(rc)
        return

    def on_userscript_browsebutton_clicked(self, button):
        self.fileSelection = self.glade_xml.get_widget('fileSelectionDialog')
        self.fileSelection.set_title("Select Service User Script")
        while 1:
            status = self.fileSelection.run()
            if status == gtk.RESPONSE_OK:
                try:
                    self.glade_xml.get_widget('service_userscript_entry').set_text(self.fileSelection.get_filename())
                    self.fileSelection.hide()
                    return status
                except:
                    raise
            else:
                self.fileSelection.hide()
                return status

    def run(self, service, action):
        self.service = service
        self.action  = action

        ### Now connect the buttons to the functions ###
        self.ok     = self.glade_xml.get_widget('serviceDialog_okbutton').connect("clicked", self.on_okbutton_clicked)
        self.cancel = self.glade_xml.get_widget('serviceDialog_cancelbutton').connect("clicked", self.on_cancelbutton_clicked)
        self.browse = self.glade_xml.get_widget('serviceDialog_userscript_browsebutton').connect("clicked", self.on_userscript_browsebutton_clicked)

        ### Load screen widgets with data from XML DOM for selected service ###

        if self.refresh:

            ### find failover domains currently in XML (in memory) ###

            ### To get current list of failover domains, need to get DOM XML
            ### ownerDocument(root DOM XML node), so we can then
            ### find(using getElementsByTagName) the 'failoverdomains' node.
            ownerDocument = service.node.ownerDocument
            failoverdomains_node = ownerDocument.getElementsByTagName("failoverdomains")[0]
            failoverdomains_entity = failoverdomains(failoverdomains_node)


            self.name = service.get("name")
            self.glade_xml.get_widget('service_name_entry').set_text(self.name)

            failoverdomain_combo_widget = self.glade_xml.get_widget('service_failoverdomain_combo')
            failoverdomains_list = ["None"]
            for f in failoverdomains_entity.get_list():
                failoverdomains_list.append(f.get("name"))
            failoverdomain_combo_widget.set_popdown_strings(failoverdomains_list)

            self.failoverdomain = service.get("failoverdomain")
            failoverdomain_entry = self.glade_xml.get_widget('service_failoverdomain_combo-entry')
            failoverdomain_entry.set_text(self.failoverdomain)

            self.checkinterval = service.get("checkinterval")
            self.glade_xml.get_widget('service_checkinterval_spinbutton').set_value(self.checkinterval)

            self.userscript = service.get("userscript")
            self.glade_xml.get_widget('service_userscript_entry').set_text(self.userscript)
            self.maxrestarts = service.get("maxrestarts")
            self.glade_xml.get_widget('max_restarts_entry').set_text(str(self.maxrestarts))
            self.maxfalsestarts = service.get("maxfalsestarts")
            self.glade_xml.get_widget('max_false_starts_entry').set_text(str(self.maxfalsestarts))

        rc = self.dialog.run()
        self.glade_xml.get_widget('serviceDialog_okbutton').disconnect(self.ok)
        self.glade_xml.get_widget('serviceDialog_cancelbutton').disconnect(self.cancel)
        self.glade_xml.get_widget('serviceDialog_userscript_browsebutton').disconnect(self.browse)
        return rc

    def hide(self):
        return self.dialog.hide()

class serviceItem:
    def __init__(self, parent, service):
        ### parent is "servicesTreeview" instance ###
        self.parent    = parent
        self.glade_xml = parent.glade_xml
        self.treeview  = parent.treeview
        self.model     = parent.treeview.get_model()
        self.columns   = parent.columns
        self.service   = service
        self.dialog    = parent.dialog

    def GUI_load_row_columns(self, service_item, iter):
        row_display_string = _("%s  <service>") % service_item.service.get("name")
        self.model.set(iter,
                       self.columns['COLUMN_KEY'], row_display_string,
                       self.columns['COLUMN_DATA'], service_item)

    def GUI_tree_load_service(self):
        iter = self.model.append(None)
        self.GUI_load_row_columns(self, iter)

        for n in self.service.get_service_ipaddresses():
            service_ipaddress_item = serviceIPAddressDialog.serviceIPAddressItem(self, n)
            service_ipaddress_item.GUI_tree_load_service_ipaddress(iter)

        for d in self.service.get_devices():
            device_item = deviceDialog.deviceItem(self, d)
            device_item.GUI_tree_load_device(iter)

    def get_type(self):
        return "service"

    def get_value_of_key_field(self):
        return self.get("name")

    def get(self, option):
        return self.service.get(option)

    def __modify(self, iter, action="modify"):
        if action == "modify":
            service = self.service
            service_item = self
        else:
            ### Must be an "add" ###
            services_entity = self.parent.services_entity
            service = services_entity.add_service()
            service_item = serviceItem(self.parent, service)

        while 1:
            rc = self.dialog.run(service, action)
            if rc == gtk.RESPONSE_OK:
                if action == "add":
                    ### Since when we're adding a service we're at the top
                    ### of the tree, give append "None" to signify we're
                    ### appending to the root.
                    iter = self.model.append(None)

                self.GUI_load_row_columns(service_item, iter)

            elif rc == gtk.RESPONSE_CANCEL:
                if action == "add":
                    services_entity.delete_service_by_id(service.get("id"))
            else:
                ### rc was gtk.RESPONSE_REJECT:
                ###   1. User got a FATAL error and clicked 'OK' (only option)
                ###      so display same dialog again,
                ###        OR
                ###   2. User got a WARNING and clicked 'Cancel' to go back
                ###      and re-enter.
                continue
            break

        self.dialog.hide()

    def add(self, iter):
        self.__modify(iter, "add")

    def add_child(self, iter):
        action = "add"

        ### 1st, does user want to add a 'Device' or a 'Service IP Address'???
        device_or_service_ipaddress_dialog = deviceOrServiceIPAddressDialog.deviceOrServiceIPAddressDialog(self.glade_xml)
        while 1:
            rc = device_or_service_ipaddress_dialog.run()
            device_or_service_ipaddress_dialog.hide()
            if rc == gtk.RESPONSE_OK:
                device_selected = device_or_service_ipaddress_dialog.get_device_selected()
                service_ipaddress_selected = device_or_service_ipaddress_dialog.get_service_ipaddress_selected()
                break
            if rc == gtk.RESPONSE_CANCEL:
                return

        if device_selected:
            #device = self.service.add_device()
            #device_item = deviceDialog.deviceItem(self, device)
            #device_item.add(iter)
            device = self.service.add_device()
            device_item = deviceDialog.deviceItem(self, device)
            device_dialog = deviceDialog.deviceDialog(self.glade_xml, device)
            while 1:
                rc = device_dialog.run(device, action)
                if rc == gtk.RESPONSE_OK:
                    iter = self.model.append(iter)

                    device_item.GUI_load_row_columns(device_item, iter)
                    break
                if rc == gtk.RESPONSE_CANCEL:
                    self.service.delete_device_by_id(device.get("id"))
                    break
            device_dialog.hide()

        if service_ipaddress_selected:
            ### If these next 3 commented 'service_ipaddress' lines are used
            ### as the only code underneath 'if service_ipaddress_selected',
            ### it causes 2 service ipaddress tags to be created(one "None"),
            ### since add_service_ipaddress() is invoked once here, and then
            ### again inside serviceIPAddressItem __modify (add logic).
            ### This is bad, so don't use them.
            ###
            #service_ipaddress = self.service.add_service_ipaddress()
            #service_ipaddress_item = serviceIPAddressDialog.serviceIPAddressItem(self, service_ipaddress)
            #service_ipaddress_item.add(iter)
            service_ipaddress = self.service.add_service_ipaddress()
            service_ipaddress_item = serviceIPAddressDialog.serviceIPAddressItem(self, service_ipaddress)
            service_ipaddress_dialog = serviceIPAddressDialog.serviceIPAddressDialog(self.glade_xml, service_ipaddress)
            while 1:
                rc = service_ipaddress_dialog.run(service_ipaddress, action)
                if rc == gtk.RESPONSE_OK:
                    iter = self.model.append(iter)
                    service_ipaddress_item.GUI_load_row_columns(service_ipaddress_item, iter)
                    break
                if rc == gtk.RESPONSE_CANCEL:
                    self.service.delete_service_ipaddress_by_id(service_ipaddress.get("id"))
                    break

            service_ipaddress_dialog.hide()

    def properties(self, iter):
        self.__modify(iter)

    def delete(self):
        services_entity = self.parent.services_entity
        services_entity.delete_service_by_id(self.service.get("id"))
