#!/usr/bin/python

## serviceIPAddressDialog.py

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import gtk
import gtk.glade
import gettext
import os, string

from clusterpkg.cluster_functions     import OptionError
from clusterpkg.cluster_functions_gui import errorMessage, warningMessage

##
## I18N
##

class serviceIPAddressDialog: #(gtk.Dialog):
    def __init__(self, glade_xml, service_ipaddress):
        self.glade_xml = glade_xml
        self.service_ipaddress = service_ipaddress

        self.dialog = glade_xml.get_widget('serviceIPAddressDialog')

        self.refresh = gtk.TRUE

    def on_cancelbutton_clicked(self, button):
        self.refresh = gtk.TRUE

    def on_okbutton_clicked(self, button):

        ### "get" the new values from the Gui widgets ###

        ipaddress = self.glade_xml.get_widget('service_ipaddress_ipaddress_entry').get_text()
        netmask = self.glade_xml.get_widget('service_ipaddress_netmask_entry').get_text()
        broadcast = self.glade_xml.get_widget('service_ipaddress_broadcast_entry').get_text()
        monitor_link = self.glade_xml.get_widget('monitor_link_box').get_active()

        ### If user didn't update any fields, then just return.
        if ipaddress == self.ipaddress and \
           netmask   == self.netmask   and \
           broadcast == self.broadcast and \
           monitor_link == self.monitor_link:
              self.dialog.response(gtk.RESPONSE_OK)
              return

        options = { "ipaddress" : ipaddress,
                    "netmask"   : netmask,
                    "broadcast" : broadcast,
                    "monitor_link"   : monitor_link  }

        self.refresh = gtk.TRUE
        rc = gtk.RESPONSE_OK
        try:
            self.service_ipaddress.validate(options)
        except OptionError, e:
            if e.getSeverity() == 'FATAL':
                errorMessage(e.getMessage())
                self.refresh = gtk.FALSE # Don't refresh - leave bad values
                                         # on screen.
                rc = gtk.RESPONSE_REJECT

            elif e.getSeverity() == 'WARNING':
                if e.getOptionName() == "ipaddress":
                    ### WARNING because IP address gethostbyname(DNS lookup)
                    ### failed.
                    msg = _("%s\n\nPress 'OK' to Accept, or 'Cancel' to change it.") % e.getMessage()
                else:
                    msg = _("%s\n\nPress 'OK' to Continue, or 'Cancel' to Exit.") % e.getMessage()
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                   self.refresh = gtk.FALSE # Don't refresh - leave bad values
                                            # on screen.
                   rc = gtk.RESPONSE_REJECT # Need to differentiate this
                                            # 'Cancel' from the main Dialog
                                            # 'Cancel', so make this rc
                                            # gtk.RESPONSE_REJECT. 

        #print "AAAAAAAAAAAAA - rc =", rc
        #print "   gtk.RESPONSE_OK =", gtk.RESPONSE_OK
        #print "   gtk.RESPONSE_CANCEL =", gtk.RESPONSE_CANCEL
        #print "   gtk.RESPONSE_REJECT =", gtk.RESPONSE_REJECT

        if rc == gtk.RESPONSE_OK:
            ### Validation passed - now 'set' DOM XML options ###
            self.service_ipaddress.set("ipaddress", ipaddress)
            self.service_ipaddress.set("netmask", netmask)
            self.service_ipaddress.set("broadcast", broadcast)
            self.service_ipaddress.set("monitor_link", monitor_link)

            self.service_ipaddress.change_action_to_modify()

        self.dialog.response(rc)
        return

    def run(self, service_ipaddress, action):
        self.service_ipaddress = service_ipaddress
        self.action = action

        ### Now connect the buttons to the functions ###
        self.ok = self.glade_xml.get_widget('serviceIPAddressDialog_okbutton').connect("clicked", self.on_okbutton_clicked)
        self.cancel = self.glade_xml.get_widget('serviceIPAddressDialog_cancelbutton').connect("clicked", self.on_cancelbutton_clicked)

        ### Load screen widgets with data from XML DOM for selected
        ### service_ipaddress.

        if self.refresh:
            self.ipaddress = service_ipaddress.get("ipaddress")
            self.glade_xml.get_widget('service_ipaddress_ipaddress_entry').set_text(self.ipaddress)

            self.netmask = service_ipaddress.get("netmask")
            self.glade_xml.get_widget('service_ipaddress_netmask_entry').set_text(self.netmask)

            self.broadcast = service_ipaddress.get("broadcast")
            self.glade_xml.get_widget('service_ipaddress_broadcast_entry').set_text(self.broadcast)

            self.monitor_link = service_ipaddress.get("monitor_link")
            #if self.monitor_link != None:
            if self.monitor_link != "":
              self.glade_xml.get_widget('monitor_link_box').set_active(int(self.monitor_link))
            else:
              self.glade_xml.get_widget('monitor_link_box').set_active(0)

        rc = self.dialog.run()
        self.glade_xml.get_widget('serviceIPAddressDialog_okbutton').disconnect(self.ok)
        self.glade_xml.get_widget('serviceIPAddressDialog_cancelbutton').disconnect(self.cancel)

        return rc

    def show(self):
        return self.dialog.show()

    def hide(self):
        return self.dialog.hide()

class serviceIPAddressItem:
    def __init__(self, parent, service_ipaddress):
        self.parent    = parent
        self.glade_xml = parent.glade_xml
        self.treeview  = parent.treeview
        self.model     = parent.treeview.get_model()
        self.columns   = parent.columns
        self.service_ipaddress = service_ipaddress

        self.dialog = serviceIPAddressDialog(self.glade_xml, service_ipaddress)

    def GUI_load_row_columns(self, service_ipaddress_item, iter):
        row_display_string = _("%s  <service ip address>") % service_ipaddress_item.service_ipaddress.get("ipaddress")
        self.model.set(iter,
                       self.columns['COLUMN_KEY'], row_display_string,
                       self.columns['COLUMN_DATA'], service_ipaddress_item)

    def GUI_tree_load_service_ipaddress(self, iter):
        service_ipaddress_iter = self.model.append(iter)
        self.GUI_load_row_columns(self, service_ipaddress_iter)

    def get_type(self):
        return "service_ipaddress"

    def get_value_of_key_field(self):
        return self.get("ipaddress") # ipaddress is the "key" field

    def get(self, option):
        return self.service_ipaddress.get(option)

    def __modify(self, iter, action="modify"):
        #print "serviceIPAddressItem __modify TOP: action =", action
        if action == "modify":
            service_ipaddress = self.service_ipaddress
            service_ipaddress_item = self
        else:
            ### Must be an "add" ###
            service_item = self.parent
            service_entity = service_item.service

            service_ipaddress = service_entity.add_service_ipaddress()
            service_ipaddress_item = serviceIPAddressItem(self.parent, service_ipaddress)

        while 1:
            rc = self.dialog.run(service_ipaddress, action)
            if rc == gtk.RESPONSE_OK:
                if action == "add":
                    ### model.append needs the "iter" of the *parent* of the
                    ### thing that is being appended.
                    parent_iter = iter
                    iter = self.model.append(parent_iter)

                self.GUI_load_row_columns(service_ipaddress_item, iter)

            elif rc == gtk.RESPONSE_CANCEL:
                if action == "add":
                    service_entity.delete_service_ipaddress_by_id(service_ipaddress.get("id"))
            else:
                ### rc was gtk.RESPONSE_REJECT:
                ###   1. User got a FATAL error and clicked 'OK' (only option)
                ###      so display same dialog again,
                ###        OR
                ###   2. User got a WARNING and clicked 'Cancel' to go back
                ###      and re-enter.
                continue
            break 

        self.dialog.hide()

    def add(self, parent_iter):
        #print "serviceIPAddressItem add TOP"
        self.__modify(parent_iter, "add")

    def properties(self, iter):
        #print "serviceIPAddressItem properties TOP"
        self.__modify(iter)

    def delete(self):
        service_item = self.parent
        service_entity = service_item.service
        id = self.service_ipaddress.get("id")
        service_entity.delete_service_ipaddress_by_id(id)
