#!/usr/bin/python

## sharedstateDialog.py

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

TRUE = (1==1)
FALSE = not TRUE

import gtk
import gtk.glade
import gettext

from clusterpkg.cluster_functions     import OptionError
from clusterpkg.cluster_functions_gui import errorMessage, warningMessage
from clusterpkg.sharedstate_module    import DRIVERS, ALLOW_SFS

##
## I18N
##

class sharedstateDialog(gtk.Dialog):
    def __init__(self, glade_xml, clu):
        gtk.Dialog.__init__(self, "")
        self.glade_xml = glade_xml
        self.clu = clu
        self.sharedstate = clu.getsharedstate()

        self.firsttime = TRUE

        type = self.sharedstate.get("type")
        if type == "sfs" and ALLOW_SFS:
           self.glade_xml.get_widget('sharedstate_raw_radiobutton').set_active(FALSE)
           self.glade_xml.get_widget('sharedstate_sfs_radiobutton').set_active(TRUE)
        else:
           self.glade_xml.get_widget('sharedstate_raw_radiobutton').set_active(TRUE)
           self.glade_xml.get_widget('sharedstate_sfs_radiobutton').set_active(FALSE)

        self.rawprimary = self.sharedstate.get("rawprimary")
        self.glade_xml.get_widget('sharedstate_rawprimary_entry').set_text(self.rawprimary)

        self.rawshadow = self.sharedstate.get("rawshadow")
        self.glade_xml.get_widget('sharedstate_rawshadow_entry').set_text(self.rawshadow)

        self.sfspath = self.sharedstate.get("sfspath")
        self.glade_xml.get_widget('sharedstate_sfspath_entry').set_text(self.sfspath)
        ### Now connect the buttons to the functions ###
        self.glade_xml.get_widget('sharedstateDialog_cancelbutton').connect("clicked", self.on_cancelbutton_clicked)
        self.glade_xml.get_widget('sharedstateDialog_okbutton').connect("clicked", self.on_okbutton_clicked)
        self.raw_radiobutton = self.glade_xml.get_widget('sharedstate_raw_radiobutton').connect("clicked", self.on_raw_or_sfs_radiobutton_clicked)
        self.sfs_radiobutton = self.glade_xml.get_widget('sharedstate_sfs_radiobutton').connect("clicked", self.on_raw_or_sfs_radiobutton_clicked)

        if type == 'raw':
           raw_selected = TRUE
           sfs_selected = FALSE
        else:
           raw_selected = FALSE
           sfs_selected = TRUE
        self.set_raw_or_sfs_sensitive(raw_selected, sfs_selected)

        self.dialog = glade_xml.get_widget('sharedstateDialog')

        if not ALLOW_SFS:
           self.glade_xml.get_widget('sharedstate_sfs_radiobutton').hide()
           self.glade_xml.get_widget('sharedstate_sfspath_label').hide()
           self.glade_xml.get_widget('sharedstate_sfspath_entry').hide()

    def on_cancelbutton_clicked(self, button):
        pass

    def on_okbutton_clicked(self, button):
        ### Copy the new values from the Gui widgets into the DOM xml ###
        self.__raw_selected  = self.glade_xml.get_widget('sharedstate_raw_radiobutton').get_active()
        self.__sfs_selected = self.glade_xml.get_widget('sharedstate_sfs_radiobutton').get_active()

        rawprimary = ""
        rawshadow  = ""
        sfspath    = ""
        if self.__raw_selected:
           type = "raw"
           rawprimary = self.glade_xml.get_widget('sharedstate_rawprimary_entry').get_text()
           rawshadow  = self.glade_xml.get_widget('sharedstate_rawshadow_entry').get_text()
        else:
           type = "sfs"
           sfspath = self.glade_xml.get_widget('sharedstate_sfspath_entry').get_text()
        options = { "type"       : type,
                    "rawprimary" : rawprimary,
                    "rawshadow"  : rawshadow,
                    "sfspath"    : sfspath }

        rc = gtk.RESPONSE_OK
        try:
            ### Invoke 'validate' to validate all options
            self.sharedstate.validate(options)
        except OptionError, e:
            if e.getSeverity() == 'FATAL':
                errorMessage(e.getMessage())
                self.refresh = FALSE # Don't refresh - leave bad values
                                         # on screen.
                self.dialog.response(gtk.RESPONSE_REJECT)
                rc = gtk.RESPONSE_REJECT

            elif e.getSeverity() == 'WARNING':
                msg = _("%s\n\nPress 'OK' to Continue, or  'Cancel' to Exit.") % e.getMessage()
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                   self.refresh = FALSE # Don't refresh - leave bad values
                                            # on screen.
                   self.dialog.response(gtk.RESPONSE_REJECT)

        if rc != gtk.RESPONSE_OK:
            return

        ### Validation passed - now 'set' options ###
        prev_type = self.sharedstate.get("type")
        if self.__raw_selected:
            self.sharedstate.set("type", "raw")
            self.sharedstate.set("rawprimary", rawprimary)
            self.sharedstate.set("rawshadow", rawshadow)
            self.sharedstate.set("driver", DRIVERS["raw"])
            ### If sharestate was previously 'sfs', then delete sfs options ###
            if prev_type == "sfs":
                self.sharedstate.delete_option("sfspath")
        else:
            self.sharedstate.set("type", "sfs")
            self.sharedstate.set("sfspath", sfspath)
            self.sharedstate.set("driver", DRIVERS["sfs"])
            ### If sharestate was previously 'raw', then delete raw options ###
            if prev_type == "raw":
                self.sharedstate.delete_option("rawprimary")
                self.sharedstate.delete_option("rawshadow")

        self.sharedstate.change_action_to_modify()

    def set_raw_or_sfs_sensitive(self, raw_selected, sfs_selected):
        if raw_selected or not ALLOW_SFS:
           ### 'Raw' selected - 1. unprotect 'Raw' fields and load  ###
           self.glade_xml.get_widget('sharedstate_rawprimary_entry').set_sensitive(TRUE)
           self.glade_xml.get_widget('sharedstate_rawshadow_entry').set_sensitive(TRUE)
           self.glade_xml.get_widget('sharedstate_rawprimary_entry').set_text(self.rawprimary)
           self.glade_xml.get_widget('sharedstate_rawshadow_entry').set_text(self.rawshadow)

           ### 'Raw' selected - 2. protect 'SFS' fields and init to null ###
           self.glade_xml.get_widget('sharedstate_sfspath_entry').set_sensitive(FALSE)
           self.glade_xml.get_widget('sharedstate_sfspath_entry').set_text("")

        else:
           ### 'SFS' selected - 1. unprotect 'SFS' fields and load ###
           self.glade_xml.get_widget('sharedstate_sfspath_entry').set_sensitive(TRUE)
           self.glade_xml.get_widget('sharedstate_sfspath_entry').set_text(self.sfspath)

           ### 'SFS' selected - 2. protect 'Raw' fields and init to null ###
           self.glade_xml.get_widget('sharedstate_rawprimary_entry').set_sensitive(FALSE)
           self.glade_xml.get_widget('sharedstate_rawshadow_entry').set_sensitive(FALSE)
           self.glade_xml.get_widget('sharedstate_rawprimary_entry').set_text("")
           self.glade_xml.get_widget('sharedstate_rawshadow_entry').set_text("")

    def on_raw_or_sfs_radiobutton_clicked(self, button):
        if self.firsttime:
            ### When loading screen for the 1st time, we invoke radio button
            ### set_active to set one of the radio buttons to "on", which fires
            ### this function.  When that happens(firsttime) just return.
            self.firsttime = FALSE
            return

        self.__raw_selected  = self.glade_xml.get_widget('sharedstate_raw_radiobutton').get_active()
        self.__sfs_selected = self.glade_xml.get_widget('sharedstate_sfs_radiobutton').get_active()

        self.set_raw_or_sfs_sensitive(self.__raw_selected, self.__sfs_selected)

    def run(self):
        return self.dialog.run()

    def hide(self):
        return self.dialog.hide()
