#!/usr/bin/python

import gtk
import gobject
import os
import stat
import string
from clusterpkg.cluster_functions     import cluStat, cluPing, hostInDNS
from clusterpkg.cluster_functions_gui import errorMessage, warningMessage
from clusterpkg.util_df_module        import cluDF
from clusterpkg.util_ipconvert_module import ipConvert
from clusterpkg.util_smbconf_module   import getAbsShareFilename

TRUE = 1
FALSE = 0

### Existing Service columns
COLUMN_NAME = 0
COLUMN_DATA = 1

### Existing Device columns
COL_DEVICE_NAME = 0
COL_DEVICE_MOUNTPOINT = 1
COL_DEVICE_DATA = 2

class smbShareDruid:
    def __init__(self, glade_xml, clu):
        self.glade_xml = glade_xml
        self.clu = clu
        self.services_entity = self.clu.getservices()
        self.dlg = self.glade_xml.get_widget("addSMBDruidDialog")
        self.druid = self.glade_xml.get_widget("addSMBDruid")
        self.druid.connect("cancel",self.cancel)
        self.page1 = self.glade_xml.get_widget("smbdruidpagestart")

        self.glade_xml.get_widget("smbdruidpagestart").connect("next", self.on_StartPage_next)

        ###########################
        ### Select Service Page ###
        ###########################
        self.existSvcRadiobutton = self.glade_xml.get_widget("smbExistSvcRadiobutton")
        self.newSvcRadiobutton = self.glade_xml.get_widget("smbNewSvcRadiobutton")
        self.newSvcNameEntry = self.glade_xml.get_widget("smbNewSvcNameEntry")
        self.newSvcIPAddressEntry = self.glade_xml.get_widget("smbNewSvcIPAddressEntry")
        self.glade_xml.get_widget("smbServicePage").connect("next", self.on_ServicePage_next)

        self.existSvcRadiobutton.set_active(TRUE)
        self.newSvcRadiobutton.set_active(FALSE)

        self.existSvcRadiobutton.connect("clicked", self.on_svcRadiobuttonChanged)
        self.newSvcRadiobutton.connect("clicked", self.on_svcRadiobuttonChanged)

        self.svcTreeview = self.glade_xml.get_widget ('smbExistSvcTreeview')
        ### Just a simple list - no children, so use 'ListStore' (not TreeStore)
        self.svcModel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
        self.svcTreeview.set_model(self.svcModel)
        column = gtk.TreeViewColumn(_("Name"),
                                    gtk.CellRendererText(),
                                    text=COLUMN_NAME)
        self.svcTreeview.append_column(column)


        ##########################
        ### Select Device Page ###
        ##########################
        self.deviceServiceLabel = self.glade_xml.get_widget("smbDeviceServiceLabel")
        self.existDeviceRadiobutton = self.glade_xml.get_widget("smbExistDeviceRadiobutton")
        self.newDeviceRadiobutton = self.glade_xml.get_widget("smbNewDeviceRadiobutton")
        self.newDeviceNameEntry = self.glade_xml.get_widget("smbNewDeviceSpecialFileEntry")
        self.newDeviceMountpointEntry = self.glade_xml.get_widget("smbNewDeviceMountpointEntry")

        self.existDeviceRadiobutton.connect("clicked", self.on_deviceRadiobuttonChanged)
        self.newDeviceRadiobutton.connect("clicked", self.on_deviceRadiobuttonChanged)

        self.deviceTreeview = self.glade_xml.get_widget ('smbExistDeviceTreeview')
        ### Just a simple list - no children, so use 'ListStore' (not TreeStore)
        self.deviceModel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT)
        self.deviceTreeview.set_model(self.deviceModel)
        column = gtk.TreeViewColumn(_("Name"),
                                    gtk.CellRendererText(),
                                    text=COL_DEVICE_NAME)
        self.deviceTreeview.append_column(column)
        column = gtk.TreeViewColumn(_("Mountpoint"),
                                    gtk.CellRendererText(),
                                    text=COL_DEVICE_MOUNTPOINT)
        self.deviceTreeview.append_column(column)


        self.glade_xml.get_widget("smbDevicePage").connect("next", self.on_devicePage_next)

        #######################
        ### Share Name Page ###
        #######################
        self.shareNameEntry = self.glade_xml.get_widget("smbShareNameEntry")
        self.glade_xml.get_widget("smbShareNamePage").connect("next", self.on_shareNamePage_next)

        ##########################
        ### Last Page - Finish ###
        ##########################
        self.finishPage = self.glade_xml.get_widget("addSMBDruidFinish")
        self.finishPage.connect("finish",self.on_finish)

    def on_svcRadiobuttonChanged(self, button):
        if self.existSvcRadiobutton.get_active():
            ### 'Existing Service' was selected, so
            ###   * turn on 'Existing'
            ###   * turn off 'New'
            self.svcTreeview.set_sensitive(TRUE)
            self.newSvcNameEntry.set_sensitive(FALSE)
            self.newSvcIPAddressEntry.set_sensitive(FALSE)
        else:
            ### 'New Service' was selected, so
            ###   * turn on 'New'
            ###   * turn off 'Existing'
            self.newSvcNameEntry.set_sensitive(TRUE)
            self.newSvcIPAddressEntry.set_sensitive(TRUE)
            self.svcTreeview.set_sensitive(FALSE)

    def deviceRadiobuttonChanged(self, new_or_existing):
        if new_or_existing == "existing":
            ### 'Existing Device' was selected, so
            ###   * turn on 'Existing'
            ###   * turn off 'New'
            self.deviceTreeview.set_sensitive(TRUE)
            self.newDeviceNameEntry.set_sensitive(FALSE)
            self.newDeviceMountpointEntry.set_sensitive(FALSE)
        else:
            ### 'New Device' was selected, so
            ###   * turn on 'New'
            ###   * turn off 'Existing'
            self.newDeviceNameEntry.set_sensitive(TRUE)
            self.newDeviceMountpointEntry.set_sensitive(TRUE)
            self.deviceTreeview.set_sensitive(FALSE)

    def on_deviceRadiobuttonChanged(self, *args):
        "Invoked when one of the Device radiobuttons is clicked"
        if self.existDeviceRadiobutton.get_active():
            self.deviceRadiobuttonChanged(new_or_existing="existing")
        else:
            self.deviceRadiobuttonChanged(new_or_existing="new")

    def cancel(self,button):
        self.dlg.response(gtk.RESPONSE_CANCEL)
                                                                                
    def run(self):
        ### prep for Service page ###
        self.svcModel.clear()
        services_list = self.services_entity.get_list()
        self.count_services = 0
        for s in services_list:
            service_ipaddresses = s.get_service_ipaddresses()
            if len(service_ipaddresses) == 0:
                continue # don't load service if it doesn't have an ip
            self.count_services += 1
            service_name = s.get("name")
            iter = self.svcModel.append()
            self.svcModel.set(iter,
                              COLUMN_NAME, service_name,
                              COLUMN_DATA, s)

        ### initialize screen fields ###
        self.newSvcNameEntry.set_text("")
        self.newSvcIPAddressEntry.set_text("")
        self.deviceModel.clear()
        self.newDeviceNameEntry.set_text("")
        self.newDeviceMountpointEntry.set_text("")
        self.shareNameEntry.set_text("")

        self.first_time_service = TRUE
        self.first_time_device = TRUE

        self.on_svcRadiobuttonChanged(None)
        self.existSvcRadiobutton.grab_focus()

        self.druid.set_page(self.page1)
        self.druid.show_all()
        rc=self.dlg.run()
        self.dlg.hide()
        return rc

    def on_StartPage_next(self, *args):
        if not self.first_time_service:
            return # return to druid

        if self.count_services > 0:
            self.existSvcRadiobutton.set_active(TRUE) # after connect, causes
                                        # on_svcRadiobutton_changed to be fired
        else:
            self.newSvcRadiobutton.set_active(TRUE)


    def __validate_ipaddress(self, ip_to_validate):
        dup_found = FALSE
        for service in self.services_entity.get_list():
            ipaddress_entities = service.get_service_ipaddresses()
            for ipaddress_entity in ipaddress_entities:
                if ipaddress_entity.get("ipaddress") == ip_to_validate:
                    dup_found = TRUE
                    break
            if dup_found:
                break
        if dup_found:
            errorMessage(_("Service IP Address already exists in cluster"))
            return FALSE

        ### This whole section copied from service_ipaddress_module.py ###
        if hostInDNS(ip_to_validate) == None:
            errorMessage(_("Service IP Address not found in DNS"))
            return FALSE

        netmask = ""
        ipconvert = ipConvert()
        if not ipconvert.ipIsInSameNetAsHost(ip_to_validate, netmask):
            errorMessage(_("Service IP Address not on same network as this host"))
            return FALSE

        if cluPing(ip_to_validate):
            errorMessage(_("Service IP Address already in use on network"))
            return FALSE

        return TRUE # Passed validation

    def on_ServicePage_next(self, *args):
        self.first_time_service = FALSE
        if self.existSvcRadiobutton.get_active():
            ### 'Existing Service' selected ###
            selection = self.svcTreeview.get_selection()
            (model, iter) = selection.get_selected()
            if iter == None:
                errorMessage(_("When 'Existing Service' is selected, you must select one of the existing services"))
                return TRUE
            self.service_name = model.get_value(iter, COLUMN_NAME)
            self.service      = model.get_value(iter, COLUMN_DATA)

            service_id = self.service.get("id")
            clustat = cluStat()
            if clustat.serviceIsRunning(service_id):
                errorMessage(_("Service must be disabled before updates can be made"))
                return TRUE

            self.existing_service = TRUE
            self.new_service = FALSE

            ### Set Device Model before display Device page ###
            self.deviceModel.clear()
            service = self.service
            devices_list = service.get_devices()
            exist_svc_num_devices = len(devices_list)
            for d in devices_list:
                device_name = d.get("name")
                mount_entity = d.get_mount()
                device_mountpoint = mount_entity.get("mountpoint")
                iter = self.deviceModel.append()
                self.deviceModel.set(iter,
                                     COL_DEVICE_NAME, device_name,
                                     COL_DEVICE_MOUNTPOINT, device_mountpoint,
                                     COL_DEVICE_DATA, d)
        else:
            ### 'New Service' selected ###
            self.existing_service = FALSE
            self.new_service = TRUE

            self.service_name = self.newSvcNameEntry.get_text()
            self.service_ipaddress = self.newSvcIPAddressEntry.get_text()
            if self.service_name == "":
                errorMessage(_("When 'New Service' is selected, you must enter a name for the new service"))
                return TRUE
            if self.service_name == "":
                errorMessage(_("When 'New Service' is selected, you must enter an ip address for the new service"))
                return TRUE

            ### Verify that the new service name does not already exist ###
            search_options = {"name" : self.service_name}
            if self.services_entity.service_found(search_options, service_being_validated=None):
                errorMessage(_("New Service name already exists"))
                return TRUE

            ### Validate IP Address ###
            passed_validation = self.__validate_ipaddress(self.service_ipaddress)
            if not passed_validation:
                return TRUE # return to druid

        ####################################################
        ### Service Page done - now prep for Device Page ###
        ####################################################
        if not self.first_time_device:
            return # return to druid

        if self.existing_service:
            ### User selected an existing service ###
            label = _("Existing Service '%s' was selected") % self.service_name
            self.deviceServiceLabel.set_text(label)

            if exist_svc_num_devices > 0:
                ### The existing service DOES have devices defined.

                ### Enable choice of existing 'device' from that existing svc
                self.existDeviceRadiobutton.set_sensitive(TRUE)

                self.existDeviceRadiobutton.set_active(TRUE)
                self.existDeviceRadiobutton.grab_focus()
                self.deviceRadiobuttonChanged(new_or_existing="existing")
            else:
                ### The existing service selected has NO devices defined.
                ### So treat it like the "New" radiobutton was selected.
                self.deviceRadiobuttonChanged(new_or_existing="new")
                self.existDeviceRadiobutton.set_sensitive(FALSE)
                self.newDeviceRadiobutton.set_active(TRUE)
        else:
            ### User added a new service, so only allow user to
            ### *add a new* device
            label = _("New Service '%s' was specified") % self.service_name
            self.deviceServiceLabel.set_text(label)
            self.existDeviceRadiobutton.set_sensitive(FALSE)

            #self.existDeviceRadiobutton.set_active(FALSE)
            self.newDeviceRadiobutton.set_active(TRUE)
            self.newDeviceNameEntry.grab_focus()
            self.deviceRadiobuttonChanged(new_or_existing="new")

    def on_devicePage_next(self, *args):
        self.first_time_device = FALSE
        if self.existDeviceRadiobutton.get_active():
            ### 'Existing Device' selected ###
            self.existing_device = TRUE
            self.new_device = FALSE

            selection = self.deviceTreeview.get_selection()
            (model, iter) = selection.get_selected()
            if iter == None:
                errorMessage(_("When 'Existing Device' is selected, you must select one of the existing devices"))
                return TRUE
            self.device_name       = model.get_value(iter, COL_DEVICE_NAME)
            self.device_mountpoint = model.get_value(iter, COL_DEVICE_MOUNTPOINT)
            self.device            = model.get_value(iter, COL_DEVICE_DATA)
            if not self.device.get("sharename") in ("", "None", "none"):
                errorMessage(_("Selected Device already contains a Samba Share Name - please select a different device"))
                return TRUE
            self.shareNameEntry.set_text(self.device.get("sharename"))
        else:
            ### 'New Device' selected ###
            self.existing_device = FALSE
            self.new_device = TRUE

            self.device_name       = self.newDeviceNameEntry.get_text()
            self.device_mountpoint = self.newDeviceMountpointEntry.get_text()
            if self.device_name == "":
                errorMessage(_("Special File name is required for a new device"))
                return TRUE
            try:
                mode = os.stat(self.device_name)[stat.ST_MODE]
            except OSError, e:
                errorMessage(_("Device Special File \"%s\" does not exist") % self.device_name)
                return TRUE
            if not stat.S_ISBLK(mode):
                errorMessage(_("Device Special File must be a block special device file"))
                return TRUE

            if self.device_mountpoint == "":
                errorMessage(_("Device Mountpoint is required for a new device"))
                return TRUE
            try:
                mode = os.stat(self.device_mountpoint)[stat.ST_MODE]
            except OSError, e:
                #errorMessage(_("Mount Point does not exist"))
                #return TRUE
                msg = _("Device Mountpoint '%s' does not exist, but will be created when the service is started.") % self.device_mountpoint
                rc = warningMessage(msg)
                if rc != gtk.RESPONSE_OK:
                   return TRUE

            #if not stat.S_ISDIR(mode):
            #    errorMessage(_("Mount Point must be a directory"))
            #    return TRUE

            ### Verify that neither
            ###   * the new device name, nor
            ###   * the new device mountpoint
            ### already exists in the cluster.
            for service in self.services_entity.get_list():
               devices = service.get_devices()
               for device in devices:
                  if device.get("name") == self.device_name:
                      errorMessage(_("Device Special File already exists in cluster"))
                      return TRUE
                  mount = device.get_mount()
                  mountpoint = mount.get("mountpoint")
                  if mountpoint == self.device_mountpoint:
                      errorMessage(_("Device mountpoint already exists in cluster"))
                      return TRUE

            ### New device, so set sharename to null ###
            self.shareNameEntry.set_text("")

        self.shareNameEntry.grab_focus()

    def on_shareNamePage_next(self, *args):
        sharename = self.shareNameEntry.get_text()
        if sharename == "":
            errorMessage(_("Share Name is required"))
            return TRUE # return to druid

        self.sharename = sharename

        if string.find(sharename, ' ') != -1:
            errorMessage(_("Share Name may not contain spaces"))
            return TRUE # return to druid

        self.sharename = sharename
        ### Verify that sharename isn't a dup of existing ###
        for service in self.services_entity.get_list():
            devices = service.get_devices()
            for device in devices:
                sharename = device.get("sharename")
                if sharename == self.sharename:
                    errorMessage(_("Samba Share Name already exists in cluster"))
                    return TRUE # return to druid

        ############################################################
        ### Done with Share Name Page - Now prep for Finish Page ###
        ############################################################
        if self.existing_device:
            existing_sharename = self.device.get("sharename")
            if existing_sharename in ("", "None", "none"):
                existing_sharename = ""
        else:
            existing_sharename = ""

        ### We know that self.sharename is not null ###
        if existing_sharename == "":
            ### sharename being added ###
            abs_share_filename = getAbsShareFilename(self.sharename)
            finish_text = _("""
When you press 'Apply' below, and then
File => Save on the menu, samba configuration
file

    %s

will be created ***ON THIS MEMBER ONLY***.

Remember to copy the samba configuration file
to *ALL* other cluster members.
""") % abs_share_filename

        else:
            ### sharename being changed ###
            abs_share_filename = getAbsShareFilename(self.sharename)
            abs_share_filename_before = getAbsShareFilename(existing_sharename)
            finish_text = _("""
When you press 'Apply' below, and then
File => Save on the menu, the following
will occur:

  1. file '%s'
     will be renamed to '%s'

  2. inside that file, all occurrences of
     '%s' will be changed to '%s'

These changes will be made ***ON THIS MEMBER ONLY***.

Remember to copy the new samba configuration file
to, and remove the old samba configuration file from,
 *ALL* other cluster members.
""") % (abs_share_filename_before, abs_share_filename, existing_sharename, self.sharename)

        self.finishPage.set_text(finish_text)

    def on_finish(self, *args):
        #print "\nSmb Druid Finish now"
        #print "--------------------"
        #print "  Svc name      = [%s]" % self.service_name
        #print "  Svc existing  = [%s]" % self.existing_service
        #print "  Svc new       = [%s]" % self.new_service
        #print
        #print "  Dev name      = [%s]" % self.device_name
        #print "  Dev mount     = [%s]" % self.device_mountpoint
        #print "  Dev sharename = [%s]" % self.sharename
        if self.new_service:
            service = self.services_entity.add_service()
            service.set("name", self.service_name)
            service.change_action_to_modify() # for "validate"
            self.service = service

            service_ipaddress_entity = service.add_service_ipaddress()
            service_ipaddress_entity.set("ipaddress", self.service_ipaddress)

        if self.new_device:
            device = self.service.add_device()
            device.set("name", self.device_name)
            self.device = device

            mount = device.get_mount()
            mount.set("mountpoint", self.device_mountpoint)

        ### do 'df [smbexport dir]' to get device ###
        #dir_df = cluDF(self.dirname)
        #filesystem = dir_df.get_filesystem()
        #mountpoint = dir_df.get_mountpoint()
        #print "\ndf filesystem=[%s]" % filesystem
        #print "\ndf mountpoint=[%s]" % mountpoint

        ### Now add the SMB Sharename to the Device ###
        self.device.set("sharename", self.sharename)

        self.dlg.response(gtk.RESPONSE_OK)
