#!/usr/bin/python
import sys

DEFAULT_POLLINTERVAL = "4"
DEFAULT_CONFIG_VIEWNUMBER = "1"

PROGNAME="convert-cluster"
import gettext
try:
    gettext.install(PROGNAME, "/usr/share/locale", 1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode    

if not "/usr/lib/python2.2/site-packages/rhpl" in sys.path:
    sys.path.append("/usr/lib/python2.2/site-packages/rhpl")
import string
import ConfCluster

    
def buildAttributes(ddic):
    attributes=""
    for key in ddic.keys():
        if isinstance(ddic[key],type("")):
            attributes= "%s %s=\"%s\"" %  (attributes,key.lower(),ddic[key])
    return attributes

    
def getMembersXML(members,powercontrollers):
    rec=""

    pzero=powercontrollers["powercontroller0"]
    ### powercontroller types can be UPPER case in old cluster.conf, so
    ### convert them to lower case here.
    pzero["type"] = string.lower(pzero["type"])

    for key in members.keys():
        m=members[key]
        if "id" in m:
            rec='%s\t\t<member id="%s" name="%s" watchdog="yes">\n' % (rec,m["id"],m["name"])
        else:
            raise KeyError, "members"

	if "powerSwitchIPaddr" in m and "IPaddr" in pzero and pzero["type"] != "rps10" and m["powerSwitchIPaddr"] == pzero["IPaddr"] and pzero["IPaddr"] != "unused":
	    #
	    # Single Network Power Switch.
	    #
	    p=pzero
	else:
	    #
	    # Multiple network power switches.
	    #
            p=powercontrollers["powercontroller"+members[key]["id"]]
            p["type"] = string.lower(p["type"])

        if p["type"] != "null" and p["type"] != "sw_watchdog":
            #
            # added 8/25/2003 HM - convert 'apcplus' to 'apcmaster'
            #
            if p["type"] == "apcplus":
                p["type"] = "apcmaster"

            attr='id="0" type="%s"' % (p["type"])
        
            #
            # The 'login' was used as the /dev/ttyS* device in the old version
            # of RHCM.  Filter out "unused" things...
            #
            if "IPaddr" in p:
                attr=attr+' ipaddress=\"%s\"' % p["IPaddr"]

                if "login" in p and p["login"] != "unused":
                        attr=attr+' login=\"%s\"' % p["login"]
            else:
                if "login" in p and p["login"] != "unused":
                    attr=attr+' device=\"%s\"' % p["login"]
    
            if "passwd" in p and p["passwd"] != "unused":
                attr=attr+' password=\"%s\"' % p["passwd"]
            #
            # The 'power switch port name' was stored as a member attribute in
            # the old version of RHCM.
            #
            if "powerSwitchPortName" in m and m["powerSwitchPortName"] != "unused":
                attr=attr+' port=\"%s\"' % m["powerSwitchPortName"]

            rec="%s\t\t\t<powercontroller %s/>\n" % (rec,attr)

        rec=rec+"\t\t</member>\n"
    return rec

    
def getSharedStateXML(members):
    rec=""
    qp=""
    qs=""
    for key in members.keys():
        m=members[key]
        if ("quorumPartitionPrimary" in m) and ("quorumPartitionShadow" in m):
            #
            # Use the first member's quorum partition definitions.
            # Complain if the quorum partition definitions do not match.
            #
            if qp == "":
                qp=m["quorumPartitionPrimary"]
            elif qp != m["quorumPartitionPrimary"]:
                print _("Warning: Raw Primary Partition mismatch: %s != %s") % (qp, m["quorumPartitionPrimary"])
                print _("Warning: Cluster will use %s") % qp
                if "name" in m:
                     print _("Warning: Reconfigure member %s to match.") % m["name"]

            if qs == "":
                qs=m["quorumPartitionShadow"]
            elif qs != m["quorumPartitionShadow"]:
                print _("Warning: Raw Shadow Partition mismatch: %s != %s") % (qs, m["quorumPartitionShadow"])
                print _("Warning: Cluster will use %s") % qs
                if "name" in m:
                     print _("Warning: Reconfigure member %s to match.") % m["name"]

            if rec == "":
                rec='<sharedstate driver="libsharedraw.so" type="raw" rawprimary="%s" rawshadow="%s"/>' % (qp,qs)
    return rec

    
def getNetworksXML(ddic):
    rec=""
    ctr=0
    for key in ddic.keys():
        if key.find("network")==0:
            rec='%s\t\t\t\t<service_ipaddress id=\"%d\" %s/>\n' % (rec,ctr, buildAttributes(ddic[key]))
            ctr=ctr+1
    return "\t\t<service_ipaddresses>\n%s\t\t\t</service_ipaddresses>\n" % rec

    
def getNFSClientsXML(ddic):
    rec=""
    ctr=0
    for key in ddic.keys():
        if key.find("client")==0:
            rec='%s\t\t\t\t\t<client id=\"%d\" %s/>\n' % (rec,ctr,buildAttributes(ddic[key]))
	    ctr=ctr+1
    return rec

    
def getNFSExportsXML(ddic):
    rec=""
    ctr=0
    for key in ddic.keys():
        if key.find("directory")==0:
            rec='%s\t\t<nfsexport id=\"%d\" %s>\n' % (rec,ctr,buildAttributes(ddic[key]))
            ctr=ctr+1
            clients=ddic[key]
            if "client0" in clients.keys():
                rec="%s%s" % (rec,getNFSClientsXML(clients))
            rec=rec+"\t\t\t\t</nfsexport>"
    return rec

    
def getDevicesXML(ddic):
    rec=""
    ctr=0
    for key in ddic.keys():
        if key.find("device")==0:
            rec='%s\t\t<device id=\"%d\" %s>\n' % (rec,ctr, buildAttributes(ddic[key]))
            ctr=ctr+1
            if "mount" in ddic[key]:
                mounts=ddic[key]["mount"]
                mountAttributes = buildAttributes(mounts)
                mountAttributes = string.replace(mountAttributes, ' name=\"', ' mountpoint=\"')
                rec="%s\t\t\t\t<mount %s/>\n" % (rec,mountAttributes)
                if "NFSexports" in mounts:
                    exports=mounts["NFSexports"]
                    #rec="%s\t\t%s/>\n" % (rec,getNFSExportsXML(exports))
                    rec="%s\t\t%s\n" % (rec,getNFSExportsXML(exports))
            rec=rec+"\t\t\t</device>\n"
    return rec

    
def getServicesXML(services):
    rec=""
    ctr=0
    for key in services.keys():
        rec='%s\t\t<service id=\"%d\" %s failoverdomain="None">\n' % (rec,ctr,buildAttributes(services[key]))
        ctr=ctr+1
        if "network0" in services[key].keys():
            rec="%s\t%s" % (rec, getNetworksXML(services[key]))
        if "device0" in services[key].keys():
            rec="%s\t%s" % (rec, getDevicesXML(services[key]))
        rec=rec+"\t\t</service>\n"
    return "<services>\n%s\t</services>" % rec

    
def convert(file):
    conf=ConfCluster.ConfCluster(file)

    rec= '<?xml version="1.0" ?>\n<cluconfig version="3.0">\n'

    #####################
    ### 'cluster' tag ###
    #####################
    clusterAttributes = buildAttributes(conf.vars["cluster"])
    ### add 'config_viewnumber' option ###
    clusterAttributes += " config_viewnumber=\"%s\"" % DEFAULT_CONFIG_VIEWNUMBER
    rec="%s\t<cluster %s/>\n" % (rec, clusterAttributes)

    #
    # Get daemons
    #
    #daemons=["cluquorumd","clurmtabd","clusvcmgrd"]
    #for d in daemons:
    #    rec="%s\t<%s%s/>\n" %  (rec,d,buildAttributes(conf.vars[d]))
    #########################
    ### 'clumembd' daemon ###
    #########################
    rec="%s\t%s\n" % (rec,'<clumembd broadcast="no" interval="750000" loglevel="4" multicast="yes" multicast_ipaddress="225.0.0.11" tko_count="20"/>')

    #########################
    ### 'clulockd' daemon ###
    #########################
    rec="%s\t%s\n" % (rec,'<clulockd loglevel="4"/>')

    ###########################
    ### 'cluquorumd' daemon ###
    ###########################
    rec="%s\t<%s%s/>\n" %  (rec, 'cluquorumd', buildAttributes(conf.vars['cluquorumd']))

    ##########################
    ### 'clurmtabd' daemon ###
    ##########################
    clurmtabdAttributes = buildAttributes(conf.vars['clurmtabd'])
    if string.find(clurmtabdAttributes, 'pollinterval') == -1:
        ### 'pollinterval' option was not included, so provide one
        clurmtabdAttributes += " pollinterval=\"%s\"" % DEFAULT_POLLINTERVAL
    rec="%s\t<%s%s/>\n" %  (rec, 'clurmtabd', clurmtabdAttributes)

    ###########################
    ### 'clusvcmgrd' daemon ###
    ###########################
    rec="%s\t<%s%s/>\n" %  (rec, 'clusvcmgrd', buildAttributes(conf.vars['clusvcmgrd']))

    #
    # Get the member + switch information from the members section.
    #    
    rec="%s\t<members>\n%s\t</members>\n" % (rec,getMembersXML(conf.vars["members"],conf.vars["powercontrollers"]))

    #
    # Get the shared state information from the members section.
    #
    rec="%s\t%s\n" % (rec,getSharedStateXML(conf.vars["members"]))

    #
    # Get the service information from the members section.
    #
    try:
        rec="%s\t%s\n" % (rec,getServicesXML(conf.vars["services"]))
    except:
        rec=rec+"\t<services/>\n"

    rec=rec+"\t<failoverdomains/>\n"
    return rec+"</cluconfig>\n"

    
def Usage():
    print _("cluster-convert.py - convert old style cluster conf to cluster.xml\n\nUsage: cluster-convert.py inputfile outputfile")

    
if __name__ == "__main__":
    if len(sys.argv) < 3:
        Usage()
        sys.exit(1)
    try:
        out=convert(sys.argv[1])
    except KeyError,e:
        print "%s selection invalid!" % e
    else:
        fd=open(sys.argv[2],"w")
        fd.write(out)
        fd.close()
