#!/usr/bin/python
import sys
import getopt
import os
import stat
import signal
from time import sleep
CLUSTERCONFIG_APP="/usr/share/redhat-config-cluster/configure/cluster-config.py"
CLUSTERCONFIG_FILE="/etc/cluster.xml"

CLUSTER_START_RETRY_SECS=5

#Columns for "treeview"
COLUMN_PIXBUF=0
COLUMN_DATA=1

COLUMN_NAME=2
COLUMN_STATE=3

COLUMN_PIXBUF=0
COLUMN_STATE=3
COLUMN_NODE=4
COLUMN_LAST_TRANSITION=5
COLUMN_MONITOR_INTERVAL=6
COLUMN_RESTARTS=7

COLUMN_TYPE=3

COLUMN_IPADDRESS=0
COLUMN_NETMASK=1
COLUMN_BROADCAST=2

COLUMN_HOST=0
COLUMN_OPTIONS=1

COLUMN_PSNAME=0
COLUMN_PSDEVICE=1
COLUMN_PSPORT=2
COLUMN_PSIP=3
COLUMN_PSLOGIN=4

#INSTALLDIR="."
INSTALLDIR="/usr/share/redhat-config-cluster"
CONFIGDIR="/usr/share/redhat-config-cluster/configure"
sys.path.append(CONFIGDIR)
from clusterpkg.cluconfig_module  import cluconfig
from clusterpkg.cluster_functions import getStatusOutput, cluStat
from clusterpkg.util_lock_module  import lockFile
from ForkedCommand import ForkedCommand

PATIENCE_MESSAGE=_("Please be patient.\n Starting and Stopping Services\n can sometimes take a minute or two.")

TRUE=1
FALSE=0

quiting=0
PROGNAME='redhat-config-cluster'
VERSION = "1.0.8"

RHCC_VERSION=_("%s %s") % (PROGNAME, VERSION)
### Get Clumanager version to present in 'About' ###
(status, output) = getStatusOutput("clustat -v")
if status == 0:
    CLUMANAGER_VERSION = _("clumanager %s") % output
else:
    CLUMANAGER_VERSION = ""
if CLUMANAGER_VERSION != "":
    ABOUT_VERSION = RHCC_VERSION + "\n" + CLUMANAGER_VERSION
else:
    ABOUT_VERSION = RHCC_VERSION

### gettext ("_") must come before import gtk ###
import gettext
gettext.bindtextdomain(PROGNAME, "/usr/share/locale")
gettext.textdomain(PROGNAME)
try:
    gettext.install(PROGNAME, "/usr/share/locale", 1)
except IOError:
    import __builtin__
    __builtin__.__dict__['_'] = unicode    

### gettext first, then import gtk (exception prints gettext "_") ###
try:
    import gtk
    import gtk.gdk
except RuntimeError,e:
    print _("""
  Unable to initialize graphical environment. Most likely cause of failure
  is that the tool was not run using a graphical environment. Please either
  start your graphical user interface or set your DISPLAY variable.

  Caught exception: %s
""") % e
    sys.exit(-1)

import gnome
import gnome.ui
import socket
import gobject
import gtk.glade

gnome.program_init (PROGNAME,VERSION)
gnome.app_version =VERSION
FORMALNAME=_("redhat-config-cluster")

def Usage():
    print _("""
redhat-config-cluster - Python GUI to configure a Red Hat Clumanager cluster.

See redhat-config-cluster(8) for more information.
""")

busy_cursor = gtk.gdk.Cursor(gtk.gdk.WATCH)
ready_cursor = gtk.gdk.Cursor(gtk.gdk.LEFT_PTR)

python_code_targets = [('execable_python', 0, 0),
                       ('pickled_python', 0, 1)];

def getValue(node,var):
    try:
        return node.getElementsByTagName(var)[0].firstChild.data
    except:
        return ""

def getAttribute(node,var):
    try:
        return node.getAttribute(var)
    except:
        return ""

class PowerController:
    def __init__(self,node):
        self.node=node
    def getType(self):
        return getAttribute(self.node,"type")
    def getDevice(self):
        return getAttribute(self.node,"device")
    def getIPAddress(self):
        return getAttribute(self.node,"ipaddress")
    def getPort(self):
        return getAttribute(self.node,"port")
    def getPassword(self):
        return getAttribute(self.node,"password")
    def getUser(self):
        return getAttribute(self.node,"user")
    
class Member:
    #stateTable=[_("Down"),_("Up"), _("Unknown")] 
    stateTable=[_("Inactive"),_("Active"), _("Unknown")] 
    def __init__(self,node):
        self.node=node
    def getName(self):
        return getAttribute(self.node,"name")
    def getState(self):
        state = getAttribute(self.node,"state")
        if state == "": # 'state' attribute doesn't exist, or has null value
            return 2
        try:
            return int(state)
        except:
            return 2
    def getStateString(self):
        return self.stateTable[self.getState()]
    def getPowerControllers(self):
        l=[]
        eElements=self.node.getElementsByTagName("powercontroller")
        for i in eElements:
            l.append(PowerController(i))
        return l
        

class Client:
    def __init__(self,node):
        self.node=node
    def getName(self):
        return getAttribute(self.node,"name")
    def getOptions(self):
        return getAttribute(self.node,"options")

class IPAddress:
    def __init__(self,node):
        self.node=node
    def getIPAddress(self):
        return getAttribute(self.node,"ipaddress")
    def getNetmask(self):
        return getAttribute(self.node,"netmask")
    def getBroadcast(self):
        return getAttribute(self.node,"broadcast")

class Export:
    def __init__(self,node):
        self.node=node
    def getName(self):
        return getAttribute(self.node,"name")
    def getOptions(self):
        return ""
    def getChildren(self):
        eElements=self.node.getElementsByTagName("client")
        l=[]
        for i in eElements:
            l.append(Client(i))
        return l

class Device:
    def __init__(self,node):
        self.node=node
    def getName(self):
        return getAttribute(self.node,"name")
    def getMountFstype(self):
        return getAttribute(self.node,"mountFstype")
    def getMountPoint(self):
        return getAttribute(self.node,"mountPoint")
    def getMountOptions(self):
        return getAttribute(self.node,"mountOptions")
    def getForceUnmount(self):
        return getAttribute(self.node,"forceunmount")
    def getOwner(self):
        return getAttribute(self.node,"owner")
    def getGroup(self):
        return getAttribute(self.node,"group")
    def getMode(self):
        return getAttribute(self.node,"mode")
    def getSambaShare(self):
        return getAttribute(self.node,"sambaShare")
    def getChildren(self):
        eElements=self.node.getElementsByTagName("nfsexport")
        l=[]
        for i in eElements:
            l.append(Export(i))
        return l


class Service:
    stateTable=[_("Uninitialized"),
                _("Running"),
                _("Stopped"),
                _("Disabled"),
                _("Pending"),
                _("Failed"),
                _("Unknown")] 
    def __init__(self,node):
        self.node=node
    def getName(self):
        return getAttribute(self.node,"name")
    def getState(self):
        state = getAttribute(self.node,"state")
        unknown = 6
        if state == "": # 'state' attribute doesn't exist, or has null value
            return unknown
        try:
            return int(state)
        except:
            return unknown
    def getStateString(self):
        state = self.getState()
        #print "getStateString - self.getState() = <%s>" % state
        return self.stateTable[self.getState()]
    def getOwner(self):
        return getAttribute(self.node,"owner")
    def getLastOwner(self):
        return getAttribute(self.node,"lastowner")
    def getScript(self):
        return getAttribute(self.node,"userscript")
    def getFailoverDomain(self):
        return getAttribute(self.node,"failoverdomain")
    def getMonitorInterval(self):
        x=getAttribute(self.node,"checkinterval")
        if x=="":
            return _("Not Monitored")
        else:
            return x
    def getRestarts(self):
        restart_count = getAttribute(self.node,"restart_count")
        if restart_count == "": # option not present, or has null value
            return 0
        return int(restart_count)
    def getLastTransition(self):
        return getAttribute(self.node,"last_transition")
    def getIPAddresses(self):
        try:
            ipAddressesElement=self.node.getElementsByTagName("service_ipaddresses")[0]
        except IndexError, e:
            return [] # return an empty list
        ipElements=ipAddressesElement.getElementsByTagName("service_ipaddress")
        l=[]
        for i in ipElements:
            l.append(IPAddress(i))
        return l
    def getDevices(self):
        dElements=self.node.getElementsByTagName("device")
        l=[]
        for i in dElements:
            l.append(Device(i))
        return l

class Gui:
    """This class handles everything gui for the redhat-config-cluster application"""
    def __init__(self):

        signal.signal (signal.SIGINT, signal.SIG_DFL)


        # make sure threading is disabled
        try:
            from gtk import _disable_gdk_threading
            _disable_gdk_threading()
        except ImportError:
            pass
        self.dirty=0

        gladepath = "redhat-config-cluster.glade"
        if not os.path.exists(gladepath):
            gladepath = "%s/%s" % (INSTALLDIR,gladepath)
        gtk.glade.bindtextdomain(PROGNAME)
        self.xml = gtk.glade.XML(gladepath, domain=PROGNAME)

        # map the event signals to specific methods
        self.xml.signal_autoconnect(
            {
            "on_mainwin_delete_event" : self.quit,
            "on_mainwin_hide" : self.quit,
            "on_quit_activate" : self.quit,
            "on_about_activate" : self.on_about_activate,
            "on_contents_activate" : self.on_contents_activate,
            "on_start_cluster_activate" : self.on_start_cluster,
            "on_stop_cluster_activate" : self.on_stop_cluster,
            "on_enable_activate" : self.on_enable_service,
            "on_disable_activate" : self.on_disable_service,
            "on_restart_activate" : self.on_restart_service,
            "on_properties_activate" : self.on_servicesEditButton_clicked,
            "on_configure_activate" : self.configure,
              } )

        self.prevXML=""
        self.membersTreeview = self.xml.get_widget("membersTreeview")
        self.servicesTreeview = self.xml.get_widget("servicesTreeview")
        self.ipAddressTreeview = self.xml.get_widget("ipAddressTreeview")
        self.deviceTreeview = self.xml.get_widget("deviceTreeview")
        self.variableTreeview = self.xml.get_widget("variableTreeview")
        self.haveQuorum = self.xml.get_widget("quorumCheckbutton")
        self.clusterStatus = self.xml.get_widget("clusterStatusLabel")
        self.currentMember = self.xml.get_widget("currentMemberLabel")
        self.clusterName = self.xml.get_widget("clusterNameEntry")
        self.ownerOptionMenu = self.xml.get_widget("ownerOptionMenu")
        self.serviceStateOptionMenu = self.xml.get_widget("serviceStateOptionMenu")
        self.winMain = self.xml.get_widget("clustatApp")
        self.winMain.connect("delete-event", self.quit)
        self.winMain.connect("hide", self.quit)
        self.setupPSList()
        self.setupList()
        self.onTimer()

        ### Parse cluster.xml into DOM nodes, and do not lock the cluster.xml
        ### file, since this program only needs read access.
        self.clu = cluconfig(node=None, lock=FALSE)

        ### set current node label ###
        (status, output) = getStatusOutput("hostname")
        self.currentMember.set_text(output)

    def on_start_cluster(self, button):
        clumembd = self.clu.getclumembd()
        interval = clumembd.get("interval")
        tko_count = clumembd.get("tko_count")
        milliseconds = interval * tko_count
        seconds = milliseconds / 1000000
        max_seconds = seconds * 3
        ### When cluster is coming up from stopped, we need to allow
        ### enough time for the cluster to come all the way up.
        #print "on_start_cluster: interval =", interval
        #print "                  tko_count =", tko_count
        #print "                  milliseconds =", milliseconds
        #print "                  seconds =", seconds
        #print "                  max_seconds =", max_seconds

        #(status, output) = getStatusOutput("service clumanager start")
        rc = os.system("service clumanager start")
        ct_seconds = 0
        while 1:
            ct_seconds = ct_seconds + CLUSTER_START_RETRY_SECS
            if ct_seconds > max_seconds:
                break
            sleep(CLUSTER_START_RETRY_SECS)
            (status, output) = getStatusOutput("service clumanager status")
            if status == 0:
                break

        if ct_seconds > max_seconds:
            self.errorMessage(_("Error: Cluster did *NOT* come up as expected - will stop cluster now"))
            self.on_stop_cluster(button)

        ### Cluster came up as expected - now safe to call 'updateList' ###
        self.updateList()

    def on_stop_cluster(self, button):
        rc = os.system("service clumanager stop")
        self.updateList()

    def on_owner_changed(self,item,data):
        s=self.getCurrentService()
        errorstring = _("A Problem has occurred while changing ownership of this service. Please check logs for details")
        commandstring = "clusvcadm -r \"" + s.getName() + "\" -m " + item.get_data("name")
        self.grayOutMainWindow()
        fm = ForkedCommand(commandstring, PATIENCE_MESSAGE, errorstring, self.ungrayOutAndResetMainWindow)

    def onTimer(self):
        self.updateList()
#        gtk.timeout_add(60000,self.onTimer)
        gtk.timeout_add(10000,self.onTimer)
        
    def setWaitCursor(self):
        pass
#       self.winMain.window.set_cursor(busy_cursor)

    def setReadyCursor(self):
        pass
#       self.winMain.window.set_cursor(ready_cursor)

    def errorMessage(self,message):
        dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                                message)
        dlg.show_all()
        rc = dlg.run()
        dlg.destroy()
        return rc
        
    def getCurrentService(self):
        s=None
        selection=self.servicesTreeview.get_selection()
        result = selection.get_selected ()
        if result != None:
            (model, iter) = result
            try:
                s = model.get_value (iter, COLUMN_DATA)
            except:
                pass
        return s
    
    def on_disable_service(self,button):
        s=self.getCurrentService()
        if s != None:

          commandstring = "clusvcadm -d \"" + s.getName() + "\""
          errorstring = _(" An error has occurred while disabling this service. Please check logs for details.")
          self.grayOutMainWindow()
          fm = ForkedCommand(commandstring, PATIENCE_MESSAGE, errorstring, self.ungrayOutAndResetMainWindow)

    def on_enable_service(self,button):
        s=self.getCurrentService()
        if s != None:
            self.grayOutMainWindow()
            commandstring = "clusvcadm -e \"" + s.getName() + "\""
            errorstring = (_("Error: Service Enable failed - please check the logs for error messages.\n\nOnce the problem has been corrected, the 'Failed' service must first be Disabled before it can be Enabled."))
            fm = ForkedCommand(commandstring, PATIENCE_MESSAGE, errorstring, self.ungrayOutAndResetMainWindow)


    def on_restart_service(self,button):
        s=self.getCurrentService()
        if s != None:
            commandstring = "clusvcadm -R \"" + s.getName() + "\""
            errorstring = ""
            self.grayOutMainWindow()
            fm = ForkedCommand(commandstring, PATIENCE_MESSAGE, errorstring,self.ungrayOutAndResetMainWindow)

    def properties(self,button):
        pass

    def configure(self,button):
        if os.fork()==0:
            os.execl(CLUSTERCONFIG_APP,"")
        
    def show(self):
        self.winMain.show()

    def grayOutMainWindow(self):
      #Temporarily mothball main window
      watch = gtk.gdk.Cursor (gtk.gdk.WATCH)
      self.winMain.window.set_cursor(watch)
      self.winMain.set_sensitive(FALSE)

    def ungrayOutMainWindow(self):
      self.winMain.window.set_cursor(None)
      self.winMain.set_sensitive(TRUE)

    def ungrayOutAndResetMainWindow(self):
      self.winMain.window.set_cursor(None)
      self.winMain.set_sensitive(TRUE)
      self.updateList()
 

    def getMemberImage(self,status):
        if status==1:
            return gtk.gdk.pixbuf_new_from_file("%s/node_up.png" % INSTALLDIR)
        else:
            return gtk.gdk.pixbuf_new_from_file("%s/down.png" % INSTALLDIR)

    def getImage(self,status):
        if status==1:
            return gtk.gdk.pixbuf_new_from_file("%s/up.png" % INSTALLDIR)
        else:
            return gtk.gdk.pixbuf_new_from_file("%s/down.png" % INSTALLDIR)

    def populateServicesList(self):
        model=self.servicesTreeview.get_model ()
        prevName=""
        prevIter=None
        selected=self.servicesTreeview.get_selection().get_selected()
        if selected != None:
            (model, iter) = selected
            if iter != None:
                i = model.get_value (iter, COLUMN_DATA)
                prevName=i.getName()
        model.clear()
        hElements=self.dom.getElementsByTagName("service")
        for i in hElements:
            s=Service(i)
            iter = model.append ()
            state = s.getState()
            color="color='red'"
            if (state == 1):
                color=""
            if prevName==s.getName():
                prevIter=iter
            if s.getOwner() == "":
                last_owner = s.getLastOwner()
                if last_owner != "":
                    owner="(%s)" % last_owner
                else:
                    owner=""
            else:
                owner=s.getOwner()
                
            model.set (iter,
                       COLUMN_PIXBUF,  self.getImage(state),
                       COLUMN_NAME,  "<span %s>%s</span>" % (color,s.getName()),
                       COLUMN_STATE, "<span %s>%s</span>" % (color, s.getStateString()),
                       COLUMN_NODE,   "<span %s>%s</span>" % (color, owner),
                       COLUMN_LAST_TRANSITION,   "<span %s>%s</span>" % (color, s.getLastTransition()),
                       COLUMN_MONITOR_INTERVAL,   "<span %s>%s</span>" % (color, s.getMonitorInterval()),
                       COLUMN_RESTARTS,   "<span %s>%s</span>" % (color, s.getRestarts()),
                       COLUMN_DATA, s
                       )
        if prevIter!=None:
            #print prevIter
            selection=self.servicesTreeview.get_selection().select_iter (prevIter)

    def setOwnerAndServiceState(self, service):
        if service.getState() == 1:
            ### service is 'enabled'
            service_state = 0
        else:
            ### set service to 'disabled'
            service_state = 1
        self.ownerOptionMenu.set_sensitive(service_state==0)
        self.serviceStateOptionMenu.set_history(service_state)

    def populateMembersList(self):
        model=self.membersTreeview.get_model ()
        prevName=""
        prevIter=None

        owner=""
        service_selected=self.servicesTreeview.get_selection().get_selected()
        if service_selected != None:
            (model, iter) = service_selected
            if iter != None:
                s = model.get_value (iter, COLUMN_DATA)
                if s.getOwner() == "":
                    owner=s.getLastOwner()
                else:
                    owner=s.getOwner()
                self.setOwnerAndServiceState(s)

        selected=self.membersTreeview.get_selection().get_selected()
        if selected != None:
            (model, iter) = selected
            if iter != None:
                i = model.get_value (iter, COLUMN_DATA)
                prevName=i.getName()
        model.clear()
        memberElements=self.dom.getElementsByTagName("member")

        self.ownerOptionMenu.remove_menu()
        menu=gtk.Menu()
        
        ctr=0
        cur_ctr=0
        for mem in memberElements:
            iter = model.append ()
            m=Member(mem)
            item=gtk.MenuItem(m.getName())
            item.set_data("name",m.getName())
            item.show()
            item.connect("activate",self.on_owner_changed,None)
            menu.append(item)
            state = m.getState()
            color="color='red'"
            if (state == 1):
                color=""
            if prevName==m.getName():
                prevIter=iter
            if owner==m.getName():
                cur_ctr=ctr
            ctr=ctr+1
            model.set (iter,
                       COLUMN_PIXBUF,  self.getMemberImage(state),
                       COLUMN_NAME,  "<span %s>%s</span>" % (color,m.getName()),
                       COLUMN_STATE, "<span %s>%s</span>" % (color,m.getStateString()),
                       COLUMN_DATA, m
                       )
        menu.show()
        self.ownerOptionMenu.set_menu(menu)
        self.ownerOptionMenu.set_history(cur_ctr)
        if prevIter!=None:
            selection=self.membersTreeview.get_selection().select_iter (prevIter)

    def on_tree_view_row_activated (self,tree_view, path, column, func):
        apply (func)

    def on_tree_view_selection_changed (self,selection):
        response = selection.get_selected ()
        
    def on_servicesEditButton_clicked(self,*args):
        sDialog = self.xml.get_widget("serviceDialog")
        selection=self.servicesTreeview.get_selection()
        result = selection.get_selected ()
        if result == None:
            return 
        else:
            (model, iter) = result
            if iter==None:
                return
            try:
                s = model.get_value (iter, COLUMN_DATA)
                self.xml.get_widget("serviceNameEntry").set_text(s.getName())
                self.xml.get_widget("serviceScriptEntry").set_text(s.getScript())
                self.xml.get_widget("serviceFailoverDomainEntry").set_text(s.getFailoverDomain())
                self.xml.get_widget("serviceMonitorIntervalEntry").set_text(s.getMonitorInterval())
                self.xml.get_widget("serviceRestartsEntry").set_text(str(s.getRestarts()))
                self.xml.get_widget("serviceLastTimeEntry").set_text(s.getLastTransition())

                menu=self.ownerOptionMenu.get_menu()
                x=menu.get_children()
                i=0
                for item in menu.get_children():
                    if s.getOwner() == "":
                        ### Service is NOT running - 'owner' is not present,
                        ### so get "lastowner".
                        owner=s.getLastOwner()
                    else:
                        owner=s.getOwner()
                    if item.get_data("name") == owner:
                        break
                    i=i+1
                self.ownerOptionMenu.set_history(i)
                self.setOwnerAndServiceState(s)
                model=self.ipAddressTreeview.get_model ()
                model.clear()
                ipElements=s.getIPAddresses()
                for i in ipElements:
                    iter = model.append ()
                    model.set (iter,
                               COLUMN_IPADDRESS, i.getIPAddress(),
                               COLUMN_NETMASK,   i.getNetmask(),
                               COLUMN_BROADCAST, i.getBroadcast()
                               )

                model=self.deviceTreeview.get_model ()
                model.clear()
                dElements=s.getDevices()
                deviceImage=gtk.gdk.pixbuf_new_from_file("%s/device.png" % INSTALLDIR)
                exportImage=gtk.gdk.pixbuf_new_from_file("%s/export_nfs.png" % INSTALLDIR)
                for d in dElements:
                    iter = model.append (None)
                    model.set (iter,
                               COLUMN_PIXBUF,  deviceImage,
                               COLUMN_NAME,   d.getName(),
                               COLUMN_DATA,   d
                               )
                    for e in d.getChildren():
                        eiter = model.append (iter)
                        model.set (eiter,
                                   COLUMN_PIXBUF,  exportImage,
                                   COLUMN_NAME,   e.getName(),
                                   COLUMN_DATA,   e
                                   )
                status=sDialog.run()
                sDialog.hide()
            except TypeError,e:
                print e
                pass
                
    def on_deviceEditButton_clicked(self,*args):
        selection=self.deviceTreeview.get_selection()
        column=self.deviceTreeview.get_column(0)
        column.set_title("Name")
        result = selection.get_selected ()
        if result == None:
            return 
        else:
            (model, iter) = result
            try:
                d = model.get_value (iter, COLUMN_DATA)
                model=self.variableTreeview.get_model ()
                model.clear()
                hElements=d.getChildren()
                for h in hElements:
                    iter = model.append ()
                    model.set (iter,
                               COLUMN_HOST,   h.getName(),
                               COLUMN_OPTIONS,  h.getOptions(),
                               )

            except TypeError,e:
                print e
                pass
                
    def on_membersEditButton_clicked(self,*args):
        mDialog = self.xml.get_widget("memberDialog")
        selection=self.membersTreeview.get_selection()
        result = selection.get_selected ()
        if result == None:
            return 
        else:
            (model, iter) = result
            try:
                m = model.get_value (iter, COLUMN_DATA)
                self.xml.get_widget("memberNameEntry").set_text(m.getName())
                self.xml.get_widget("memberStatusEntry").set_text(m.getStateString())
                model=self.xml.get_widget("powerSwitchTreeview").get_model()
                model.clear()
                for p in m.getPowerControllers():
                    iter=model.append()
                    model.set(iter,
                              COLUMN_PSNAME,p.getType(),
                              COLUMN_PSDEVICE,p.getDevice(),
                              COLUMN_PSPORT,p.getPort(),
                              COLUMN_PSIP,p.getIPAddress(),
                              COLUMN_PSLOGIN,p.getUser())
                          
                status=mDialog.run()
                mDialog.hide()
            except TypeError:
                pass


    def dest_drag_data_received(self,w, context, x, y, selection_data, info, time):
        errorstring1 = ""
        errorstring2 = ""
        row = w.get_path_at_pos(x,y-25)[0][0]
        model=w.get_model()
        iter=model.get_iter_first()
        for i in range(row):
            iter=model.iter_next(iter)
        s = model.get_value (iter, COLUMN_DATA)
        #os.system("clusvcadm -r '%s' -m %s" % (selection_data.data,s.getName()))
        if s.getStateString() == 'Disabled':
            ### Service is 'Disabled' - just 'enable' on new member
            commandstring = "clusvcadm -e " + selection_data.data + " -m " + s.getName()
            self.grayOutMainWindow()
            fm = ForkedCommand(commandstring, PATIENCE_MESSAGE, errorstring1, self.ungrayOutAndResetMainWindow)
        else:
            ### Service is not 'Disabled' - restart it on new member
            commandstring = "clusvcadm -r \"" + selection_data.data + "\" -m " + s.getName()
            self.grayOutMainWindow()
            fm = ForkedCommand(commandstring, PATIENCE_MESSAGE, errorstring2, self.ungrayOutAndResetMainWindow)

        return

    # __ The callback that sends the drag-and-drop data
    def source_drag_data_get(self,w, context, selection_data, info, time):
        selection=w.get_selection()
        result = selection.get_selected ()
        if result != None:
            (model, iter) = result
            try:
                s = model.get_value (iter, COLUMN_DATA)
            except:
                pass
        selection_data.set(selection_data.target, 0, s.getName())
        return


    def setupMembersList(self):
        """Populates membersTreeview with member names"""
        model = gtk.ListStore (gtk.gdk.Pixbuf, gobject.TYPE_PYOBJECT, gobject.TYPE_STRING,gobject.TYPE_STRING, gobject.TYPE_STRING)
        model.set_sort_column_id(2, gtk.SORT_ASCENDING)        
        self.membersTreeview.set_model (model)
        column = gtk.TreeViewColumn (None, gtk.CellRendererPixbuf(), pixbuf=COLUMN_PIXBUF)
        self.membersTreeview.append_column (column)
        
        # __ The DnD stuff
        self.membersTreeview.drag_dest_set(gtk.DEST_DEFAULT_ALL, python_code_targets, gtk.gdk.ACTION_COPY)
        self.membersTreeview.connect('drag_data_received', self.dest_drag_data_received)

        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("Name"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_NAME)
        column.set_sort_column_id(2)
        self.membersTreeview.append_column (column)

        column = gtk.TreeViewColumn (_("Status"), None)
        column.set_sort_column_id(3)
        rend=gtk.CellRendererText()
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_STATE)
        self.membersTreeview.append_column (column)

        self.membersTreeview.get_selection().connect ("changed",
                                               self.on_tree_view_selection_changed)
        self.membersTreeview.connect ("row_activated",self.on_tree_view_row_activated,
                               self.on_membersEditButton_clicked)

    def setupIPAddressList(self):
        """Populates ipAddressTreeview with the IP Address, Netmask, and Broadcast"""
        model = gtk.ListStore (gobject.TYPE_STRING,gobject.TYPE_STRING, gobject.TYPE_STRING)
        model.set_sort_column_id(1, gtk.SORT_ASCENDING)        
        self.ipAddressTreeview.set_model (model)
        column = gtk.TreeViewColumn (_("Address"), gtk.CellRendererText(), text=COLUMN_IPADDRESS)
        column.set_sort_column_id(0)
        self.ipAddressTreeview.append_column (column)

        column = gtk.TreeViewColumn (_("Netmask"), gtk.CellRendererText(), text=COLUMN_NETMASK)
        column.set_sort_column_id(1)
        self.ipAddressTreeview.append_column (column)

        column = gtk.TreeViewColumn (_("Broadcast"), gtk.CellRendererText(), text=COLUMN_BROADCAST)
        column.set_sort_column_id(2)
        self.ipAddressTreeview.append_column (column)

    def setupVariableList(self):
        """Populates variableTreeview with Host and options"""
        model = gtk.ListStore (gobject.TYPE_STRING,gobject.TYPE_STRING)
        model.set_sort_column_id(1, gtk.SORT_ASCENDING)        
        self.variableTreeview.set_model (model)

        column = gtk.TreeViewColumn (_("Host"), gtk.CellRendererText(), text=COLUMN_HOST)
        column.set_sort_column_id(0)
        self.variableTreeview.append_column (column)

        column = gtk.TreeViewColumn (_("options"), gtk.CellRendererText(), text=COLUMN_OPTIONS)
        column.set_sort_column_id(1)
        self.variableTreeview.append_column (column)

    def setupDeviceList(self):
        """Populates deviceTreeview"""
        model = gtk.TreeStore (gtk.gdk.Pixbuf, gobject.TYPE_PYOBJECT, gobject.TYPE_STRING)
        self.deviceTreeview.set_model (model)

        column = gtk.TreeViewColumn (None, gtk.CellRendererPixbuf(), pixbuf=COLUMN_PIXBUF)
        self.deviceTreeview.append_column (column)

        column = gtk.TreeViewColumn (None, gtk.CellRendererText(), text=COLUMN_NAME)
        self.deviceTreeview.append_column (column)
        self.deviceTreeview.connect ("cursor_changed",self.on_deviceEditButton_clicked)

    def setupServicesList(self):
        """Populates servicesTreeview """
        model = gtk.ListStore (gtk.gdk.Pixbuf, gobject.TYPE_PYOBJECT, gobject.TYPE_STRING,gobject.TYPE_STRING, gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)
        model.set_sort_column_id(2, gtk.SORT_ASCENDING)        
        self.servicesTreeview.set_model (model)
        target = [('GLOOPNODE',0,-1)]
        self.servicesTreeview.drag_source_set(gtk.gdk.BUTTON1_MASK, python_code_targets, gtk.gdk.ACTION_COPY)
        self.servicesTreeview.connect('drag_data_get', self.source_drag_data_get)

        column = gtk.TreeViewColumn (None, gtk.CellRendererPixbuf(), pixbuf=COLUMN_PIXBUF)
        self.servicesTreeview.append_column (column)
        
        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("Name"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_NAME)
        column.set_sort_column_id(2)
        self.servicesTreeview.append_column (column)

        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("State"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_STATE)
        column.set_sort_column_id(3)
        self.servicesTreeview.append_column (column)

        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("Member"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_NODE)
        column.set_sort_column_id(4)
        self.servicesTreeview.append_column (column)

        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("Last\nTransition"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_LAST_TRANSITION)
        column.set_sort_column_id(5)
        self.servicesTreeview.append_column (column)

        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("Monitor\nInterval"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_MONITOR_INTERVAL)
        column.set_sort_column_id(6)
        self.servicesTreeview.append_column (column)

        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("Restarts"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_RESTARTS)
        column.set_sort_column_id(7)
        self.servicesTreeview.append_column (column)


        self.servicesTreeview.get_selection().connect ("changed",
                                               self.on_tree_view_selection_changed)
        self.servicesTreeview.connect ("row_activated",self.on_tree_view_row_activated,
                               self.on_servicesEditButton_clicked)

    def updateList(self):
        #self.prevXML=""
        clu_stat = cluStat()
        XML = clu_stat.getXML()
        if XML != self.prevXML:
            self.dom = clu_stat.getDOM()
            cluquorumdElement=self.dom.getElementsByTagName("cluquorumd")[0]
            have_quorum = int(cluquorumdElement.getAttribute("have_quorum"))
            self.haveQuorum.set_active(have_quorum)
            if clu_stat.clusterIsRunning():
                status_msg = _("Cluster is running")
                color = "dark green"
                self.xml.get_widget("start_cluster_menu").set_sensitive(FALSE)
                self.xml.get_widget("stop_cluster_menu").set_sensitive(TRUE)
            else:
                status_msg = _("Cluster is not running")
                color = "red"
                self.xml.get_widget("start_cluster_menu").set_sensitive(TRUE)
                self.xml.get_widget("stop_cluster_menu").set_sensitive(FALSE)

            self.clusterStatus.set_text(status_msg)
            self.clusterStatus.set_markup("<span foreground='%s'>%s</span>" %
                (color, status_msg))

            hElement=self.dom.getElementsByTagName("cluster")[0]
            self.clusterName.set_text(getAttribute(hElement,"name"))

            self.populateServicesList()
            self.populateMembersList()

        self.prevXML=XML

    def on_about_activate (self,*args):
        dialog = gnome.ui.About(
            ABOUT_VERSION,
            '', ### Don't specify version - already in ABOUT_VERSION
            _("Copyright (c) 2003 Red Hat, Inc. All rights reserved."),
            _("This software is licensed under the terms of the GPL."),
            [ 'Hardy Merrill (redhat-config-cluster) <hmerrill at redhat.com>',
              'Daniel Walsh (redhat-config-cluster) <dwalsh at redhat.com>',
              '',
              'Brian Stevens (clumanager) <bstevens at redhat.com>',
              'Tim Burke (clumanager) <tburke at redhat.com>',
              'Lon Hohberger (clumanager) <lhh at redhat.com>',
              '',
              'Portions of Red Hat Cluster Manager (clumanager) based on Kimberlite v1.1.0 (C) 2000-2001 Mission Critical Linux, Inc.' ],
            [ 'Valerie Rodgers <vrodgers at redhat.com>',
              'John Ha <jha at redhat.com>',
              'Tammy Fox <tfox at redhat.com>' ], # doc people
        ) ### end dialog
        dialog.set_title (FORMALNAME)
        dialog.show()

#----------------------------------------------------------------------------
# Methods pertaining to the "Help" menu items, there will be more here
# once I have "real" help :)
#----------------------------------------------------------------------------
    def on_contents_activate(self,args):
        page = "file:///usr/share/doc/redhat-config-cluster-" + VERSION + "/ch-admin-overview.html"
        gnome.url_show(page)

    def setupPSList(self):
        self.psTreeview=self.xml.get_widget("powerSwitchTreeview")
        model = gtk.ListStore (gobject.TYPE_STRING,gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        model.set_sort_column_id(0, gtk.SORT_ASCENDING)        
        self.psTreeview.set_model (model)

        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("Name"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_PSNAME)
        column.set_sort_column_id(2)
        self.psTreeview.append_column (column)

        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("Device"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_PSDEVICE)
        column.set_sort_column_id(2)
        self.psTreeview.append_column (column)

        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("PORT"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_PSPORT)
        column.set_sort_column_id(2)
        self.psTreeview.append_column (column)

        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("IP"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_PSIP)
        column.set_sort_column_id(2)
        self.psTreeview.append_column (column)

        rend=gtk.CellRendererText()
        column = gtk.TreeViewColumn (_("User"),None)
        column.pack_start(rend,gtk.TRUE)
        column.add_attribute (rend, 'markup', COLUMN_PSLOGIN)
        column.set_sort_column_id(3)
        self.psTreeview.append_column (column)

    def setupList(self):
        self.setupMembersList()
        self.setupServicesList()
        self.setupIPAddressList()
        self.setupDeviceList()
        self.setupVariableList()
        
    def quit(self,arg1=None,arg2=None):
        global quiting
        quiting=1
        gtk.mainquit()
        

if __name__ == "__main__":
    cmdline = sys.argv[1:]
    sys.argv = sys.argv[:1]

    if os.getuid() != 0:
        print _("Please restart %s with root permissions!") % (sys.argv[0])
        sys.exit(10)
        
    signal.signal (signal.SIGINT, signal.SIG_DFL)
    class BadUsage: pass

    configure = FALSE
    status    = FALSE
    try:
        opts, args = getopt.getopt(cmdline, "hcs", ["help","configure","status"])
        for opt, val in opts:
            if opt == '-c' or opt == '--configure':
                configure = TRUE

            if opt == '-s' or opt == '--status':
                status = TRUE

            if opt == '-h' or opt == '--help':
                Usage()
                sys.exit(0)

    except (getopt.error, BadUsage):
        Usage()
        sys.exit(1)

    if configure:
        os.execl(CLUSTERCONFIG_APP, "")

    ### To get here, user just ran 'redhat-config-cluster' - so,
    ### if configuration file(/etc/cluster.xml) exists, run this
    ### GUI which is the cluster status GUI.  If the configuration
    ### file does *NOT* exist, then run the Cluster Configuration
    ### (CLUSTERCONFIG_APP) application.
    try:
        mode = os.stat(CLUSTERCONFIG_FILE)[stat.ST_MODE]
    except OSError, e:
        #print "Cluster configuration file does not exist - running cluconfig now!"
        os.execl(CLUSTERCONFIG_APP, "")

    ### Cluster configuration file *DOES* exist - run this
    ### Cluster Status GUI now.

    #print "Cluster configuration file *DOES* exist - running clustat now!"

    try:
        gui=Gui()
        gui.show()
                
        gtk.mainloop()
    except:
        # if you quit during initialization exceptions can be raised
        if not quiting:
            raise
