#!/usr/bin/python

import sys
import os
import socket
import stat
import string
import gobject

INSTALLDIR="/usr/share/redhat-config-cluster"
CONFIGDIR="/usr/share/redhat-config-cluster/configure"
sys.path.append(CONFIGDIR)
from clusterpkg.ClusterList import ClusterList
from clusterpkg.cluconfig_module import cluconfig

PROGNAME = "redhat-config-gfscluster"
VERSION = "1.0.8"

TRUE = (1==1)
FALSE = not TRUE

### gettext first, then import gtk (exception prints gettext "_") ###
try:
    import gtk
    import gtk.glade
except RuntimeError, e:
    print _("""
  Unable to initialize graphical environment. Most likely cause of failure
  is that the tool was not run using a graphical environment. Please either
  start your graphical user interface or set your DISPLAY variable.
                                                                                
  Caught exception: %s
""") % e
    sys.exit(-1)
                                                                                
import gnome
import gnome.ui

gnome.program_init (PROGNAME,VERSION)
gnome.app_version =VERSION
FORMALNAME=_("redhat-config-gfscluster")

NO_DIRECTORY_FOUND_MESSAGE1=_("The directory ")
NO_DIRECTORY_FOUND_MESSAGE2=_(" does not currently exist. Either create this directory, or back up to the directory selection page and choose another location for the configuration files.")

CANT_OPEN_FILE1=_("The config file ")
CANT_OPEN_FILE2=_(" can not be created. Please check permissions for the selected directory and try again.")

CANT_WRITE_FILE1=_("The config file ")
CANT_WRITE_FILE2=_(" can not be created. Please disk space and permissions for the selected directory and try again.")

CONFIRM_FENCE_DELETE=_("You are about to delete the selected fence device. Do you wish to continue?")
CONFIRM_MEMBER_DELETE=_("You are about to delete a member and all fence devices associated with it. Do you wish to continue?")

ASSOCIATED_FENCE_INSTANCES1=_("Fence instances in the Member/Fence Instance pane reference this device, ")
ASSOCIATED_FENCE_INSTANCES2=_(". All references to this device must be deleted before the device can be deleted.")

NO_CLUSTER_XML_FOUND_MESSAGE=_("An /etc/cluster.xml is not present on this system. Please rerun the GFS Config tool when this file exists.")

QUIT_MESSAGE=_("Your CCS files have been written to your specified location. To leave this application, click 'Yes'. To back up and keep editing, click 'No'. Do you want to quit?")

FENCE_WARNING_MESSAGE=_("Warning: One or more Cluster member nodes do not have Fence Instances associated with them. If you would like to continue anyway, click 'YES'. To add missing fences to a member node, click 'NO', and then use the 'Back' button to return to the Cluster Member page. Do you want to continue anyway?")

CLUSTER_XML="/etc/cluster.xml"

##Column numbers for Member and Fence Instance view
MB_LOCKSRV_COL=0
MB_NAME_COL=1
MB_DEVNAME_COL=2   #Mixed use IF name and fence device name
MB_IPADDR_COL=3    #Mixed use member and gnbd instance
MB_PORT_COL=4
MB_METHOD_COL=5
MB_SWITCH_COL=6
MB_XCATNODE_COL=7  #For XCAT instances only
MB_LPAN_COL=8     #For EGENERA instances only
MB_PSERVER_COL=9  #For EGENERA instances only
MB_AGENTTYPE_COL=10
MB_ISMEMBR_COL=11

##Column numbers for Fence Device view
FD_DEVNAME_COL=0
FD_AGENTTYPE_COL=1
FD_IPADDR_COL=2
FD_LOGIN_COL=3
FD_PASSWD_COL=4
FD_SERVER_COL=5    #For GNBD only
FD_HOSTNAME_COL=6  #for RILOE only
FD_CSERVER_COL=7  #for EGENERA only
FD_RPOWER_COL=8  #for XCAT only


FENCE_OPTS = ["fence_apc",
              "fence_wti",
              "fence_brocade",
              "fence_vixel",
              "fence_gnbd",
              "fence_rib",
              "fence_xcat",
              "fence_mcdata",
              "fence_egenera",
              "fence_manual"]


###################################################
###################################################
### This class has one purpose: To write out the
### three files that GFS needs (cluster.ccs,
### nodes.ccs, and fence.ccs) to start up. It
### reads in information from a redhat clumanager
### cluster.xml file and then allows the user
### to add, edit, and delete member nodes,
### fence devices, and fence instances before
### writing out the three GFS files. Means for
### setting values in the GFS files are provided
### by a four page GNOME Druid.
###
### This file is long and dense. After a very lengthy
### init section, it is primarily composed of callback
### handlers and support methods for 6 dialog boxes that
### are launched via buttons on the fourth druid page:
### 1) Add Member
### 2) Edit Member
### 3) Add Fence Device
### 4) Edit Fence Device
### 5) Add Fence Instance
### 6) Edit Fence Instance
###
### Both the fence device and fence instance dialogs for
### adding and editing have optionmenu's in them. Change
### listeners for these option menu's change sensitivity
### on labels and text entry fields in the dialog boxes
### dependant on the option selected. The fence instance
### dialog box optionmenu's options are dynamic, and set
### up at run time.
###
### There are two Tree Models used in this UI on page 4 of the druid:
### one for the members and fence instances (fence instances
### are rendered as children of the members) and another
### Tree Model is used for fence devices
###
### Two additional methods below are worth mentioning.
### The 'on_finish()' method is the callback
### for the final druid 'apply' button. This is where
### a final validation pass is made, and the three files are
### actually written out.
### The 'do_member_and_fence_list()' method is where
### the cluster.xml file is parsed and the two tree 
### models used in this UI are initially populated.
#######################################################

class gfsconfig:
  def __init__(self, glade_xml):
    self.glade_xml = glade_xml

    #First, find all widgets.
    self.druid = self.glade_xml.get_widget('druid1')
    self.druid.connect("cancel",self.cancel)

    ###########
    ##Incidental fields...
    self.clustername_entry = self.glade_xml.get_widget('entry1')
    self.allowedmisses_entry = self.glade_xml.get_widget('entry2')
    self.heartbeatrate_entry = self.glade_xml.get_widget('entry3')
    self.filelocation_entry = self.glade_xml.get_widget('combo-entry2')
    self.device_name_dict = {}
    self.is_first_node = TRUE
    self.is_first_fence = TRUE
    self.just_wrote_fence = FALSE
    self.last_fencemethod_written = ""
    self.first_locksrv_flag = TRUE

    ############
    ##Put list dialog entries here...
    self.add_fenceinstance_dlg = glade_xml.get_widget('add_fence_instance') 
    self.edit_fenceinstance_dlg = glade_xml.get_widget('edit_fence_instance') 

    self.add_fencedevice_dlg = glade_xml.get_widget('dialog1')
    self.edit_fencedevice_dlg = glade_xml.get_widget('edit_fence_props')

    self.add_member_dlg = glade_xml.get_widget('dialog2')
    self.edit_member_dlg = glade_xml.get_widget('edit_member_props')

    ############
    ##Dialog fields for edit_fencedevice_dlg
    self.edit_fencedevice_name = self.glade_xml.get_widget('entry11')
    self.edit_fencedevice_agent_type = self.glade_xml.get_widget('optionmenu1')
    self.edit_fencedevice_agent_type.connect('changed',self.edit_fencedevice_optionmenu_change)
    self.edit_fencedevice_ipaddr = self.glade_xml.get_widget('entry12')
    self.edit_fencedevice_ipaddr_label = self.glade_xml.get_widget('label26')
    self.edit_fencedevice_server = self.glade_xml.get_widget('entry33')
    self.edit_fencedevice_server_label = self.glade_xml.get_widget('label62')
    self.edit_fencedevice_hostname = self.glade_xml.get_widget('entry34')
    self.edit_fencedevice_hostname_label = self.glade_xml.get_widget('label63')
    self.edit_fencedevice_user = self.glade_xml.get_widget('entry15')
    self.edit_fencedevice_user_label = self.glade_xml.get_widget('label28')
    self.edit_fencedevice_passwd = self.glade_xml.get_widget('entry16')
    self.edit_fencedevice_passwd_label = self.glade_xml.get_widget('label29')
    self.edit_fencedevice_cserver = self.glade_xml.get_widget('entry40')
    self.edit_fencedevice_cserver_label = self.glade_xml.get_widget('label71')
    self.edit_fencedevice_rpower = self.glade_xml.get_widget('entry41')
    self.edit_fencedevice_rpower_label = self.glade_xml.get_widget('label72')

    ############
    ##Dialog fields and labels for add_fencedevice_dlg
    self.add_fencedevice_name = self.glade_xml.get_widget('entry4')
    self.add_fencedevice_agent_type = self.glade_xml.get_widget('optionmenu2')
    self.add_fencedevice_agent_type.connect('changed',self.add_fencedevice_optionmenu_change)
    self.add_fencedevice_ipaddr = self.glade_xml.get_widget('entry35')
    self.add_fencedevice_server = self.glade_xml.get_widget('entry36')
    self.add_fencedevice_hostname = self.glade_xml.get_widget('entry37')
    self.add_fencedevice_user = self.glade_xml.get_widget('entry6')
    self.add_fencedevice_passwd = self.glade_xml.get_widget('entry7')
    self.add_fencedevice_cserver = self.glade_xml.get_widget('entry42')
    self.add_fencedevice_rpower = self.glade_xml.get_widget('entry43')
    self.add_fencedevice_ipaddr_label = self.glade_xml.get_widget('label64')
    self.add_fencedevice_server_label = self.glade_xml.get_widget('label65')
    self.add_fencedevice_hostname_label = self.glade_xml.get_widget('label66')
    self.add_fencedevice_user_label = self.glade_xml.get_widget('label12')
    self.add_fencedevice_passwd_label = self.glade_xml.get_widget('label15')
    self.add_fencedevice_cserver_label = self.glade_xml.get_widget('label73')
    self.add_fencedevice_rpower_label = self.glade_xml.get_widget('label74')

    ############
    ##Dialog fields for edit_fenceinstance_dlg
    self.edit_fenceinstance_name = self.glade_xml.get_widget('optionmenu4')
    self.edit_fenceinstance_name.connect('changed',self.edit_fenceinstance_optionmenu_change)
    self.edit_fenceinstance_method_dynlabel = self.glade_xml.get_widget('label68')
    self.edit_fenceinstance_port = self.glade_xml.get_widget('entry30')
    self.edit_fenceinstance_port_label = self.glade_xml.get_widget('label59')
    self.edit_fenceinstance_ipaddr = self.glade_xml.get_widget('entry31')
    self.edit_fenceinstance_ipaddr_label = self.glade_xml.get_widget('label60')
    self.edit_fenceinstance_switch = self.glade_xml.get_widget('entry32')
    self.edit_fenceinstance_switch_label = self.glade_xml.get_widget('label61')
    self.edit_fenceinstance_xcatnode = self.glade_xml.get_widget('entry44')
    self.edit_fenceinstance_xcatnode_label = self.glade_xml.get_widget('label75')
    self.edit_fenceinstance_lpan = self.glade_xml.get_widget('entry45')
    self.edit_fenceinstance_lpan_label = self.glade_xml.get_widget('label76')
    self.edit_fenceinstance_pserver = self.glade_xml.get_widget('entry46')
    self.edit_fenceinstance_pserver_label = self.glade_xml.get_widget('label77')

    ############
    ##Dialog fields and labels for add_fenceinstance_dlg
    self.add_fenceinstance_name = self.glade_xml.get_widget('optionmenu3')
    self.add_fenceinstance_name.connect('changed',self.add_fenceinstance_optionmenu_change)
    self.add_fenceinstance_method_dynlabel = self.glade_xml.get_widget('label67')
    self.add_fenceinstance_port = self.glade_xml.get_widget('entry21')
    self.add_fenceinstance_ipaddr = self.glade_xml.get_widget('entry22')
    self.add_fenceinstance_switch = self.glade_xml.get_widget('entry23')
    self.add_fenceinstance_xcatnode = self.glade_xml.get_widget('entry49')
    self.add_fenceinstance_lpan = self.glade_xml.get_widget('entry47')
    self.add_fenceinstance_pserver = self.glade_xml.get_widget('entry48')
    self.add_fenceinstance_port_label = self.glade_xml.get_widget('label43')
    self.add_fenceinstance_ipaddr_label = self.glade_xml.get_widget('label44')
    self.add_fenceinstance_switch_label = self.glade_xml.get_widget('label45')
    self.add_fenceinstance_xcatnode_label = self.glade_xml.get_widget('label80')
    self.add_fenceinstance_lpan_label = self.glade_xml.get_widget('label78')
    self.add_fenceinstance_pserver_label = self.glade_xml.get_widget('label79')

    ############
    ##Dialog fields for add_member_dlg 
    self.add_member_hostname = self.glade_xml.get_widget('entry9')
    self.add_member_netdevice = self.glade_xml.get_widget('entry10')
    self.add_member_ipaddr = self.glade_xml.get_widget('entry39')

    ############
    ##Dialog fields for edit_member_dlg
    self.editmember_hostname = self.glade_xml.get_widget('entry17')
    self.editmember_netdevice = self.glade_xml.get_widget('entry18')
    self.editmember_ipaddr = self.glade_xml.get_widget('entry38')

    #############
    ##Now for my favorite part - the action buttons!
    #
    #Fence instance add
    self.add_fenceinstance_button = self.glade_xml.get_widget('button1')
    self.add_fenceinstance_button.connect("clicked",self.on_fenceinstance_add)

    #Fence instance add ok button
    self.add_fenceinstance_ok = self.glade_xml.get_widget('button16').connect('clicked',self.on_fenceinstance_add_ok)

    #Fence instance add cancel button
    self.add_fenceinstance_cancel = self.glade_xml.get_widget('button15').connect('clicked',self.on_fenceinstance_add_cancel)

    #Fence Device List add
    self.add_fencedevice_button = self.glade_xml.get_widget('button12')
    self.add_fencedevice_button.connect("clicked",self.on_fencedevice_add)

    #Fence device add ok button
    self.add_fencedevice_ok = self.glade_xml.get_widget('button3')
    self.add_fencedevice_ok.connect("clicked",self.on_fencedevice_add_ok)

    #Fence device add cancel button
    self.add_fencedevice_cancel = self.glade_xml.get_widget('button2')
    self.add_fencedevice_cancel.connect("clicked",self.on_fencedevice_add_cancel)

    #Fence Device List Edit 
    self.edit_fencedevice_button = self.glade_xml.get_widget('button93')
    self.edit_fencedevice_button.connect("clicked",self.on_fencedevice_edit)

    #Fence device edit update button
    self.edit_fencedevice_ok = self.glade_xml.get_widget('button8')
    self.edit_fencedevice_ok.connect("clicked",self.on_fencedevice_edit_ok)

    #Fence device edit cancel button
    self.edit_fencedevice_cancel = self.glade_xml.get_widget('button7')
    self.edit_fencedevice_cancel.connect("clicked",self.on_fencedevice_edit_cancel)

    #Member add
    self.add_member_button = self.glade_xml.get_widget('button4').connect('clicked',self.on_member_add)

    #Member add ok button
    self.add_member_ok = self.glade_xml.get_widget('okbutton1').connect('clicked',self.on_member_add_ok)

    #Menber add cancel button
    self.newmember_cancelmember = self.glade_xml.get_widget('cancelbutton1').connect('clicked',self.on_member_cancelmember)

    #Member / Fence instance edit props
    self.editprops = self.glade_xml.get_widget('button5')
    self.editprops.connect("clicked",self.on_member_fenceinstance_edit)

    #################
    ## Delete handlers  
    #Member / Fence instance delete entry
    self.deleteentry = self.glade_xml.get_widget('button6')
    self.deleteentry.connect("clicked",self.on_member_fenceinstance_deleteentry)
   
    #Fence device delete entry
    self.fencedevice_deleteentry = self.glade_xml.get_widget('button14')
    self.fencedevice_deleteentry.connect("clicked",self.on_fencedevice_deleteentry)
   
    #Fence instance edit props ok button 
    self.edit_fenceinstance_ok = self.glade_xml.get_widget('button19').connect('clicked',self.on_fenceinstance_edit_ok)

    #Fence instance edit props cancel button
    self.edit_fenceinstance_cancel = self.glade_xml.get_widget('button18').connect('clicked',self.on_fenceinstance_edit_cancel)

    #Fence device edit props
    self.edit_fencedevice_props = self.glade_xml.get_widget('button93')
    #self.edit_fencedevice_props.connect('clicked', on_edit_fencedevice_props)

    #Member edit props ok button
    self.editmember_addmember = self.glade_xml.get_widget('button10').connect('clicked',self.on_editmember_addmember)

    #Member edit props cancel button 
    self.editmember_cancelmember = self.glade_xml.get_widget('button9').connect('clicked',self.on_editmember_cancelmember)


    #Find all pages of druid
    #Then find buttons and connect them to methods
    self.druid_page1 = self.glade_xml.get_widget("druidpagestandard1")
    self.druid_page1.connect("next", self.on_forward_standard1) 
    self.glade_xml.get_widget("druidpagestandard2").connect("next", self.on_forward_standard2) 
    self.glade_xml.get_widget("druidpagestandard3").connect("next", self.on_forward_standard3) 
    self.glade_xml.get_widget("druidpagefinish1").connect("finish", self.on_finish) 

    self.memberview = self.glade_xml.get_widget('treeview2')
    self.fenceview = self.glade_xml.get_widget('treeview3')

    self.membermodel = gtk.TreeStore (gobject.TYPE_BOOLEAN, #lockserver?
                                    gobject.TYPE_STRING,  #Member name
                                    gobject.TYPE_STRING,  #device/fence name
                                    gobject.TYPE_STRING,  #ipaddress
                                    gobject.TYPE_STRING,  #port
                                    gobject.TYPE_STRING,  #fence method field
                                    gobject.TYPE_STRING,  #switch field
                                    gobject.TYPE_STRING,  #xcatnode
                                    gobject.TYPE_STRING,  #lpan - egenera
                                    gobject.TYPE_STRING,  #pserver - egenera
                                    gobject.TYPE_STRING,  #fence type hidden
                                    gobject.TYPE_BOOLEAN) #Member line? hidden

    self.memberview.set_model(self.membermodel)
    self.memberview.connect('test-collapse-row',self.on_collapse_row)

    ################
    ##Now set up all columns for Member View Store
    renderer = gtk.CellRendererToggle()
    renderer.connect('toggled', self.on_toggle_checkbox)
    #The visible=ISMEMBR_COL uses the value in ISMEMBR_COL to determine 
    # the visibility of the checkbox
    column = gtk.TreeViewColumn("LockServer?",renderer, active=0, visible=MB_ISMEMBR_COL)
    self.memberview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Member Name",renderer,text=MB_NAME_COL)
    self.memberview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Device/Fence Name",renderer,text=MB_DEVNAME_COL)
    self.memberview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("IP Addr",renderer,text=MB_IPADDR_COL)
    self.memberview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Fence Method",renderer,text=MB_METHOD_COL)
    self.memberview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Port",renderer,text=MB_PORT_COL)
    self.memberview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Switch",renderer,text=MB_SWITCH_COL)
    self.memberview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("XCAT Node",renderer,text=MB_XCATNODE_COL)
    self.memberview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("LPAN",renderer,text=MB_LPAN_COL)
    self.memberview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("PServer",renderer,text=MB_PSERVER_COL)
    self.memberview.append_column(column)

    #The following code makes a new column, sets it invisible,
    #and then assigns the expander column for the tree to it.
    #Now the expander/collapser icons are invisible!
    column =  gtk.TreeViewColumn()
    column.set_visible (FALSE)
    self.memberview.append_column(column)
    self.memberview.set_expander_column (column)

    selection = self.memberview.get_selection()
    selection.connect('changed', self.on_member_selection_changed)

    ##############
    ##Now set up the fence view store
    self.fencemodel = gtk.TreeStore (gobject.TYPE_STRING,  #fence name
                                     gobject.TYPE_STRING,  #fence type 
                                     gobject.TYPE_STRING,  #ipaddress
                                     gobject.TYPE_STRING,  #login
                                     gobject.TYPE_STRING,  #passwd
                                     gobject.TYPE_STRING,  #server
                                     gobject.TYPE_STRING,  #hostname
                                     gobject.TYPE_STRING,  #cserver - egenera
                                     gobject.TYPE_STRING)  #rpower - xcat
    self.fenceview.set_model(self.fencemodel)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Fence Device Name",renderer,text=FD_DEVNAME_COL)
    self.fenceview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Fence Agent",renderer,text=FD_AGENTTYPE_COL)
    self.fenceview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("IP Address",renderer,text=FD_IPADDR_COL)
    self.fenceview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Login",renderer,text=FD_LOGIN_COL)
    self.fenceview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Password",renderer,text=FD_PASSWD_COL)
    self.fenceview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Server",renderer,text=FD_SERVER_COL)
    self.fenceview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("Hostname",renderer,text=FD_HOSTNAME_COL)
    self.fenceview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("CServer",renderer,text=FD_CSERVER_COL)
    self.fenceview.append_column(column)

    renderer = gtk.CellRendererText()
    column = gtk.TreeViewColumn("RPower Path",renderer,text=FD_RPOWER_COL)
    self.fenceview.append_column(column)

    selection = self.fenceview.get_selection()
    selection.connect('changed', self.on_fence_selection_changed)

    #Here, check and see if cluster.xml exists
    try:
      mode = os.stat(CLUSTER_XML)[stat.ST_MODE]
    except OSError, e:
      self.errorMessage(NO_CLUSTER_XML_FOUND_MESSAGE)
      gtk.mainquit()
      sys.exit(1)

    self.do_member_and_fence_list()

  ##############################################
  ## Below are all of the button callbacks

  def on_fenceinstance_add(self, button):
    #First check if a member is selected. This check will 
    #go away after sensitivity of button is set properly
    selection = self.memberview.get_selection()
    model,iter = selection.get_selected()

    if(iter):
      val = model.get_value(iter,MB_ISMEMBR_COL)
    else:
      self.errorMessage(_("Please select a member to be fenced from the list"))
      return

    if (val == FALSE): ##This means row is a fence instance - not a member
      self.errorMessage(_("Please select a member node to add the fence device to"))
      return

    ##Now check that fencemodel is not empty. If it is,
    ##then we need to tell user to add fence devices first
    fencemodel = self.fenceview.get_model()
    test_iter = fencemodel.get_iter_first()
    if(test_iter == None):
      self.errorMessage(_("Please populate the Fence Device area of this view before attempting to create a fencing instance. All fence instances must refer to an existing fence device by device name."))
      return

    ###########################################
    ## Now we must generate a hash table with fence names/agent types
    ## so we know what fields in the form to turn on. This
    ## dictionary will be used by the change listener for the Optionmenu
    self.device_name_dict = self.prep_add_fenceinstance_optionmenu()
    self.add_fenceinstance_optionmenu_change(None)
    self.add_fenceinstance_dlg.show()

  def on_member_fenceinstance_edit(self, *args):
    #Three tasks here: 
    ##1) check for type of selection 
    ##2) preload form 
    ##3) show

    selection = self.memberview.get_selection()
    model,iter = selection.get_selected()

    if(iter):
      val = model.get_value(iter,MB_ISMEMBR_COL)
    else:
      self.errorMessage(_("Please select a member or fence from the list"))
      return
   
    if(val):
      self.preload_member_dlg() 
      self.edit_member_dlg.show()
    else:
      self.preload_fenceinstance_edit_dlg()
      self.edit_fenceinstance_dlg.show()

  def preload_fenceinstance_edit_dlg(self):
    selection = self.memberview.get_selection()
    model,iter = selection.get_selected()

    ## Again we must generate a hash table with fence names/agent types
    ## so we know what fields in the form to turn on. This
    ## dictionary will be used by the change listener for the Optionmenu
    self.device_name_dict.clear()
    self.device_name_dict = self.prep_edit_fenceinstance_optionmenu()
    self.edit_fenceinstance_optionmenu_change(None, None)

  def preload_fencedevice_edit_dlg(self):
    #get selection. fill in fields. get name, find in optionmenu, then select
    selection = self.fenceview.get_selection()
    model,iter = selection.get_selected()

    agent_val = model.get_value(iter, FD_AGENTTYPE_COL)
    self.edit_fencedevice_agent_type.set_history(0)
    idx = 0
    history_idx = 0
    if(agent_val == self.edit_fencedevice_agent_type.get_children()[0]):
      idx = 0
      history_idx = 0
    else:
      menu = self.edit_fencedevice_agent_type.get_menu()
      idx = -1
      for item in menu.get_children():
        idx = idx + 1
        children = item.get_children()
        if children:
          lbl = item.get_children()[0]
          if(lbl.get_text() == agent_val):
            history_idx = idx
            break

    #Now we know what menu item to preset
    self.clear_edit_fencedevice_dlg()
    self.edit_fencedevice_agent_type.set_history(history_idx)
    self.edit_fencedevice_optionmenu_change(None)
     

  def on_fencedevice_add(self, *args):
    #show dialog
    self.add_fencedevice_agent_type.set_history(0)
    self.add_fencedevice_optionmenu_change(None)
    self.add_fencedevice_dlg.show()

  def on_fencedevice_edit(self, *args):
    #show dialog
    self.preload_fencedevice_edit_dlg()
    self.edit_fencedevice_dlg.show()

  def prep_add_fenceinstance_optionmenu(self):
    menu = gtk.Menu()
    dict = {}

    user_data = list()
    user_data.append(menu)
    user_data.append(dict)

    model = self.fenceview.get_model()
    model.foreach(self.load_optionmenu, user_data)

    self.add_fenceinstance_name.set_menu(menu)
    self.add_fenceinstance_name.set_history(0)

    return user_data[1]
   
  def prep_edit_fenceinstance_optionmenu(self):
    menu = gtk.Menu()
    dict = {}

    user_data = list()
    user_data.append(menu)
    user_data.append(dict)

    model = self.fenceview.get_model()
    model.foreach(self.load_optionmenu, user_data)

    self.edit_fenceinstance_name.set_menu(menu)

    #Now the optionmenu must be preset to current device name
    #as this is an edit menu. 
    selection = self.memberview.get_selection()
    model, iter = selection.get_selected()
    name = model.get_value(iter, MB_DEVNAME_COL)
    idx = -1
    found = FALSE
    for item in menu.get_children():
      idx = idx +1
      children = item.get_children()
      if children:
        lbl = item.get_children()[0]
        if(lbl.get_text() == name):
          found = TRUE
          break
    if found: 
      self.edit_fenceinstance_name.set_history(idx)
    else:
      self.edit_fenceinstance_name.set_history(0)

    return user_data[1]
   
  #################################################
  ## This 'foreach' handler reads each device name entry
  ## and loads an option menu with the choices 
  def load_optionmenu(self, model, path, iter, user_data):
    menu = user_data[0]
    dict = user_data[1]

    name_val = model.get_value(iter, FD_DEVNAME_COL)
    new_name_val = name_val.replace('_','__')
    type_val = model.get_value(iter, FD_AGENTTYPE_COL)
    dict[name_val] = type_val

    m = gtk.MenuItem(new_name_val)
    m.show()
    menu.append(m)

  def on_fenceinstance_edit_ok(self, *args):
    opt_str = self.edit_fenceinstance_name.get_children()[0].get()

    agent_val = self.device_name_dict[opt_str]
  
    #Find out what member we are fencing
    selection = self.memberview.get_selection()
    model,child_iter = selection.get_selected()


    #validation and model setting for each variant

    if(agent_val == 'fence_apc'):
      if (self.edit_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify Port for fence instance"))
        return

      if (self.edit_fenceinstance_switch.get_text() == ""):
        self.errorMessage(_("Please specify Port for fence instance"))
        return

      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.edit_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.edit_fenceinstance_port.get_text(),
                MB_SWITCH_COL,self.edit_fenceinstance_switch.get_text(),
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    elif(agent_val == 'fence_wti'):
      if (self.edit_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify Port for fence instance"))
        return

      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.edit_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.edit_fenceinstance_port.get_text(),
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    elif(agent_val == 'fence_brocade'):
      if (self.edit_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify Port for fence instance"))
        return

      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.edit_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.edit_fenceinstance_port.get_text(),
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    elif(agent_val == 'fence_vixel'):
      if (self.edit_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify Port for fence instance"))
        return

      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.edit_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.edit_fenceinstance_port.get_text(),
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    if(agent_val == 'fence_gnbd'):
      if (self.edit_fenceinstance_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for fence instance"))
        return

      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, self.edit_fenceinstance_ipaddr.get_text(),
                MB_METHOD_COL,self.edit_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL, "N/A",
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    if(agent_val == 'fence_rib'):
      if (self.edit_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify Port for fence instance"))
        return

      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.edit_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.edit_fenceinstance_port.get_text(),
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    if(agent_val == 'fence_xcat'):
      if (self.edit_fenceinstance_xcatnode.get_text() == ""):
        self.errorMessage(_("Please specify XCAT Environment Nodename for fence instance"))
        return

      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.edit_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL, "N/A",
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, self.edit_fenceinstance_xcatnode.get_text(),
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    if(agent_val == 'fence_mcdata'):
      if (self.edit_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify XCAT Environment Nodename for fence instance"))
        return

      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.edit_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.edit_fenceinstance_port.get_text(),
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    if(agent_val == 'fence_egenera'):
      if (self.edit_fenceinstance_lpan.get_text() == ""):
        self.errorMessage(_("Please specify Egenera LPAN for fence instance"))
        return

      if (self.edit_fenceinstance_pserver.get_text() == ""):
        self.errorMessage(_("Please specify Egenera PServer for fence instance"))
        return

      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.edit_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL, "N/A",
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, self.edit_fenceinstance_lpan.get_text(),
                MB_PSERVER_COL, self.edit_fenceinstance_pserver.get_text(),
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    if(agent_val == 'fence_manual'):

      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.edit_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL, "N/A",
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    self.memberview.expand_all() 

    #add fields to columns, then clear fields and hide dlg
    self.clear_edit_fenceinstance_dlg()
    self.device_name_dict.clear()
    self.edit_fenceinstance_dlg.hide()
    

  def add_fenceinstance_optionmenu_change(self, optionmenu, *args):
    selected_string = self.add_fenceinstance_name.get_children()[0].get()

    #We will need the selected member's ip addr for the ipaddr field
    #in fence_gnbd
    selection = self.memberview.get_selection()
    model, iter = selection.get_selected()
    ipaddr = model.get_value(iter, MB_IPADDR_COL)
    
    
    #Now we have the chosen string - get agent from hash table...
    # but first test if this callback was fired from menu setup...
    if(len(self.device_name_dict.keys()) != 0):
      agent_val = self.device_name_dict[selected_string] 

      #Set all variable fields to off
      self.add_fenceinstance_port.set_sensitive(FALSE)
      self.add_fenceinstance_port_label.set_sensitive(FALSE)
      self.add_fenceinstance_ipaddr.set_sensitive(FALSE)
      self.add_fenceinstance_ipaddr_label.set_sensitive(FALSE)
      self.add_fenceinstance_switch.set_sensitive(FALSE)
      self.add_fenceinstance_switch_label.set_sensitive(FALSE)
      self.add_fenceinstance_xcatnode.set_sensitive(FALSE)
      self.add_fenceinstance_xcatnode_label.set_sensitive(FALSE)
      self.add_fenceinstance_lpan.set_sensitive(FALSE)
      self.add_fenceinstance_lpan_label.set_sensitive(FALSE)
      self.add_fenceinstance_pserver.set_sensitive(FALSE)
      self.add_fenceinstance_pserver_label.set_sensitive(FALSE)
      
      if(agent_val == 'fence_apc'):
        self.add_fenceinstance_method_dynlabel.set_text("power")
        self.add_fenceinstance_port.set_sensitive(TRUE)
        self.add_fenceinstance_port_label.set_sensitive(TRUE)
        self.add_fenceinstance_switch.set_sensitive(TRUE)
        self.add_fenceinstance_switch_label.set_sensitive(TRUE)
        
      elif(agent_val == 'fence_wti'):
        self.add_fenceinstance_method_dynlabel.set_text("power")
        self.add_fenceinstance_port.set_sensitive(TRUE)
        self.add_fenceinstance_port_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_brocade'):
        self.add_fenceinstance_method_dynlabel.set_text("san")
        self.add_fenceinstance_port.set_sensitive(TRUE)
        self.add_fenceinstance_port_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_vixel'):
        self.add_fenceinstance_method_dynlabel.set_text("san")
        self.add_fenceinstance_port.set_sensitive(TRUE)
        self.add_fenceinstance_port_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_gnbd'):
        self.add_fenceinstance_method_dynlabel.set_text("server")
        self.add_fenceinstance_ipaddr_label.set_sensitive(TRUE)
        self.add_fenceinstance_ipaddr.set_text(ipaddr)
        self.add_fenceinstance_ipaddr.set_sensitive(FALSE)

      elif(agent_val == 'fence_rib'):
        self.add_fenceinstance_method_dynlabel.set_text("riloe")
        self.add_fenceinstance_port.set_sensitive(TRUE)
        self.add_fenceinstance_port_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_xcat'):
        self.add_fenceinstance_method_dynlabel.set_text("power")
        self.add_fenceinstance_xcatnode.set_sensitive(TRUE)
        self.add_fenceinstance_xcatnode_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_mcdata'):
        self.add_fenceinstance_method_dynlabel.set_text("san")
        self.add_fenceinstance_port.set_sensitive(TRUE)
        self.add_fenceinstance_port_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_egenera'):
        self.add_fenceinstance_method_dynlabel.set_text("power")
        self.add_fenceinstance_lpan.set_sensitive(TRUE)
        self.add_fenceinstance_lpan_label.set_sensitive(TRUE)
        self.add_fenceinstance_pserver.set_sensitive(TRUE)
        self.add_fenceinstance_pserver_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_manual'):
        self.add_fenceinstance_method_dynlabel.set_text("human")

  def edit_fenceinstance_optionmenu_change(self, optionmenu, *args):
    ###
    ###Need to do some serious work here...
    ###Gotta get the current val for name in this row.
    selection = self.memberview.get_selection()
    model,iter = selection.get_selected()

    currently_selected_name = model.get_value(iter, MB_DEVNAME_COL)

    selected_string = self.edit_fenceinstance_name.get_children()[0].get()
    
    
    #Now we have the chosen string - get agent from hash table...
    # but first test if this callback was fired from menu setup...
    if(len(self.device_name_dict.keys()) != 0):
      agent_val = self.device_name_dict[selected_string] 

      ##At this point we know the agent type for the selected fenceinstance,
      ##as well as the agenttype for the name selected within the
      ##edit props dialog box. If they match, fill in fields. If they
      ##do not match, do the normal response, turning on appropriate fields.
      ##Note: What we are essentially doing here, is checking if
      ##the user wants to edit his existing choice for agenttype, or
      ##choose a new agenttype. If editing, (stored agenttype == current
      ##optionmenu choice) then we preload the fields
      ## in the dialog box. 

      #Set all variable fields to off
      self.edit_fenceinstance_port.set_sensitive(FALSE)
      self.edit_fenceinstance_port.set_text("")
      self.edit_fenceinstance_port_label.set_sensitive(FALSE)
      self.edit_fenceinstance_ipaddr.set_sensitive(FALSE)
      self.edit_fenceinstance_ipaddr.set_text("")
      self.edit_fenceinstance_ipaddr_label.set_sensitive(FALSE)
      self.edit_fenceinstance_switch.set_sensitive(FALSE)
      self.edit_fenceinstance_switch.set_text("")
      self.edit_fenceinstance_switch_label.set_sensitive(FALSE)
      self.edit_fenceinstance_xcatnode.set_sensitive(FALSE)
      self.edit_fenceinstance_xcatnode.set_text("")
      self.edit_fenceinstance_xcatnode_label.set_sensitive(FALSE)
      self.edit_fenceinstance_lpan.set_sensitive(FALSE)
      self.edit_fenceinstance_lpan.set_text("")
      self.edit_fenceinstance_lpan_label.set_sensitive(FALSE)
      self.edit_fenceinstance_pserver.set_sensitive(FALSE)
      self.edit_fenceinstance_pserver.set_text("")
      self.edit_fenceinstance_pserver_label.set_sensitive(FALSE)
      
      if(agent_val == 'fence_apc'):
        self.edit_fenceinstance_method_dynlabel.set_text("power")
        self.edit_fenceinstance_port.set_sensitive(TRUE)
        self.edit_fenceinstance_port_label.set_sensitive(TRUE)
        self.edit_fenceinstance_switch.set_sensitive(TRUE)
        self.edit_fenceinstance_switch_label.set_sensitive(TRUE)
        if(selected_string == currently_selected_name):
          self.edit_fenceinstance_port.set_text(model.get_value(iter,MB_PORT_COL))
          self.edit_fenceinstance_switch.set_text(model.get_value(iter, MB_SWITCH_COL))
        
      elif(agent_val == 'fence_wti'):
        self.edit_fenceinstance_method_dynlabel.set_text("power")
        self.edit_fenceinstance_port.set_sensitive(TRUE)
        self.edit_fenceinstance_port_label.set_sensitive(TRUE)
        if(selected_string == currently_selected_name):
          self.edit_fenceinstance_port.set_text(model.get_value(iter,MB_PORT_COL))

      elif(agent_val == 'fence_brocade'):
        self.edit_fenceinstance_method_dynlabel.set_text("san")
        self.edit_fenceinstance_port.set_sensitive(TRUE)
        self.edit_fenceinstance_port_label.set_sensitive(TRUE)
        if(selected_string == currently_selected_name):
          self.edit_fenceinstance_port.set_text(model.get_value(iter,MB_PORT_COL))

      elif(agent_val == 'fence_vixel'):
        self.edit_fenceinstance_method_dynlabel.set_text("san")
        self.edit_fenceinstance_port.set_sensitive(TRUE)
        self.edit_fenceinstance_port_label.set_sensitive(TRUE)
        if(selected_string == currently_selected_name):
          self.edit_fenceinstance_port.set_text(model.get_value(iter,MB_PORT_COL))

      elif(agent_val == 'fence_gnbd'):
        self.edit_fenceinstance_method_dynlabel.set_text("server")
        self.edit_fenceinstance_ipaddr_label.set_sensitive(TRUE)
        if(selected_string == currently_selected_name):
          self.edit_fenceinstance_ipaddr.set_text(model.get_value(iter,MB_IPADDR_COL))
        self.edit_fenceinstance_ipaddr.set_sensitive(FALSE)

      elif(agent_val == 'fence_rib'):
        self.edit_fenceinstance_method_dynlabel.set_text("riloe")
        self.edit_fenceinstance_port.set_sensitive(TRUE)
        self.edit_fenceinstance_port_label.set_sensitive(TRUE)
        if(selected_string == currently_selected_name):
          self.edit_fenceinstance_port.set_text(model.get_value(iter,MB_PORT_COL))

      elif(agent_val == 'fence_xcat'):
        self.edit_fenceinstance_method_dynlabel.set_text("power")
        self.edit_fenceinstance_xcatnode.set_sensitive(TRUE)
        self.edit_fenceinstance_xcatnode_label.set_sensitive(TRUE)
        if(selected_string == currently_selected_name):
          self.edit_fenceinstance_xcatnode.set_text(model.get_value(iter,MB_XCATNODE_COL))

      elif(agent_val == 'fence_mcdata'):
        self.edit_fenceinstance_method_dynlabel.set_text("san")
        self.edit_fenceinstance_port.set_sensitive(TRUE)
        self.edit_fenceinstance_port_label.set_sensitive(TRUE)
        if(selected_string == currently_selected_name):
          self.edit_fenceinstance_port.set_text(model.get_value(iter,MB_PORT_COL))

      elif(agent_val == 'fence_egenera'):
        self.edit_fenceinstance_method_dynlabel.set_text("power")
        self.edit_fenceinstance_lpan.set_sensitive(TRUE)
        self.edit_fenceinstance_lpan_label.set_sensitive(TRUE)
        self.edit_fenceinstance_pserver.set_sensitive(TRUE)
        self.edit_fenceinstance_pserver_label.set_sensitive(TRUE)
        if(selected_string == currently_selected_name):
          self.edit_fenceinstance_lpan.set_text(model.get_value(iter,MB_LPAN_COL))
          self.edit_fenceinstance_pserver.set_text(model.get_value(iter,MB_PSERVER_COL))

      elif(agent_val == 'fence_manual'):
        self.edit_fenceinstance_method_dynlabel.set_text("human")

      
  def add_fencedevice_optionmenu_change(self, optionmenu, *args):
      agent_val = self.add_fencedevice_agent_type.get_children()[0].get()
    
    
      #Now we have the chosen string - 

      #Set all variable fields to off
      self.add_fencedevice_server.set_sensitive(FALSE)
      self.add_fencedevice_server_label.set_sensitive(FALSE)
      self.add_fencedevice_ipaddr.set_sensitive(FALSE)
      self.add_fencedevice_ipaddr_label.set_sensitive(FALSE)
      self.add_fencedevice_hostname.set_sensitive(FALSE)
      self.add_fencedevice_hostname_label.set_sensitive(FALSE)
      self.add_fencedevice_user.set_sensitive(FALSE)
      self.add_fencedevice_user_label.set_sensitive(FALSE)
      self.add_fencedevice_passwd.set_sensitive(FALSE)
      self.add_fencedevice_passwd_label.set_sensitive(FALSE)
      self.add_fencedevice_cserver.set_sensitive(FALSE)
      self.add_fencedevice_cserver_label.set_sensitive(FALSE)
      self.add_fencedevice_rpower.set_sensitive(FALSE)
      self.add_fencedevice_rpower_label.set_sensitive(FALSE)

      if(agent_val == 'fence_apc'):
        self.add_fencedevice_ipaddr.set_sensitive(TRUE)
        self.add_fencedevice_ipaddr_label.set_sensitive(TRUE)
        self.add_fencedevice_user.set_sensitive(TRUE)
        self.add_fencedevice_user_label.set_sensitive(TRUE)
        self.add_fencedevice_passwd.set_sensitive(TRUE)
        self.add_fencedevice_passwd_label.set_sensitive(TRUE)
        
      elif(agent_val == 'fence_wti'):
        self.add_fencedevice_ipaddr.set_sensitive(TRUE)
        self.add_fencedevice_ipaddr_label.set_sensitive(TRUE)
        self.add_fencedevice_passwd.set_sensitive(TRUE)
        self.add_fencedevice_passwd_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_brocade'):
        self.add_fencedevice_ipaddr.set_sensitive(TRUE)
        self.add_fencedevice_ipaddr_label.set_sensitive(TRUE)
        self.add_fencedevice_user.set_sensitive(TRUE)
        self.add_fencedevice_user_label.set_sensitive(TRUE)
        self.add_fencedevice_passwd.set_sensitive(TRUE)
        self.add_fencedevice_passwd_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_vixel'):
        self.add_fencedevice_ipaddr.set_sensitive(TRUE)
        self.add_fencedevice_ipaddr_label.set_sensitive(TRUE)
        self.add_fencedevice_passwd.set_sensitive(TRUE)
        self.add_fencedevice_passwd_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_gnbd'):
        self.add_fencedevice_server.set_sensitive(TRUE)
        self.add_fencedevice_server_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_rib'):
        self.add_fencedevice_hostname.set_sensitive(TRUE)
        self.add_fencedevice_hostname_label.set_sensitive(TRUE)
        self.add_fencedevice_user.set_sensitive(TRUE)
        self.add_fencedevice_user_label.set_sensitive(TRUE)
        self.add_fencedevice_passwd.set_sensitive(TRUE)
        self.add_fencedevice_passwd_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_xcat'):
        self.add_fencedevice_rpower.set_sensitive(TRUE)
        self.add_fencedevice_rpower_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_mcdata'):
        self.add_fencedevice_ipaddr.set_sensitive(TRUE)
        self.add_fencedevice_ipaddr_label.set_sensitive(TRUE)
        self.add_fencedevice_user.set_sensitive(TRUE)
        self.add_fencedevice_user_label.set_sensitive(TRUE)
        self.add_fencedevice_passwd.set_sensitive(TRUE)
        self.add_fencedevice_passwd_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_egenera'):
        self.add_fencedevice_cserver.set_sensitive(TRUE)
        self.add_fencedevice_cserver_label.set_sensitive(TRUE)

  def edit_fencedevice_optionmenu_change(self, optionmenu, *args):
      agent_val = self.edit_fencedevice_agent_type.get_children()[0].get()

      selection = self.fenceview.get_selection()
      model,iter = selection.get_selected()

    
    
      #Now we have the chosen string - 

      #Set all variable fields to off
      self.edit_fencedevice_server.set_sensitive(FALSE)
      self.edit_fencedevice_server_label.set_sensitive(FALSE)
      self.edit_fencedevice_ipaddr.set_sensitive(FALSE)
      self.edit_fencedevice_ipaddr_label.set_sensitive(FALSE)
      self.edit_fencedevice_hostname.set_sensitive(FALSE)
      self.edit_fencedevice_hostname_label.set_sensitive(FALSE)
      self.edit_fencedevice_user.set_sensitive(FALSE)
      self.edit_fencedevice_user_label.set_sensitive(FALSE)
      self.edit_fencedevice_passwd.set_sensitive(FALSE)
      self.edit_fencedevice_passwd_label.set_sensitive(FALSE)

      #set the devname field
      self.edit_fencedevice_name.set_text(model.get_value(iter, FD_DEVNAME_COL))

      if(agent_val == 'fence_apc'):
        self.edit_fencedevice_ipaddr.set_sensitive(TRUE)
        self.edit_fencedevice_ipaddr.set_text(model.get_value(iter, FD_IPADDR_COL))
        self.edit_fencedevice_ipaddr_label.set_sensitive(TRUE)
        self.edit_fencedevice_user.set_sensitive(TRUE)
        self.edit_fencedevice_user.set_text(model.get_value(iter, FD_LOGIN_COL))
        self.edit_fencedevice_user_label.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_text(model.get_value(iter,FD_PASSWD_COL))
        self.edit_fencedevice_passwd_label.set_sensitive(TRUE)
        
      elif(agent_val == 'fence_wti'):
        self.edit_fencedevice_ipaddr.set_sensitive(TRUE)
        self.edit_fencedevice_ipaddr.set_text(model.get_value(iter,FD_IPADDR_COL))
        self.edit_fencedevice_ipaddr_label.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_text(model.get_value(iter,FD_PASSWD_COL))
        self.edit_fencedevice_passwd_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_brocade'):
        self.edit_fencedevice_ipaddr.set_sensitive(TRUE)
        self.edit_fencedevice_ipaddr.set_text(model.get_value(iter,FD_IPADDR_COL))
        self.edit_fencedevice_ipaddr_label.set_sensitive(TRUE)
        self.edit_fencedevice_user.set_sensitive(TRUE)
        self.edit_fencedevice_user.set_text(model.get_value(iter,FD_LOGIN_COL))
        self.edit_fencedevice_user_label.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_text(model.get_value(iter,FD_PASSWD_COL))
        self.edit_fencedevice_passwd_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_vixel'):
        self.edit_fencedevice_ipaddr.set_sensitive(TRUE)
        self.edit_fencedevice_ipaddr.set_text(model.get_value(iter,FD_IPADDR_COL))
        self.edit_fencedevice_ipaddr_label.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_text(model.get_value(iter,FD_PASSWD_COL))
        self.edit_fencedevice_passwd_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_gnbd'):
        self.edit_fencedevice_server.set_sensitive(TRUE)
        self.edit_fencedevice_server.set_text(model.get_value(iter,FD_SERVER_COL))
        self.edit_fencedevice_server_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_rib'):
        self.edit_fencedevice_hostname.set_sensitive(TRUE)
        self.edit_fencedevice_hostname.set_text(model.get_value(iter,FD_HOSTNAME_COL))
        self.edit_fencedevice_hostname_label.set_sensitive(TRUE)
        self.edit_fencedevice_user.set_sensitive(TRUE)
        self.edit_fencedevice_user.set_text(model.get_value(iter,FD_LOGIN_COL))
        self.edit_fencedevice_user_label.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_text(model.get_value(iter,FD_PASSWD_COL))
        self.edit_fencedevice_passwd_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_xcat'):
        self.edit_fencedevice_rpower.set_sensitive(TRUE)
        self.edit_fencedevice_rpower.set_text(model.get_value(iter, FD_RPOWER_COL))
        self.edit_fencedevice_rpower_label.set_sensitive(TRUE)

      elif(agent_val == 'fence_mcdata'):
        self.edit_fencedevice_ipaddr.set_sensitive(TRUE)
        self.edit_fencedevice_ipaddr.set_text(model.get_value(iter, FD_IPADDR_COL))
        self.edit_fencedevice_ipaddr_label.set_sensitive(TRUE)
        self.edit_fencedevice_user.set_sensitive(TRUE)
        self.edit_fencedevice_user.set_text(model.get_value(iter, FD_LOGIN_COL))
        self.edit_fencedevice_user_label.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_sensitive(TRUE)
        self.edit_fencedevice_passwd.set_text(model.get_value(iter,FD_PASSWD_COL))
        self.edit_fencedevice_passwd_label.set_sensitive(TRUE)
        

      elif(agent_val == 'fence_egenera'):
        self.edit_fencedevice_cserver.set_sensitive(TRUE)
        self.edit_fencedevice_cserver.set_text(model.get_value(iter, FD_CSERVER_COL))
        self.edit_fencedevice_cserver_label.set_sensitive(TRUE)


  def on_member_add(self,button):
    self.add_member_dlg.show()

  def on_fenceinstance_add_ok(self, button):
    opt_str = self.add_fenceinstance_name.get_children()[0].get()

    agent_val = self.device_name_dict[opt_str]
  
    #Find out what member we are fencing
    selection = self.memberview.get_selection()
    model,iter = selection.get_selected()

    #validation and model setting for each variant

    if(agent_val == 'fence_apc'):
      if (self.add_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify Port for new fence instance"))
        return

      if (self.add_fenceinstance_switch.get_text() == ""):
        self.errorMessage(_("Please specify Switch for new fence instance"))
        return

      child_iter = model.insert(iter,999,None) #999 for last one
      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.add_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.add_fenceinstance_port.get_text(),
                MB_SWITCH_COL,self.add_fenceinstance_switch.get_text(),
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    elif(agent_val == 'fence_wti'):
      if (self.add_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify Port for new fence instance"))
        return

      child_iter = model.insert(iter,999,None) #999 for last one
      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.add_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.add_fenceinstance_port.get_text(),
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    elif(agent_val == 'fence_brocade'):
      if (self.add_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify Port for new fence instance"))
        return

      child_iter = model.insert(iter,999,None) #999 for last one
      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.add_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.add_fenceinstance_port.get_text(),
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    elif(agent_val == 'fence_vixel'):
      if (self.add_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify Port for new fence instance"))
        return

      child_iter = model.insert(iter,999,None) #999 for last one
      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.add_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.add_fenceinstance_port.get_text(),
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    if(agent_val == 'fence_gnbd'):
      if (self.add_fenceinstance_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for new fence instance"))
        return

      child_iter = model.insert(iter,999,None) #999 for last one
      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, self.add_fenceinstance_ipaddr.get_text(),
                MB_METHOD_COL,self.add_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL, "N/A",
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    if(agent_val == 'fence_rib'):
      if (self.add_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify Port for new fence instance"))
        return

      child_iter = model.insert(iter,999,None) #999 for last one
      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.add_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.add_fenceinstance_port.get_text(),
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    elif(agent_val == 'fence_xcat'):
      if (self.add_fenceinstance_xcatnode.get_text() == ""):
        self.errorMessage(_("Please specify XCAT Environment Nodename for new fence instance"))
        return

      child_iter = model.insert(iter,999,None) #999 for last one
      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.add_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL, "N/A",
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, self.add_fenceinstance_xcatnode.get_text(),
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    elif(agent_val == 'fence_mcdata'):
      if (self.add_fenceinstance_port.get_text() == ""):
        self.errorMessage(_("Please specify Port for new fence instance"))
        return

      child_iter = model.insert(iter,999,None) #999 for last one
      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.add_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.add_fenceinstance_port.get_text(),
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    elif(agent_val == 'fence_egenera'):
      if (self.add_fenceinstance_lpan.get_text() == ""):
        self.errorMessage(_("Please specify LPAN for new fence instance"))
        return

      if (self.add_fenceinstance_pserver.get_text() == ""):
        self.errorMessage(_("Please specify PServer for new fence instance"))
        return

      child_iter = model.insert(iter,999,None) #999 for last one
      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.add_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL,self.add_fenceinstance_port.get_text(),
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, self.add_fenceinstance_lpan.get_text(),
                MB_PSERVER_COL, self.add_fenceinstance_pserver.get_text(),
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    if(agent_val == 'fence_manual'):

      child_iter = model.insert(iter,999,None) #999 for last one
      model.set(child_iter,
                MB_DEVNAME_COL,opt_str,
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL,self.add_fenceinstance_method_dynlabel.get_text(),
                MB_PORT_COL, "N/A",
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, agent_val, 
                MB_ISMEMBR_COL, FALSE) 

    self.memberview.expand_all() 

    #add fields to columns, then clear fields and hide dlg
    self.clear_add_fenceinstance_dlg()
    self.add_fenceinstance_dlg.hide()
    
  def on_fencedevice_add_ok(self, button):
    agent_val = self.add_fencedevice_agent_type.get_children()[0].get()

    #Get the Fence model
    fencemodel = self.fenceview.get_model()

    #validation and model setting for new fence device
    if (self.add_fencedevice_name.get_text() == ""):
      self.errorMessage(_("Please specify a name for new fence device"))
      return

    ##Check for duplicate entries before adding
    duplicate = list()
    found_duplicate = FALSE
    duplicate.append(self.add_fencedevice_name.get_text())
    duplicate.append(found_duplicate)
    fencemodel.foreach(self.check_fence_duplicates, duplicate)
    if(duplicate[1] == TRUE): #another entry with same name is present
      self.errorMessage(_("A Fence Device with this name already exists. Please choose a unique name for your new Fence Device."))
      return

    if(agent_val == 'fence_apc'):
      if (self.add_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for new fence device"))
        return

      if (self.add_fencedevice_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for new fence device"))
        return

      if (self.add_fencedevice_user.get_text() == ""):
        self.errorMessage(_("Please specify User Name for new fence device"))
        return

      if (self.add_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for new fence device"))
        return

      #Now add new fence into tree model
      iter = fencemodel.append(None)
      fencemodel.set(iter,FD_DEVNAME_COL, self.add_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.add_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL,self.add_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.add_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_wti'):
      if (self.add_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for new fence device"))
        return

      if (self.add_fencedevice_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for new fence device"))
        return

      if (self.add_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for new fence device"))
        return

      #Now add new fence into tree model
      iter = fencemodel.append(None)
      fencemodel.set(iter,FD_DEVNAME_COL, self.add_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.add_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL, "N/A",
                       FD_PASSWD_COL,self.add_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_brocade'):
      if (self.add_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for new fence device"))
        return

      if (self.add_fencedevice_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for new fence device"))
        return

      if (self.add_fencedevice_user.get_text() == ""):
        self.errorMessage(_("Please specify User Name for new fence device"))
        return

      if (self.add_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for new fence device"))
        return

      #Now add new fence into tree model
      iter = fencemodel.append(None)
      fencemodel.set(iter,FD_DEVNAME_COL, self.add_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.add_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL,self.add_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.add_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_vixel'):
      if (self.add_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for new fence device"))
        return

      if (self.add_fencedevice_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for new fence device"))
        return

      if (self.add_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for new fence device"))
        return

      #Now add new fence into tree model
      iter = fencemodel.append(None)
      fencemodel.set(iter,FD_DEVNAME_COL, self.add_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.add_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL, "N/A",
                       FD_PASSWD_COL,self.add_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_gnbd'):
      if (self.add_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for new fence device"))
        return

      if (self.add_fencedevice_server.get_text() == ""):
        self.errorMessage(_("Please specify Server Address for new fence device"))
        return

      #Now add new fence into tree model
      iter = fencemodel.append(None)
      fencemodel.set(iter,FD_DEVNAME_COL, self.add_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL, "N/A",
                       FD_LOGIN_COL, "N/A",
                       FD_PASSWD_COL,"N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_SERVER_COL, self.add_fencedevice_server.get_text(),
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_rib'):
      if (self.add_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for new fence device"))
        return

      if (self.add_fencedevice_hostname.get_text() == ""):
        self.errorMessage(_("Please specify a hostname for new fence device"))
        return

      if (self.add_fencedevice_user.get_text() == ""):
        self.errorMessage(_("Please specify User Name for new fence device"))
        return

      if (self.add_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for new fence device"))
        return

      #Now add new fence into tree model
      iter = fencemodel.append(None)
      fencemodel.set(iter,FD_DEVNAME_COL, self.add_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL, "N/A",
                       FD_LOGIN_COL,self.add_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.add_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, self.add_fencedevice_hostname.get_text())

    elif(agent_val == 'fence_xcat'):
      if (self.add_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for new fence device"))
        return

      #Now add new fence into tree model
      iter = fencemodel.append(None)
      rpower_val = self.add_fencedevice_rpower.get_text()
      if(rpower_val == None):
        rpower_val = ""  #Not sure if this is necessary...safer than sorrier
      fencemodel.set(iter,FD_DEVNAME_COL, self.add_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.add_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL,self.add_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.add_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, rpower_val,
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_mcdata'):
      if (self.add_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for new fence device"))
        return

      if (self.add_fencedevice_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for new fence device"))
        return

      if (self.add_fencedevice_user.get_text() == ""):
        self.errorMessage(_("Please specify User Name for new fence device"))
        return

      if (self.add_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for new fence device"))
        return

      #Now add new fence into tree model
      iter = fencemodel.append(None)
      fencemodel.set(iter,FD_DEVNAME_COL, self.add_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.add_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL,self.add_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.add_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")

    elif(agent_val == 'fence_egenera'):
      if (self.add_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for new fence device"))
        return

      if (self.add_fencedevice_cserver.get_text() == ""):
        self.errorMessage(_("Please specify CServer for new fence device"))
        return

      #Now add new fence into tree model
      iter = fencemodel.append(None)
      fencemodel.set(iter,FD_DEVNAME_COL, self.add_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.add_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL,self.add_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.add_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, self.add_fencedevice_cserver.get_text(),
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_manual'):
      if (self.add_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for new fence device"))
        return

      #Now add new fence into tree model
      iter = fencemodel.append(None)
      fencemodel.set(iter,FD_DEVNAME_COL, self.add_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL, "N/A",
                       FD_LOGIN_COL, "N/A",
                       FD_PASSWD_COL, "N/A",
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    self.fenceview.expand_all() 

    #add fields to columns, then clear fields and hide dlg
    self.clear_add_fencedevice_dlg()
    self.add_fencedevice_dlg.hide()
    
  def on_fencedevice_edit_ok(self, button):
    agent_val = self.edit_fencedevice_agent_type.get_children()[0].get()

    selection = self.fenceview.get_selection()
    fencemodel,iter = selection.get_selected()

    #validation and model setting for new fence device

    if(agent_val == 'fence_apc'):
      if (self.edit_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for fence device"))
        return

      if (self.edit_fencedevice_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for fence device"))
        return

      if (self.edit_fencedevice_user.get_text() == ""):
        self.errorMessage(_("Please specify User Name for fence device"))
        return

      if (self.edit_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for fence device"))
        return

      #Now add new fence into tree model
      fencemodel.set(iter,FD_DEVNAME_COL, self.edit_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.edit_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL,self.edit_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.edit_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_wti'):
      if (self.edit_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for fence device"))
        return

      if (self.edit_fencedevice_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for fence device"))
        return

      if (self.edit_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for fence device"))
        return

      #Now add new fence into tree model
      fencemodel.set(iter,FD_DEVNAME_COL, self.edit_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.edit_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL, "N/A",
                       FD_PASSWD_COL,self.edit_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_brocade'):
      if (self.edit_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for fence device"))
        return

      if (self.edit_fencedevice_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for fence device"))
        return

      if (self.edit_fencedevice_user.get_text() == ""):
        self.errorMessage(_("Please specify User Name for fence device"))
        return

      if (self.edit_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for fence device"))
        return

      #Now add new fence into tree model
      fencemodel.set(iter,FD_DEVNAME_COL, self.edit_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.edit_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL,self.edit_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.edit_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_vixel'):
      if (self.edit_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for fence device"))
        return

      if (self.edit_fencedevice_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for fence device"))
        return

      if (self.edit_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for fence device"))
        return

      #Now add new fence into tree model
      fencemodel.set(iter,FD_DEVNAME_COL, self.edit_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.edit_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL, "N/A",
                       FD_PASSWD_COL,self.edit_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_gnbd'):
      if (self.edit_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for fence device"))
        return

      if (self.edit_fencedevice_server.get_text() == ""):
        self.errorMessage(_("Please specify Server Address for fence device"))
        return

      #Now add new fence into tree model
      fencemodel.set(iter,FD_DEVNAME_COL, self.edit_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL, "N/A",
                       FD_LOGIN_COL, "N/A",
                       FD_PASSWD_COL,"N/A",
                       FD_SERVER_COL, self.edit_fencedevice_server.get_text(),
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_rib'):
      if (self.edit_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for fence device"))
        return

      if (self.edit_fencedevice_hostname.get_text() == ""):
        self.errorMessage(_("Please specify a hostname for fence device"))
        return

      if (self.edit_fencedevice_user.get_text() == ""):
        self.errorMessage(_("Please specify User Name for fence device"))
        return

      if (self.edit_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for fence device"))
        return

      #Now add new fence into tree model
      fencemodel.set(iter,FD_DEVNAME_COL, self.edit_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL, "N/A",
                       FD_LOGIN_COL,self.edit_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.edit_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, self.edit_fencedevice_hostname.get_text())

    if(agent_val == 'fence_xcat'):
      if (self.edit_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for fence device"))
        return

      #Now add new fence into tree model
      rpower_val = self.add_fencedevice_rpower.get_text()
      if(rpower_val == None):
        rpower_val = "" 
      fencemodel.set(iter,FD_DEVNAME_COL, self.edit_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.edit_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL,self.edit_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.edit_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, rpower_val,
                       FD_HOSTNAME_COL, "N/A")

    if(agent_val == 'fence_mcdata'):
      if (self.edit_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for fence device"))
        return

      if (self.edit_fencedevice_ipaddr.get_text() == ""):
        self.errorMessage(_("Please specify IP Address for fence device"))
        return

      if (self.edit_fencedevice_user.get_text() == ""):
        self.errorMessage(_("Please specify User Name for fence device"))
        return

      if (self.edit_fencedevice_passwd.get_text() == ""):
        self.errorMessage(_("Please specify password for fence device"))
        return

      #Now add new fence into tree model
      fencemodel.set(iter,FD_DEVNAME_COL, self.edit_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.edit_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL,self.edit_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.edit_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")

    if(agent_val == 'fence_egenera'):
      if (self.edit_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for fence device"))
        return

      if (self.edit_fencedevice_cserver.get_text() == ""):
        self.errorMessage(_("Please specify CServer for fence device"))
        return

      #Now add new fence into tree model
      fencemodel.set(iter,FD_DEVNAME_COL, self.edit_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL,self.edit_fencedevice_ipaddr.get_text(),
                       FD_LOGIN_COL,self.edit_fencedevice_user.get_text(),
                       FD_PASSWD_COL,self.edit_fencedevice_passwd.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, self.edit_fencedevice_cserver.get_text(),
                       FD_SERVER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    elif(agent_val == 'fence_manual'):
      if (self.edit_fencedevice_name.get_text() == ""):
        self.errorMessage(_("Please specify a name for fence device"))
        return

      #Now add new fence into tree model
      fencemodel.set(iter,FD_DEVNAME_COL, self.edit_fencedevice_name.get_text(),
                       FD_AGENTTYPE_COL, agent_val,
                       FD_IPADDR_COL, "N/A",
                       FD_LOGIN_COL, "N/A",
                       FD_PASSWD_COL, "N/A",
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")


    self.fenceview.expand_all() 

    #add fields to columns, then clear fields and hide dlg
    self.clear_edit_fencedevice_dlg()
    self.edit_fencedevice_dlg.hide()
    
  def on_fenceinstance_add_cancel(self, button):
    self.clear_add_fenceinstance_dlg()
    self.add_fenceinstance_dlg.hide()

  def on_fencedevice_add_cancel(self, button):
    self.clear_add_fencedevice_dlg()
    self.add_fencedevice_dlg.hide()

  def on_fencedevice_edit_cancel(self, button):
    self.clear_edit_fencedevice_dlg()
    self.edit_fencedevice_dlg.hide()

  def on_fenceinstance_edit_cancel(self, button):
    self.edit_fenceinstance_dlg.hide()

  def clear_add_fenceinstance_dlg(self):
    #Set all variable fields to off
    self.add_fenceinstance_port.set_text("")
    self.add_fenceinstance_port.set_sensitive(FALSE)
    self.add_fenceinstance_port_label.set_sensitive(FALSE)

    self.add_fenceinstance_ipaddr.set_text("")
    self.add_fenceinstance_ipaddr.set_sensitive(FALSE)
    self.add_fenceinstance_ipaddr_label.set_sensitive(FALSE)

    self.add_fenceinstance_switch.set_text("")
    self.add_fenceinstance_switch.set_sensitive(FALSE)
    self.add_fenceinstance_switch_label.set_sensitive(FALSE)

    self.add_fenceinstance_xcatnode.set_text("")
    self.add_fenceinstance_xcatnode.set_sensitive(FALSE)
    self.add_fenceinstance_xcatnode_label.set_sensitive(FALSE)

    self.add_fenceinstance_lpan.set_text("")
    self.add_fenceinstance_lpan.set_sensitive(FALSE)
    self.add_fenceinstance_lpan_label.set_sensitive(FALSE)

    self.add_fenceinstance_pserver.set_text("")
    self.add_fenceinstance_pserver.set_sensitive(FALSE)
    self.add_fenceinstance_pserver_label.set_sensitive(FALSE)

    self.add_fenceinstance_name.set_history(0) 
    self.add_fenceinstance_method_dynlabel.set_text("unknown")
    self.device_name_dict.clear()

  def clear_edit_fenceinstance_dlg(self):
    #Set all variable fields to off
    self.edit_fenceinstance_port.set_text("")
    self.edit_fenceinstance_port.set_sensitive(FALSE)
    self.edit_fenceinstance_port_label.set_sensitive(FALSE)

    self.edit_fenceinstance_ipaddr.set_text("")
    self.edit_fenceinstance_ipaddr.set_sensitive(FALSE)
    self.edit_fenceinstance_ipaddr_label.set_sensitive(FALSE)

    self.edit_fenceinstance_switch.set_text("")
    self.edit_fenceinstance_switch.set_sensitive(FALSE)
    self.edit_fenceinstance_switch_label.set_sensitive(FALSE)

    self.edit_fenceinstance_xcatnode.set_text("")
    self.edit_fenceinstance_xcatnode.set_sensitive(FALSE)
    self.edit_fenceinstance_xcatnode_label.set_sensitive(FALSE)

    self.edit_fenceinstance_lpan.set_text("")
    self.edit_fenceinstance_lpan.set_sensitive(FALSE)
    self.edit_fenceinstance_lpan_label.set_sensitive(FALSE)

    self.edit_fenceinstance_pserver.set_text("")
    self.edit_fenceinstance_pserver.set_sensitive(FALSE)
    self.edit_fenceinstance_pserver_label.set_sensitive(FALSE)

    self.edit_fenceinstance_name.set_history(0) 
    self.edit_fenceinstance_method_dynlabel.set_text("unknown")
    self.device_name_dict.clear()


  def clear_add_fencedevice_dlg(self):
    self.add_fencedevice_name.set_text("") 

    self.add_fencedevice_agent_type.set_history(0)

    self.add_fencedevice_ipaddr.set_text("")
    self.add_fencedevice_ipaddr.set_sensitive(FALSE)
    self.add_fencedevice_ipaddr_label.set_sensitive(FALSE)

    self.add_fencedevice_user.set_text("")
    self.add_fencedevice_user.set_sensitive(FALSE)
    self.add_fencedevice_user_label.set_sensitive(FALSE)

    self.add_fencedevice_passwd.set_text("")
    self.add_fencedevice_passwd.set_sensitive(FALSE)
    self.add_fencedevice_passwd_label.set_sensitive(FALSE)

    self.add_fencedevice_hostname.set_text("")
    self.add_fencedevice_hostname.set_sensitive(FALSE)
    self.add_fencedevice_hostname_label.set_sensitive(FALSE)

    self.add_fencedevice_server.set_text("")
    self.add_fencedevice_server.set_sensitive(FALSE)
    self.add_fencedevice_server_label.set_sensitive(FALSE)

    self.add_fencedevice_cserver.set_text("")
    self.add_fencedevice_cserver.set_sensitive(FALSE)
    self.add_fencedevice_cserver_label.set_sensitive(FALSE)

    self.add_fencedevice_rpower.set_text("")
    self.add_fencedevice_rpower.set_sensitive(FALSE)
    self.add_fencedevice_rpower_label.set_sensitive(FALSE)

  def clear_edit_fencedevice_dlg(self):
    self.edit_fencedevice_name.set_text("") 

    self.edit_fencedevice_agent_type.set_history(0)

    self.edit_fencedevice_ipaddr.set_text("")
    self.edit_fencedevice_ipaddr.set_sensitive(FALSE)
    self.edit_fencedevice_ipaddr_label.set_sensitive(FALSE)

    self.edit_fencedevice_user.set_text("")
    self.edit_fencedevice_user.set_sensitive(FALSE)
    self.edit_fencedevice_user_label.set_sensitive(FALSE)

    self.edit_fencedevice_passwd.set_text("")
    self.edit_fencedevice_passwd.set_sensitive(FALSE)
    self.edit_fencedevice_passwd_label.set_sensitive(FALSE)

    self.edit_fencedevice_hostname.set_text("")
    self.edit_fencedevice_hostname.set_sensitive(FALSE)
    self.edit_fencedevice_hostname_label.set_sensitive(FALSE)

    self.edit_fencedevice_server.set_text("")
    self.edit_fencedevice_server.set_sensitive(FALSE)
    self.edit_fencedevice_server_label.set_sensitive(FALSE)

    self.edit_fencedevice_cserver.set_text("")
    self.edit_fencedevice_cserver.set_sensitive(FALSE)
    self.edit_fencedevice_cserver_label.set_sensitive(FALSE)

    self.edit_fencedevice_rpower.set_text("")
    self.edit_fencedevice_rpower.set_sensitive(FALSE)
    self.edit_fencedevice_rpower_label.set_sensitive(FALSE)

  def on_member_add_ok(self, button):
    membermodel = self.memberview.get_model()
    #validation for new device
    if (self.add_member_hostname.get_text() == ""):
      self.errorMessage(_("Please specify Host Name for new member"))
      return

    if (self.add_member_netdevice.get_text() == ""):
      self.errorMessage(_("Please specify Network Device for new member"))
      return

    if (self.add_member_ipaddr.get_text() == ""):
      self.errorMessage(_("Please specify IP Address for new member"))
      return

    user_data = list()
    is_duplicate_name = FALSE
    is_duplicate_ipaddr = FALSE
    user_data.append(self.add_member_hostname.get_text())
    user_data.append(self.add_member_ipaddr.get_text())
    user_data.append(is_duplicate_name)
    user_data.append(is_duplicate_ipaddr)
    membermodel.foreach(self.check_duplicate_members, user_data)
    if(user_data[2]):
      self.errorMessage(_("A member with this hostname has already been specified. Please add a member with a unique hostname, or cancel."))
      return
    if(user_data[3]):
      self.errorMessage(_("A member with this IP Address has already been specified. Please add a member with a unique IP Address, or cancel."))
      return
    

    #Now insert new member into tree model
    iter = membermodel.append(None)
    membermodel.set(iter, MB_NAME_COL, self.add_member_hostname.get_text(),
                        MB_DEVNAME_COL, self.add_member_netdevice.get_text(),
                        MB_IPADDR_COL, self.add_member_ipaddr.get_text(),
                        MB_ISMEMBR_COL, TRUE)

    self.memberview.expand_all() 

    self.clear_add_member_dlg()
    self.add_member_dlg.hide()

  def check_duplicate_members(self, model, path, iter, user_data):
    is_member = model.get_value(iter, MB_ISMEMBR_COL)
    if(is_member):
      name_val = model.get_value(iter, MB_NAME_COL)
      ipaddr_val = model.get_value(iter, MB_IPADDR_COL)

      if(name_val == user_data[0]):
        user_data[2] = TRUE
      if(ipaddr_val == user_data[1]):
        user_data[3] = TRUE

  def on_editmember_addmember(self, button):
    #validation for new device
    if (self.editmember_hostname.get_text() == ""):
      self.errorMessage(_("Please specify Host Name for member"))
      return

    if (self.editmember_netdevice.get_text() == ""):
      self.errorMessage(_("Please specify Network Device for member"))
      return

    if (self.editmember_ipaddr.get_text() == ""):
      self.errorMessage(_("Please specify IP Address for member"))
      return

    #Now insert new member into tree model
    selection = self.memberview.get_selection()
    model,iter = selection.get_selected()
    model.set(iter, MB_NAME_COL, self.editmember_hostname.get_text(),
                        MB_DEVNAME_COL, self.editmember_netdevice.get_text(),
                        MB_IPADDR_COL, self.editmember_ipaddr.get_text(),
                        MB_ISMEMBR_COL, TRUE)

    self.memberview.expand_all() 

    self.edit_member_dlg.hide()

  def preload_member_dlg(self):
    #Find out what entry we are editing
    selection = self.memberview.get_selection()
    model,iter = selection.get_selected()

    self.editmember_hostname.set_text(model.get_value(iter, MB_NAME_COL))
    self.editmember_netdevice.set_text(model.get_value(iter, MB_DEVNAME_COL))
    self.editmember_ipaddr.set_text(model.get_value(iter, MB_IPADDR_COL))


  def preload_fenceinstance_add_dlg(self):
    #Find out what entry we are editing
    selection = self.memberview.get_selection()
    model,iter = selection.get_selected()

    try:
      opt_idx = FENCE_OPTS.index(model.get_value(iter, MB_AGENTTYPE_COL))
    except RuntimeError, e:
      print "Problem with menu option index"
      opt_idx = 0

    #self.edit_fenceinstance_ipaddr.set_text(model.get_value(iter,MB_IPADDR_COL))
    #self.edit_fenceinstance_port.set_text(model.get_value(iter,MB_PORT_COL))
    #self.edit_fenceinstance_ipaddr.set_text(model.get_value(iter,MB_IPADDR_COL))
    #self.edit_fenceinstance_switch.set_text(model.get_value(iter,MB_SWITCH_COL))
    self.edit_fenceinstance_name.set_history(opt_idx) 

  def preload_edit_fencedevice_props_dlg(self):
    #Find out what entry we are editing
    selection = self.fenceview.get_selection()
    model,iter = selection.get_selected()

    try:
      opt_idx = FENCE_OPTS.index(model.get_value(iter, FD_AGENTTYPE_COL))
    except RuntimeError, e:
      print "Problem with menu option index"
      opt_idx = 0

    self.editfence_device_name.set_text(model.get_value(iter,FD_DEVNAME_COL)) 
    self.editfence_ipaddr.set_text(model.get_value(iter,FD_IPADDR_COL))
    self.editfence_agent_type.set_history(opt_idx)
    self.editfence_device_server.set_text(model.get_value(iter,FD_SERVER_COL))
    self.editfence_device_hostname.set_text(model.get_value(iter,FD_HOSTNAME_COL))
    self.editfence_user.set_text(model.get_value(iter,FD_LOGIN_COL))
    self.editfence_passwd.set_text(model.get_value(iter,FD_PASSWD_COL))
    self.editfence_cserver.set_text(model.get_value(iter,FD_PASSWD_COL))
    self.editfence_rpower.set_text(model.get_value(iter,FD_PASSWD_COL))

  def on_edit_fencedevice_props(self, *args):
    preload_edit_fencedevice_props_dlg()

  def on_member_fenceinstance_deleteentry(self, *args):
    selection = self.memberview.get_selection()
    model,iter = selection.get_selected()

    path = model.get_path(iter)

    #If iter, then see if it is a member or fence
    if(iter):
      val = model.get_value(iter,MB_ISMEMBR_COL)
    else:
      self.errorMessage(_("Please select a member or fence from the list"))
      return

    #If selection is a member, delete it and all child nodes
    if(val):
      rc = self.confirmDelete(CONFIRM_MEMBER_DELETE)
      if (rc == gtk.RESPONSE_NO):
        return
      while(model.iter_has_child(iter)):
        child_iter = model.iter_children(iter)
        model.remove(child_iter) 

      #Now all children are gone, so remove parent
      model.remove(iter)

    else:  #a fence device (child) was selected
      rc = self.confirmDelete(CONFIRM_FENCE_DELETE)
      if (rc == gtk.RESPONSE_NO):
        return
      model.remove(iter)

    #Finally, select next item (or previous if deleted item was last)
    selection.select_path(path)

    if not selection.path_is_selected(path):
      row = path[0]-1
      #test for empty list
      if row >= 0:
        selection.select_path((row,))  


  def on_fencedevice_deleteentry(self, *args):
    selection = self.fenceview.get_selection()
    model,iter = selection.get_selected()

    #Check that no instances depend on this device
    is_used = FALSE
    fence_name = model.get_value(iter, FD_DEVNAME_COL)
    listvals = list()
    listvals.append(is_used)
    listvals.append(fence_name)
    membermodel = self.memberview.get_model()
    membermodel.foreach(self.check_instances, listvals)
    if(listvals[0] == TRUE):
      self.errorMessage(ASSOCIATED_FENCE_INSTANCES1 + fence_name + ASSOCIATED_FENCE_INSTANCES2)
      return

    path = model.get_path(iter)

    if(iter):
      rc = self.confirmDelete(CONFIRM_FENCE_DELETE)
      if (rc == gtk.RESPONSE_NO):
        return
      model.remove(iter)

      #Finally, select next item (or previous if deleted item was last)
      selection.select_path(path)

      if not selection.path_is_selected(path):
        row = path[0]-1
      #test for empty list
        if row >= 0:
          selection.select_path((row,))  

  def check_instances(self, model, path, iter, listvals):
    name = model.get_value(iter, MB_DEVNAME_COL) 
    if(name == listvals[1]):
      listvals[0] = TRUE

  def on_member_cancelmember(self, button):
    self.clear_add_member_dlg()
    self.add_member_dlg.hide()

  def on_editmember_cancelmember(self, button):
    self.edit_member_dlg.hide()

  def clear_add_member_dlg(self):
    self.add_member_hostname.set_text("")
    self.add_member_netdevice.set_text("eth0")
    self.add_member_ipaddr.set_text("")

  def on_member_selection_changed(self, *args):
    selection = self.memberview.get_selection()
    model,iter = selection.get_selected()

    if(iter):
      val = model.get_value(iter,MB_ISMEMBR_COL)

      if(val):
        self.editprops.set_sensitive(TRUE)
        self.deleteentry.set_sensitive(TRUE)
        self.add_fenceinstance_button.set_sensitive(TRUE)

      else:
        self.add_fenceinstance_button.set_sensitive(FALSE)

    else:
      self.editprops.set_sensitive(FALSE)
      self.deleteentry.set_sensitive(FALSE)
      self.add_fenceinstance_button.set_sensitive(FALSE)


  def on_fence_selection_changed(self, *args):
    selection = self.fenceview.get_selection()
    model, iter = selection.get_selected()

    if(iter):
      self.edit_fencedevice_props.set_sensitive(TRUE)
      self.fencedevice_deleteentry.set_sensitive(TRUE)

  ###############################################
  ## The do_member_and_fence_list builds a representation
  ## of the cluster members in cluster.xml and 
  ## any fence devices associated with the member 
  ## nodes. First, a list of members is obtained.
  ## Then, a list of powercontrollers is
  ## obtained for each member.
 
  def do_member_and_fence_list(self):
    clu = cluconfig(node=None, lock=FALSE)
    members = clu.getmembers()
    members_list = members.get_list()

    membermodel = self.memberview.get_model()
    membermodel.clear()

    fencemodel = self.fenceview.get_model()
    fencemodel.clear()

    for member in members_list:
      #1) iterate through members list
      #2) for each member, append new row
      #3) append rows for each fence device
      parent_iter = membermodel.append(None)
      name = member.get("name")

      #Scary stuff -- GFS needs an actual IP address for every
      #member node, but cluster.xml allows hostnames or ip addresses.
      #So I am using my least favorite socket module call to try and
      #resolve this
      ###FIXME - error check the ipaddrs return list
      host, aliaslist, ipaddrs = socket.gethostbyname_ex(name)
      addr = ipaddrs[0]
      if(addr == ""):
        addr = "127.0.0.1"
      membermodel.set(parent_iter, MB_NAME_COL, name, 
                                   MB_DEVNAME_COL, "eth0",
                                   MB_IPADDR_COL, addr, 
                                   MB_ISMEMBR_COL, TRUE)

      pcs = member.get_powercontrollers()
      if pcs != None:
        for pc in pcs:
          fence_type = self.check_fence(pc.get("type"))
          if(fence_type == "gulm"):
            continue
          if(fence_type == "fence_baytech"):  #unsupported right now
            continue
          if(fence_type == "unknown"):
            continue
          child_iter = membermodel.insert(parent_iter, 999, None)
          membermodel.set(child_iter, MB_DEVNAME_COL, pc.get("type"),
                                    MB_IPADDR_COL, "N/A",
                                    MB_PORT_COL, pc.get("port"),
                                    MB_METHOD_COL, "power",
                                    MB_AGENTTYPE_COL, fence_type,
                                    MB_SWITCH_COL, "N/A",
                                    MB_XCATNODE_COL, "N/A",
                                    MB_LPAN_COL, "N/A",
                                    MB_PSERVER_COL, "N/A",
                                    MB_ISMEMBR_COL, FALSE)

          ##Check for duplicate entries before adding
          duplicate = list()
          found_duplicate = FALSE
          duplicate.append(pc.get("type"))
          duplicate.append(found_duplicate) 
          fencemodel.foreach(self.check_fence_duplicates, duplicate)
          if(duplicate[1] == FALSE): #no other entry is present
            device_iter = fencemodel.append(None)  #For other list
            if(fence_type == "fence_wti"):
              fencemodel.set(device_iter, FD_DEVNAME_COL, pc.get("type"),
                                      FD_AGENTTYPE_COL, fence_type,
                                      FD_IPADDR_COL, pc.get("ipaddress"),
                                      FD_LOGIN_COL, "N/A",
                                      FD_PASSWD_COL, pc.get("password"),
                                      FD_SERVER_COL, "N/A",
                                      FD_HOSTNAME_COL, "N/A",
                                      FD_CSERVER_COL, "N/A",
                                      FD_RPOWER_COL, "N/A")
            else:
              fencemodel.set(device_iter, FD_DEVNAME_COL, pc.get("type"),
                                      FD_AGENTTYPE_COL, fence_type,
                                      FD_IPADDR_COL, pc.get("ipaddress"),
                                      FD_LOGIN_COL, pc.get("user"),
                                      FD_PASSWD_COL, pc.get("password"),
                                      FD_SERVER_COL, "N/A",
                                      FD_HOSTNAME_COL, "N/A",
                                      FD_CSERVER_COL, "N/A",
                                      FD_RPOWER_COL, "N/A")

    self.memberview.expand_all() 

  def check_fence_duplicates(self, model, path, iter, duplicate):
    name = duplicate[0]
    stored_name = model.get_value(iter, FD_DEVNAME_COL)
    if(stored_name == name):
      duplicate[1] = TRUE
      
  def on_forward_standard1(self, *args):
    if(self.clustername_entry.get_text() == ""):
      self.errorMessage(_("Please provide a name for this cluster"))
      return TRUE

  def on_forward_standard2(self, *args):
    if(self.allowedmisses_entry.get_text() == ""):
      self.errorMessage(_("Please provide a heartbeat rate"))
      return TRUE

    if(self.heartbeatrate_entry.get_text() == ""):
      self.errorMessage(_("Please provide a location for files this UI writes out."))
      return TRUE

  def on_forward_standard3(self, *args):
    if(self.filelocation_entry.get_text == ""):
      self.errorMessage(_("Please provide a location for files this UI writes out."))
      return TRUE

  ###########################################################
  ###########################################################
  #This method performs some final validation, then write out
  #the three files necessary for GFS. 

  def on_finish(self, *args):
    
    #Make certain that chosen directory can be stat'd
    file_location = self.filelocation_entry.get_text()
    try:
      mode = os.stat(file_location)[stat.ST_MODE]
    except OSError, e:
      emessage = NO_DIRECTORY_FOUND_MESSAGE1 + self.filelocation_entry.get_text() + NO_DIRECTORY_FOUND_MESSAGE2
      self.errorMessage(emessage)
      return

    #Get both models needed...
    member_model = self.memberview.get_model()
    fence_model = self.fenceview.get_model()

    #Check that each member has at least one fence instance associated with it
    missing_fences = self.verify_fences()

    if(missing_fences):
      #Offer user the chance to fix missing fences themself
      rc = self.warningMessage(FENCE_WARNING_MESSAGE)
      if (rc == gtk.RESPONSE_NO):
        return

    self.check_members_for_fences()

    try:
      cluster_ccs = open(file_location + "/cluster.ccs",'w+', 1)
    except OSError, e:
      emessage = CANT_OPEN_FILE1 + file_location + "/cluster.ccs" + CANT_OPEN_FILE2 
      self.errorMessage(emessage)
    try: 
      cluster_ccs.write("cluster {\n")
      cluster_ccs.write("  name = \"" + self.clustername_entry.get_text() + "\"\n")
      ## Here we need to run through the member model twice: 
      ## First to check if any member is a lock server, then to write it
      lock = list()
      lock_present = FALSE
      lock.append(lock_present)
      member_model.foreach(self.check_lockserver, lock)
      if(lock[0]): #This means at least one member is to be a lock server
        self.first_locksrv_flag = TRUE
        cluster_ccs.write("  lock_gulm {\n")
        cluster_ccs.write("    servers = [")
        member_model.foreach(self.write_lockserver, cluster_ccs)
        cluster_ccs.write("]\n")
        hb_str = self.heartbeatrate_entry.get_text()
        if(hb_str.find(".") >= 0):
          cluster_ccs.write("    heartbeat_rate = " + self.heartbeatrate_entry.get_text() + "\n")
        else: 
          cluster_ccs.write("    heartbeat_rate = " + self.heartbeatrate_entry.get_text() + ".0\n")   #gfs file parser needs float value here
        cluster_ccs.write("    allowed_misses = " + self.allowedmisses_entry.get_text() + "\n")
        cluster_ccs.write("  }\n")
      cluster_ccs.write("}\n")
      cluster_ccs.close()
    except OSError, e:
      emessage = CANT_WRITE_FILE1 + file_location + "/cluster.ccs" + CANT_WRITE_FILE2 
      self.errorMessage(emessage)
    
    ################
    #OK, time for the next file, nodes.ccs 
    try:
      nodes_ccs = open(file_location + "/nodes.ccs",'w+', 1)
    except OSError, e:
      emessage = CANT_OPEN_FILE1 + file_location + "/nodes.ccs" + CANT_OPEN_FILE2 
      self.errorMessage(emessage)
    try: 
      nodes_ccs.write("nodes {\n")
      member_model.foreach(self.writeout_nodes, nodes_ccs)
      if(self.just_wrote_fence == TRUE):
        nodes_ccs.write("      }\n") #close last method
        nodes_ccs.write("    }\n") #close last fence
        nodes_ccs.write("  }\n") #close last node 
      nodes_ccs.write("}\n")
      nodes_ccs.close()
    except OSError, e:
      emessage = CANT_WRITE_FILE1 + file_location + "/cluster.ccs" + CANT_WRITE_FILE2 
#      self.errorMessage(emessage)
     
    ##################
    #Finally, write out fence.ccs
    try:
      fence_ccs = open(file_location + "/fence.ccs",'w+', 1)
    except OSError, e:
      emessage = CANT_OPEN_FILE1 + file_location + "/fence.ccs" + CANT_OPEN_FILE2 
      self.errorMessage(emessage)
      return

    fence_ccs.write("fence_devices {\n")
    fence_model.foreach(self.writeout_fences, fence_ccs)
    fence_ccs.write("}\n")
    fence_ccs.close()

    #Since we are all done, quit app...
    rc = self.infoMessage(QUIT_MESSAGE)
    if (rc == gtk.RESPONSE_YES):
      gtk.mainquit()
      sys.exit(0)
  
  ############################################################
  ##Loop callback that checks for the presence of lock servers 
  def check_lockserver(self, model, path, iter, lock):
    locksrv = model.get_value(iter, MB_LOCKSRV_COL)
    if(locksrv): 
      lock[0] = locksrv

  ############################################################
  ##Loop callback that writes out lock servers 
  def write_lockserver(self, model, path, iter, cluster_ccs):
    locksrv = model.get_value(iter, MB_LOCKSRV_COL)
    if(locksrv):
      name = model.get_value(iter, MB_NAME_COL)
      if(self.first_locksrv_flag):
        self.first_locksrv_flag = FALSE
        cluster_ccs.write("\"" + name + "\"")
      else:
        cluster_ccs.write(", \"" + name + "\"")
    

  #######################################################
  ##Loop callback that fills in nodes.ccs
  def writeout_nodes(self, model, path, iter, nodes_ccs):
    #1) Check if member or fence
    #2) If member, write out node info and set is_first_node false

    locksrv = model.get_value(iter, MB_LOCKSRV_COL)
    name = model.get_value(iter, MB_NAME_COL)
    devname = model.get_value(iter, MB_DEVNAME_COL)
    ipaddr = model.get_value(iter, MB_IPADDR_COL)
    port = model.get_value(iter, MB_PORT_COL)
    method = model.get_value(iter, MB_METHOD_COL)
    switch = model.get_value(iter, MB_SWITCH_COL)
    if(switch == "N/A"):
      switch = "1"
    xcatnode = model.get_value(iter, MB_XCATNODE_COL)
    lpan = model.get_value(iter, MB_LPAN_COL)
    pserver = model.get_value(iter, MB_PSERVER_COL)
    agenttype = model.get_value(iter, MB_AGENTTYPE_COL)
    ismember = model.get_value(iter, MB_ISMEMBR_COL)

    if(ismember):
      if(self.is_first_node):
        self.is_first_node = FALSE
      else:
        nodes_ccs.write("      }\n") #This finishes off last method section
        nodes_ccs.write("    }\n") #This finishes off last fence section
        nodes_ccs.write("  }\n") #This finishes off last node section
      self.is_first_fence = TRUE
      self.just_wrote_fence = FALSE
      self.last_fencemethod_written = ""
      nodes_ccs.write("  " + name + " {\n")
      nodes_ccs.write("    ip_interfaces {\n")
      nodes_ccs.write("      " + devname + " = \"" + ipaddr + "\"\n")
      nodes_ccs.write("    }\n")
    else:
      self.just_wrote_fence = TRUE
      if(self.is_first_fence):
        nodes_ccs.write("    fence {\n") #start new fence section
        nodes_ccs.write("      " + method + " {\n")
        self.is_first_fence = FALSE
        self.last_fencemethod_written = method
      else:
        if(method != self.last_fencemethod_written):
          self.last_fencemethod_written = method
          nodes_ccs.write("      }\n") #close last method
          nodes_ccs.write("      " + method + " {\n") #start new method
 
      if(agenttype == "fence_apc"):
        nodes_ccs.write("        " + devname + " {\n")
        nodes_ccs.write("          port = " + port + "\n")
        if(switch != ""):
          nodes_ccs.write("          switch = " + switch + "\n")
        nodes_ccs.write("        }\n") #end of fence device

      elif(agenttype == "fence_wti"):
        nodes_ccs.write("        " + devname + " {\n")
        nodes_ccs.write("          port = " + port + "\n")
        nodes_ccs.write("        }\n") #end of fence device

      elif(agenttype == "fence_brocade"):
        nodes_ccs.write("        " + devname + " {\n")
        nodes_ccs.write("          port = " + port + "\n")
        nodes_ccs.write("        }\n") #end of fence device

      elif(agenttype == "fence_vixel"):
        nodes_ccs.write("        " + devname + " {\n")
        nodes_ccs.write("          port = " + port + "\n")
        nodes_ccs.write("        }\n") #end of fence device

      elif(agenttype == "fence_gnbd"):
        nodes_ccs.write("        " + devname + " {\n")
        nodes_ccs.write("        ipaddr = \"" + ipaddr + "\"\n")
        nodes_ccs.write("        }\n") #end of fence device

      elif(agenttype == "fence_rib"):
        nodes_ccs.write("        " + devname + " {\n")
        nodes_ccs.write("          localport = " + port + "\n")
        nodes_ccs.write("        }\n") #end of fence device

      elif(agenttype == "fence_xcat"):
        nodes_ccs.write("        " + devname + " {\n")
        nodes_ccs.write("          nodename = \"" + xcatnode + "\"\n")
        nodes_ccs.write("        }\n") #end of fence device

      elif(agenttype == "fence_mcdata"):
        nodes_ccs.write("        " + devname + " {\n")
        nodes_ccs.write("          port = " + port + "\n")
        nodes_ccs.write("        }\n") #end of fence device

      elif(agenttype == "fence_egenera"):
        nodes_ccs.write("        " + devname + " {\n")
        nodes_ccs.write("          lpan = \"" + lpan + "\"\n")
        nodes_ccs.write("          pserver = \"" + pserver + "\"\n")
        nodes_ccs.write("        }\n") #end of fence device

      elif(agenttype == "fence_manual"):
        parent = model.iter_parent(iter)
        parent_name = model.get_value(parent, MB_NAME_COL)
        nodes_ccs.write("        " + devname + " {\n")
        nodes_ccs.write("        ipaddr = \"" + parent_name + "\"\n")
        #nodes_ccs.write(" manual    }\n") #end of fence device
        nodes_ccs.write("        }\n") #end of fence device

 
  def writeout_fences(self, model, path, iter, fence_ccs):
    devname = model.get_value(iter, FD_DEVNAME_COL)
    agenttype = model.get_value(iter, FD_AGENTTYPE_COL)
    ipaddr = model.get_value(iter, FD_IPADDR_COL)
    login = model.get_value(iter, FD_LOGIN_COL)
    passwd = model.get_value(iter, FD_PASSWD_COL)
    server = model.get_value(iter, FD_SERVER_COL)
    cserver = model.get_value(iter, FD_CSERVER_COL)
    rpower = model.get_value(iter, FD_RPOWER_COL)
    hostname = model.get_value(iter, FD_HOSTNAME_COL)

    if(agenttype == "fence_apc"):
      fence_ccs.write("  " + devname + " {\n")
      fence_ccs.write("    agent = \"" + agenttype + "\"\n")
      fence_ccs.write("    ipaddr = \"" + ipaddr + "\"\n")
      fence_ccs.write("    login = \"" + login + "\"\n")
      fence_ccs.write("    passwd = \"" + passwd + "\"\n")
      fence_ccs.write("  }\n")

    elif(agenttype == "fence_wti"):
      fence_ccs.write("  " + devname + " {\n")
      fence_ccs.write("    agent = \"" + agenttype + "\"\n")
      fence_ccs.write("    ipaddr = \"" + ipaddr + "\"\n")
      fence_ccs.write("    passwd = \"" + passwd + "\"\n")
      fence_ccs.write("  }\n")

    elif(agenttype == "fence_brocade"):
      fence_ccs.write("  " + devname + " {\n")
      fence_ccs.write("    agent = \"" + agenttype + "\"\n")
      fence_ccs.write("    ipaddr = \"" + ipaddr + "\"\n")
      fence_ccs.write("    login = \"" + login + "\"\n")
      fence_ccs.write("    passwd = \"" + passwd + "\"\n")
      fence_ccs.write("  }\n")

    elif(agenttype == "fence_vixel"):
      fence_ccs.write("  " + devname + " {\n")
      fence_ccs.write("    agent = \"" + agenttype + "\"\n")
      fence_ccs.write("    ipaddr = \"" + ipaddr + "\"\n")
      fence_ccs.write("    passwd = \"" + passwd + "\"\n")
      fence_ccs.write("  }\n")

    elif(agenttype == "fence_gnbd"):
      fence_ccs.write("  " + devname + " {\n")
      fence_ccs.write("    agent = \"" + agenttype + "\"\n")
      fence_ccs.write("    server = \"" + server + "\"\n")
      fence_ccs.write("  }\n")

    elif(agenttype == "fence_rib"):
      fence_ccs.write("  " + devname + " {\n")
      fence_ccs.write("    agent = \"" + agenttype + "\"\n")
      fence_ccs.write("    hostname = \"" + hostname + "\"\n")
      fence_ccs.write("    login = \"" + login + "\"\n")
      fence_ccs.write("    passwd = \"" + passwd + "\"\n")
      fence_ccs.write("  }\n")

    elif(agenttype == "fence_xcat"):
      fence_ccs.write("  " + devname + " {\n")
      fence_ccs.write("    agent = \"" + agenttype + "\"\n")
      fence_ccs.write("    rpower = \"" + rpower + "\"\n")
      fence_ccs.write("  }\n")

    elif(agenttype == "fence_mcdata"):
      fence_ccs.write("  " + devname + " {\n")
      fence_ccs.write("    agent = \"" + agenttype + "\"\n")
      fence_ccs.write("    ipaddr = \"" + ipaddr + "\"\n")
      fence_ccs.write("    login = \"" + login + "\"\n")
      fence_ccs.write("    passwd = \"" + passwd + "\"\n")
      fence_ccs.write("  }\n")

    elif(agenttype == "fence_egenera"):
      fence_ccs.write("  " + devname + " {\n")
      fence_ccs.write("    agent = \"" + agenttype + "\"\n")
      fence_ccs.write("    cserver = \"" + cserver + "\"\n")
      fence_ccs.write("  }\n")

    elif(agenttype == "fence_manual"):
      fence_ccs.write("  " + devname + " {\n")
      fence_ccs.write("    agent = \"" + agenttype + "\"\n")
      fence_ccs.write("  }\n")

  def verify_fences(self):
    missing = list()
    no_fence = FALSE
    missing.append(no_fence)
    member_model = self.memberview.get_model()
    member_model.foreach(self.verify_fences_loop, missing)

    if(missing[0] == FALSE):
      return FALSE
    else:
      return TRUE

  def verify_fences_loop(self, model, path, iter, missing):
    if(model.get_value(iter, MB_ISMEMBR_COL)):
        #check for children
        if(not model.iter_has_child(iter)):
          missing[0] = TRUE
      

  def check_members_for_fences(self):
    pathes = list()
    member_model = self.memberview.get_model()
    fencemodel = self.fenceview.get_model()

    #check for missing fences
    #pathes to all members w/o a fence device are returned
    #in the pathes list.
    member_model.foreach(self.fence_existence, pathes)

    if(len(pathes) > 0):
      #add fence device first
      newiter = fencemodel.append(None)
      fencemodel.set(newiter,FD_DEVNAME_COL, "admm1",
                       FD_AGENTTYPE_COL, "fence_manual",
                       FD_IPADDR_COL, "N/A",
                       FD_LOGIN_COL, "N/A",
                       FD_PASSWD_COL, "N/A",
                       FD_SERVER_COL, "N/A",
                       FD_CSERVER_COL, "N/A",
                       FD_RPOWER_COL, "N/A",
                       FD_HOSTNAME_COL, "N/A")
                                                                                
                                                                                
      self.fenceview.expand_all()

      for path in pathes:
        iter = member_model.get_iter(path)
        child_iter = member_model.insert(iter,999,None) #999 for last one
        member_model.set(child_iter,
                MB_DEVNAME_COL, "admm1",
                MB_IPADDR_COL, "N/A",
                MB_METHOD_COL, "human",
                MB_PORT_COL, "N/A",
                MB_SWITCH_COL, "N/A",
                MB_XCATNODE_COL, "N/A",
                MB_LPAN_COL, "N/A",
                MB_PSERVER_COL, "N/A",
                MB_AGENTTYPE_COL, "fence_manual",
                MB_ISMEMBR_COL, FALSE)
                                                                                
      self.memberview.expand_all()

 
  def fence_existence(self, model, path, iter, pathes):
    if(model.get_value(iter, MB_ISMEMBR_COL)):
        #check for children
        if(not model.iter_has_child(iter)):
          pathes.append(path)
      

  def cancel(self, button):
    gtk.mainquit()

  def check_fence(self, utypename):
    typename = utypename.lower()
    #2) check for substrings apc, wti, gulm, baytech
    if(typename.find("apc") >= 0):
      return "fence_apc"
    elif(typename.find("wti") >= 0):
      return "fence_wti"
    elif(typename.find("gulm") >= 0):
      return "gulm"
    elif(typename.find("bay") >= 0):
      return "fence_baytech"
    else:
      return "unknown"
    
  def on_collapse_row(self, treeview, iter, path):
    print "Sorry, you can't do that"
    return FALSE
  
  def on_toggle_checkbox(self, renderer, path):
    membermodel = self.memberview.get_model()
    iter = membermodel.get_iter(path)
    
    if(iter):
      val = membermodel.get_value(iter,MB_LOCKSRV_COL)
      if(val):
        membermodel.set(iter, MB_LOCKSRV_COL, FALSE)
      else:
        membermodel.set(iter, MB_LOCKSRV_COL, TRUE)

  def errorMessage(self, message):
    dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK,
                            message)
    dlg.show_all()
    rc = dlg.run()
    dlg.destroy()
    return rc

  def infoMessage(self, message):
    dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO,
                            message)
    dlg.show_all()
    rc = dlg.run()
    dlg.destroy()
    return rc

  def warningMessage(self, message):
    dlg = gtk.MessageDialog(None, 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO,
                            message)
    dlg.show_all()
    rc = dlg.run()
    dlg.destroy()
    return rc

  def confirmDelete(self, message):
    dlg = gtk.MessageDialog(None,0, gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO, message)

    dlg.show_all()
    rc = dlg.run()
    dlg.destroy()
    return rc

###################################################
def initGlade():
    gladepath = "gfsdruid.glade"
    if not os.path.exists(gladepath):
        gladepath = "%s/%s" % (INSTALLDIR, gladepath)
                                                                                
    gtk.glade.bindtextdomain(PROGNAME)
    glade_xml = gtk.glade.XML (gladepath, domain=PROGNAME)
    return glade_xml
                                                                                
def runFullGUI():
    glade_xml = initGlade()
    g = gfsconfig(glade_xml)
    app = glade_xml.get_widget('window1')
    app.show_all()
    app.connect("destroy", lambda w: gtk.mainquit())
    gtk.mainloop()
                                                                                
                                                                                
if __name__ == "__main__":
    runFullGUI()

